INCLUDE(CMakeForceCompiler)

IF(NOT CMAKE_TOOLCHAIN_FILE)
	MESSAGE(FATAL_ERROR "CMAKE_TOOLCHAIN_FILE not defined")
ENDIF()

SET(CAT_DIR "CAT_DIR_UNUSED")

SET(CMAKE_EXECUTABLE_SUFFIX "$CACHE{CMAKE_EXECUTABLE_SUFFIX}")

IF(${CMAKE_BUILD_TYPE} MATCHES "Release")
	SET(CAFE_BUILD_TYPE "NDEBUG")
ELSE()
	SET(CAFE_BUILD_TYPE "DEBUG")
ENDIF()

GET_FILENAME_COMPONENT(
	tmp
	"${NLIB_RELEASE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}"
	ABSOLUTE)
nlib_cygwin_getwinpath(${tmp} NLIB_DIR)
MESSAGE(STATUS "CAFE_BUILD_TYPE=${CAFE_BUILD_TYPE}")

SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Debug>:DEBUG;_DEBUG>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Release>:>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_HAS_ZLIB")
SET(ZLIB_FOUND 1)
ADD_LIBRARY(ZLIB::ZLIB STATIC IMPORTED)
SET(ZLIB_INCLUDE_DIRS "${CAFE_ROOT}/system/include/")
SET(ZLIB_LIBRARIES "${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/zlib125.a")
SET_PROPERTY(TARGET ZLIB::ZLIB PROPERTY INTERFACE_INCLUDE_DIRECTORIES "${ZLIB_INCLUDE_DIRS}")
SET_PROPERTY(TARGET ZLIB::ZLIB PROPERTY IMPORTED_LOCATION "${ZLIB_LIBRARIES}")
SET(ZLIB_MAJOR_VERSION 1)
SET(ZLIB_MINOR_VERSION 2)
SET(ZLIB_PATCH_VERSION 5)

FUNCTION(library_default_build_rule target)
	ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE INTERFACE "${NLIB_PKGROOT_DIR}/include")
	INSTALL(TARGETS ${target} DESTINATION system/lib/ghs/cafe/${CAFE_BUILD_TYPE})
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDFUNCTION()

# sdkpad -> -lsdkpad -> ${sdklibdir}/pad.a --> linker
FUNCTION(target_link_libraries_default target)
	TARGET_LINK_LIBRARIES(${target}
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/pad.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/gfd.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/mtx.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/demo.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/gx2ut.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nn_util.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nn_crypto.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nsyshid.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nlibcurl.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nsysnet.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nsysccr.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nsysuvd.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/gx2.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/avm.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/tcl.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/tve.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/dc.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/snd_core.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/snd_user.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/uvc.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/uvd.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/camera.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/h264.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/dmae.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nn_acp.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nn_save.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/vpad.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/vpadbase.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/proc_ui.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/nn_ac.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/padscore.a
		${CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}/coredyn.a
	)
ENDFUNCTION()
