
IF(DEFINED NLIB_ONCE_COMMON_CMAKE)
  RETURN()
ENDIF()
SET(NLIB_ONCE_COMMON_CMAKE True)

IF(${CMAKE_BUILD_TYPE} MATCHES "^$")
	SET(CMAKE_BUILD_TYPE "Release")
ENDIF()

CMAKE_POLICY(SET CMP0022 NEW)
CMAKE_POLICY(SET CMP0038 NEW)
CMAKE_POLICY(SET CMP0042 NEW)
CMAKE_POLICY(SET CMP0054 NEW)
SET(CMAKE_WARN_DEPRECATED TRUE)

IF(NOT NLIB_PKGROOT_DIR)
	SET(NLIB_PKGROOT_DIR ${NLIB_RELEASE_ROOT})
ENDIF()

SET(CMAKE_EXPORT_COMPILE_COMMANDS ON)
LIST(APPEND CMAKE_PREFIX_PATH ${CMAKE_CURRENT_LIST_DIR})
LIST(APPEND CMAKE_MODULE_PATH ${CMAKE_CURRENT_LIST_DIR})

IF(DEFINED ENV{NLIB_CFUNC_PREFIX})
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_RENAME_CAPI;NLIB_CFUNC_PREFIX=$ENV{NLIB_CFUNC_PREFIX}")
ENDIF()

IF(DEFINED ENV{NLIB_NAMESPACE_BEGIN} AND
   DEFINED ENV{NLIB_NAMESPACE_END} AND
   DEFINED ENV{NLIB_NS})
   # ex. export NLIB_NAMESPACE_BEGIN="namespace ci {"
   # ex. export NLIB_NAMESPACE_END="}"
   # ex. export NLIB_NS="::ci"
   IF(CYGWIN AND (CAFE OR CTR OR NX))
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_BEGIN=\"$ENV{NLIB_NAMESPACE_BEGIN}\"")
   	   SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_END=\"$ENV{NLIB_NAMESPACE_END}\"")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NS=\"$ENV{NLIB_NS}\"")
   ELSE()
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_BEGIN=$ENV{NLIB_NAMESPACE_BEGIN}")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NAMESPACE_END=$ENV{NLIB_NAMESPACE_END}")
       SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NS=$ENV{NLIB_NS}")
   ENDIF()
ENDIF()

# environment variable defined by cibuild like jenkins
IF(DEFINED ENV{NLIB_CIBUILD})
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_CIBUILD")
ENDIF()

IF(DEFINED ENV{WERROR} AND NOT MSVC_VERSION)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Werror")
ENDIF()

IF(CYGWIN OR ${CMAKE_HOST_SYSTEM_NAME} MATCHES Cygwin)
	FIND_PROGRAM(CYGPATH cygpath)
	IF(NOT CYGPATH)
		MESSAGE(FATAL_ERROR "!!!!! cannot find cygpath !!!!!")
	ENDIF()
	FUNCTION(nlib_cygwin_getcygpath _from _to)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -u "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		GET_FILENAME_COMPONENT(_toans ${_toans} ABSOLUTE)
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()

	FUNCTION(nlib_cygwin_getwinpath _from _to)
		GET_FILENAME_COMPONENT(_from ${_from} ABSOLUTE)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -m "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()
ELSE()
	FUNCTION(nlib_cygwin_getcygpath _from _to)
		SET(${_to} ${_from} PARENT_SCOPE)
	ENDFUNCTION()
	FUNCTION(nlib_cygwin_getwinpath _from _to)
		SET(${_to} ${_from} PARENT_SCOPE)
	ENDFUNCTION()
ENDIF()

FUNCTION(join VALUES GLUE OUTPUT)
  STRING(REPLACE ";" "${GLUE}" _TMP_STR "${VALUES}")
  SET(${OUTPUT} "${_TMP_STR}" PARENT_SCOPE)
ENDFUNCTION()

IF(NX)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.nx.cmake")
ELSEIF(CAFE)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.cafe.cmake")
ELSEIF(CTR)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.ctr.cmake")
ELSEIF(WIN32 OR CYGWIN OR UNIX)
	IF (NOT NLIB_APPLE_DIR_WORKAROUND AND NOT NLIB_ANDX)
		SET(NLIB_PCDEV 1)
	ENDIF()
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.pc.cmake")
ELSE()
	MESSAGE(FATAL_ERROR "!!!!! Sorry, Platform not supported !!!!!")
ENDIF()

IF(WIN32)
	IF(BUILD_SHARED_LIBS)
		SET(_VS_USER_PROPS_PATH "${NLIB_DIR}")
		SET(_VS_USER_PROPS_PATH_DEBUG "${NLIB_DIR}")
	ELSE()
		SET(_VS_USER_PROPS_PATH "")
		SET(_VS_USER_PROPS_PATH_DEBUG "")
	ENDIF()
	IF(ZLIB_FOUND)
		GET_FILENAME_COMPONENT(_path "${ZLIB_INCLUDE_DIRS}/../" ABSOLUTE)
		LIST(APPEND _VS_USER_PROPS_PATH "${_path}/bin")
		LIST(APPEND _VS_USER_PROPS_PATH_DEBUG "${_path}/debug/bin")
		LIST(APPEND _VS_USER_PROPS_PATH_DEBUG "${_path}/bin")
	ENDIF()
	IF(CURL_FOUND)
		GET_FILENAME_COMPONENT(_path "${CURL_INCLUDE_DIRS}/../" ABSOLUTE)
		LIST(APPEND _VS_USER_PROPS_PATH "${_path}/bin")
		LIST(APPEND _VS_USER_PROPS_PATH_DEBUG "${_path}/debug/bin")
		LIST(APPEND _VS_USER_PROPS_PATH_DEBUG "${_path}/bin")
	ENDIF()
	LIST(REMOVE_DUPLICATES _VS_USER_PROPS_PATH)
	LIST(REMOVE_DUPLICATES _VS_USER_PROPS_PATH_DEBUG)
ENDIF()

FUNCTION(nlib_add_executable target)
	SET(options "")
	SET(oneValueArgs NAME MSVCIDE_RUN_PATH)
	SET(multiValueArgs COMMAND "")
	cmake_parse_arguments(nlib_add_executable "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})
	ADD_EXECUTABLE(${target} ${nlib_add_executable_UNPARSED_ARGUMENTS})
	IF(WIN32)
		SET(_PATH ${nlib_add_executable_MSVCIDE_RUN_PATH} ${_VS_USER_PROPS_PATH})
		SET(_PATH_DEBUG ${nlib_add_executable_MSVCIDE_RUN_PATH} ${_VS_USER_PROPS_PATH_DEBUG})
		# https://stackoverflow.com/questions/1005901/how-to-set-path-environment-variable-using-cmake-and-visual-studio-to-run-test
		FILE(WRITE "${CMAKE_CURRENT_BINARY_DIR}/${target}.Cpp.user.props" [=[
<?xml version="1.0" encoding="utf-8"?> 
<Project DefaultTargets="Build" ToolsVersion="15.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
<PropertyGroup Condition="'$(Configuration)|$(Platform)'=='Debug|Win32'">
	<LocalDebuggerEnvironment>PATH=]=] "${_PATH_DEBUG}" [=[</LocalDebuggerEnvironment>
</PropertyGroup>
<PropertyGroup Condition="'$(Configuration)|$(Platform)'=='Release|Win32'">
	<LocalDebuggerEnvironment>PATH=]=] "${_PATH}" [=[</LocalDebuggerEnvironment>
</PropertyGroup>
<PropertyGroup Condition="'$(Configuration)|$(Platform)'=='Debug|x64'">
	<LocalDebuggerEnvironment>PATH=]=] "${_PATH_DEBUG}" [=[</LocalDebuggerEnvironment>
</PropertyGroup>
<PropertyGroup Condition="'$(Configuration)|$(Platform)'=='Release|x64'">
	<LocalDebuggerEnvironment>PATH=]=] "${_PATH}" [=[</LocalDebuggerEnvironment>
</PropertyGroup>
</Project>
		]=])
		SET_PROPERTY(TARGET ${target} PROPERTY VS_USER_PROPS "${target}.Cpp.user.props")
	ENDIF()
ENDFUNCTION()

IF(NLIB_PCDEV)
	# NLIB_TOOL_DIR may not exist yet
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/${CAT_DIR}/Release)
ELSEIF(CMAKE_HOST_SYSTEM_NAME STREQUAL Windows)
	FILE(GLOB bindir_list LIST_DIRECTORIES TRUE ${NLIB_RELEASE_ROOT}/bin/cmake/vc*/Release)
	LIST(FILTER bindir_list EXCLUDE REGEX "vc.+dll")
	IF(bindir_list)
		LIST(SORT bindir_list)
		LIST(REVERSE bindir_list)
		LIST(GET bindir_list 0 NLIB_TOOL_DIR)
	ELSE()
		MESSAGE(WARNING "MSVC build not found, may fail in compiling some samples.")
		SET(NLIB_TOOL_DIR "ERROR_TOOL_DIR_NOTFOUND")
	ENDIF()
ELSE()
	# neither PCDEV nor Windows host
	IF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
	ELSE()
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
	ENDIF()
ENDIF()

MESSAGE(STATUS "NLIB_TOOL_DIR=${NLIB_TOOL_DIR}")

SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Release>:NDEBUG>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "__STDC_WANT_LIB_EXT1__=1")

MESSAGE(STATUS "NLIB_DIR=${NLIB_DIR}")
MESSAGE(STATUS "CMAKE_BUILD_TYPE : ${CMAKE_BUILD_TYPE}")
MESSAGE(STATUS "CMAKE_CXX_COMPILER_ID : ${CMAKE_CXX_COMPILER_ID}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS : ${CMAKE_CXX_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_DEBUG : ${CMAKE_CXX_FLAGS_DEBUG}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_RELWITHDEBINFO : ${CMAKE_CXX_FLAGS_RELWITHDEBINFO}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_RELEASE : ${CMAKE_CXX_FLAGS_RELEASE}")
MESSAGE(STATUS "CMAKE_C_FLAGS : ${CMAKE_C_FLAGS}")
MESSAGE(STATUS "CMAKE_C_FLAGS_DEBUG : ${CMAKE_C_FLAGS_DEBUG}")
MESSAGE(STATUS "CMAKE_C_FLAGS_RELWITHDEBINFO : ${CMAKE_C_FLAGS_RELWITHDEBINFO}")
MESSAGE(STATUS "CMAKE_C_FLAGS_RELEASE : ${CMAKE_C_FLAGS_RELEASE}")
MESSAGE(STATUS "CMAKE_EXE_LINKER_FLAGS : ${CMAKE_EXE_LINKER_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_LINK_FLAGS: ${CMAKE_CXX_LINK_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_LINK_EXECUTABLE : ${CMAKE_CXX_LINK_EXECUTABLE}")
MESSAGE(STATUS "CMAKE_DEBUG_POSTFIX : ${CMAKE_DEBUG_POSTFIX}")
MESSAGE(STATUS "CMAKE_RELEASE_POSTFIX : ${CMAKE_RELEASE_POSTFIX}")
MESSAGE(STATUS "CMAKE_PREFIX_PATH : ${CMAKE_PREFIX_PATH}")
MESSAGE(STATUS "CMAKE_MODULE_PATH : ${CMAKE_MODULE_PATH}")

# you can check command lines if you uncomment this
# set( CMAKE_VERBOSE_MAKEFILE on )
