INCLUDE(CMakeForceCompiler)

IF(NOT CMAKE_TOOLCHAIN_FILE)
	MESSAGE(FATAL_ERROR "CMAKE_TOOLCHAIN_FILE not defined")
ENDIF()

SET(CMAKE_EXECUTABLE_SUFFIX "$CACHE{CMAKE_EXECUTABLE_SUFFIX}")

IF(${CMAKE_BUILD_TYPE} MATCHES "Release")
	SET(CTR_BUILD_TYPE "release")
	ADD_DEFINITIONS(-DNDEBUG)
ELSEIF(${CMAKE_BUILD_TYPE} MATCHES "Debug")
	SET(CTR_BUILD_TYPE "noopt")
ELSE()
	SET(CTR_BUILD_TYPE "verbose")
ENDIF()

GET_FILENAME_COMPONENT(
	tmp
	"${NLIB_RELEASE_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}"
	ABSOLUTE)
nlib_cygwin_getwinpath(${tmp} NLIB_DIR)

MESSAGE(STATUS "CTR_BUILD_TYPE=${CTR_BUILD_TYPE}")

SET(CAT_DIR "CAT_DIR_UNUSED")
SET(ZLIB_FOUND 1)
ADD_LIBRARY(ZLIB::ZLIB STATIC IMPORTED)
SET(ZLIB_INCLUDE_DIRS "${CTRSDK_ROOT}/include")
SET(ZLIB_LIBRARIES "${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libzlib.fast.a")
SET_PROPERTY(TARGET ZLIB::ZLIB PROPERTY INTERFACE_INCLUDE_DIRECTORIES "${ZLIB_INCLUDE_DIRS}")
SET_PROPERTY(TARGET ZLIB::ZLIB PROPERTY IMPORTED_LOCATION "${ZLIB_LIBRARIES}")
SET(ZLIB_MAJOR_VERSION 1)
SET(ZLIB_MINOR_VERSION 2)
SET(ZLIB_PATCH_VERSION 7)

SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_HAS_ZLIB")

FUNCTION(library_default_build_rule target)
	ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE INTERFACE "${NLIB_PKGROOT_DIR}/include")
	INSTALL(TARGETS ${target} DESTINATION libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE})
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDFUNCTION()

FUNCTION(generate_ctr_makeromopt target)
	GET_TARGET_PROPERTY(ctr_spec_desc ${target} CTR_SPEC_DESC)
	GET_TARGET_PROPERTY(ctr_spec_rsf ${target} CTR_SPEC_RSF)
	GET_TARGET_PROPERTY(ctr_banner ${target} CTR_BANNER)
	GET_TARGET_PROPERTY(ctr_icon ${target} CTR_ICON)
	nlib_cygwin_getwinpath(${ctr_spec_desc} ctr_spec_desc)
	nlib_cygwin_getwinpath(${ctr_spec_rsf} ctr_spec_rsf)
	nlib_cygwin_getwinpath(${ctr_banner} ctr_banner)
	nlib_cygwin_getwinpath(${ctr_icon} ctr_icon)
	FILE(WRITE ${PROJECT_BINARY_DIR}/CMakeFiles/${target}.dir/makeromopt.txt
		"-desc ${ctr_spec_desc} -rsf ${ctr_spec_rsf} -DROMFS_ROOT= -DTITLE=CtrApp -banner ${ctr_banner} -icon ${ctr_icon} -f card")
ENDFUNCTION()

FUNCTION(generate_ctr_default_makeromopt target)
	SET_TARGET_PROPERTIES(${target} PROPERTIES CTR_SPEC_DESC "${WIN_CTRSDK_ROOT}/resources/specfiles/Application.desc")
	SET_TARGET_PROPERTIES(${target} PROPERTIES CTR_SPEC_RSF "${WIN_CTRSDK_ROOT}/resources/specfiles/Application.rsf")
	SET_TARGET_PROPERTIES(${target} PROPERTIES CTR_BANNER "${WIN_CTRSDK_ROOT}/resources/banner/Default.bnr")
	SET_TARGET_PROPERTIES(${target} PROPERTIES CTR_ICON "${WIN_CTRSDK_ROOT}/resources/banner/Default.icn")
	generate_ctr_makeromopt(${target})
ENDFUNCTION()

FUNCTION(target_link_libraries_default target)
	TARGET_LINK_LIBRARIES(${target}
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libgles2.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ac.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_applet.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_boss.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_camera.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_cec.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_cfg.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_crypto.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_cryptoRsa.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_cx.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_dbg.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_dbm.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_dev.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_dlp.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_dsp.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ec.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_err.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_erreula.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_fnd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_font.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_friends.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_fs.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_fshio.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_gd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_gr.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_gxlow.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_hid.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_hidlow.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_hio.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_http.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_init.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ir.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_jpegmp.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_math.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_mic.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_midi.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ndm.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_news.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ngc.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_nstd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_nwm.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_os.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_phtsel.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_pl.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ptm.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_rdt.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_snd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_socket.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_srv.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ssl.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_svc.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_svcDbg.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_swkbd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_tpl.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ubl.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_uds.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_ulcd.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_util.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_voicesel.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/libnn_y2r.fast.a
		${CTRSDK_ROOT}/libraries/CTR-TS.Process.MPCore/${CTR_BUILD_TYPE}/librtport.fast.a
	)
	generate_ctr_default_makeromopt(${target})
ENDFUNCTION()
