
IF(WIN32)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.win32.cmake")
ELSE()
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.unix.cmake")
ENDIF()

MESSAGE(STATUS "ZLIB_ROOT=${ZLIB_ROOT}")
FIND_PACKAGE(ZLIB)

IF(ZLIB_FOUND)
	MESSAGE(STATUS "ZLIB_INCLUDE_DIRS=${ZLIB_INCLUDE_DIRS}")
	MESSAGE(STATUS "ZLIB_LIBRARIES=${ZLIB_LIBRARIES}")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_HAS_ZLIB")
	IF(WIN32)
		GET_FILENAME_COMPONENT(_path "${ZLIB_INCLUDE_DIRS}/../" ABSOLUTE)
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/bin")
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/debug/bin")
	ENDIF()
ELSE()
	MESSAGE(STATUS "ZLIB not found, ZlibInputStream and ZlibOutputStream not available")
	IF(WIN32)
		MESSAGE(STATUS "setup/Update-MsvcLibraries.ps1 can setup zlib for windows")
	ENDIF()
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_NO_ZLIB")
ENDIF()

FIND_PACKAGE(CURL)
IF(CURL_FOUND)
	MESSAGE(STATUS "CURL_INCLUDE_DIRS=${CURL_INCLUDE_DIRS}")
	MESSAGE(STATUS "CURL_LIBRARIES=${CURL_LIBRARIES}")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_HAS_LIBCURL")
	IF(WIN32)
		# hack for vcpkg
		SET(CURL_LIBRARY_DEBUG ${CURL_LIBRARIES})
		STRING(REPLACE "debug/" "" CURL_LIBRARY_RELEASE ${CURL_LIBRARY_DEBUG})
		IF(NOT EXISTS ${CURL_LIBRARY_DEBUG})
			UNSET(CURL_LIBRARY_RELEASE)
		ELSE()
			UNSET(CURL_LIBRARIES)
		ENDIF()
		MESSAGE(STATUS "CURL_LIBRARY_DEBUG=${CURL_LIBRARY_DEBUG}")
		MESSAGE(STATUS "CURL_LIBRARY_RELEASE=${CURL_LIBRARY_RELEASE}")
		GET_FILENAME_COMPONENT(_path "${CURL_INCLUDE_DIRS}/../" ABSOLUTE)
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/bin")
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/debug/bin")
	ENDIF()
	# needed because FIND_PACKAGE(CURL) does not set CURL::CURL
	IF (NOT TARGET CURL::CURL)
		ADD_LIBRARY(CURL::CURL UNKNOWN IMPORTED)
		set_target_properties(CURL::CURL PROPERTIES
							  INTERFACE_INCLUDE_DIRECTORIES "${CURL_INCLUDE_DIRS}")
		IF(CURL_LIBRARY_RELEASE)
			set_property(TARGET CURL::CURL APPEND PROPERTY IMPORTED_CONFIGURATIONS RELEASE)
			set_target_properties(CURL::CURL PROPERTIES
								IMPORTED_LOCATION_RELEASE "${CURL_LIBRARY_RELEASE}")
		ENDIF()
		IF(CURL_LIBRARY_DEBUG)
			set_property(TARGET CURL::CURL APPEND PROPERTY IMPORTED_CONFIGURATIONS DEBUG)
			set_target_properties(CURL::CURL PROPERTIES
								IMPORTED_LOCATION_DEBUG "${CURL_LIBRARY_DEBUG}")
		ENDIF()
		IF(NOT CURL_LIBRARY_RELEASE AND NOT CURL_LIBRARY_DEBUG)
			set_property(TARGET CURL::CURL APPEND PROPERTY
						IMPORTED_LOCATION "${CURL_LIBRARIES}")
	    ENDIF()
	ENDIF()
ELSE()
	MESSAGE(STATUS "libcurl not found, CurlInputStream not available")
	IF(WIN32)
		MESSAGE(STATUS "setup/Update-MsvcLibraries.ps1 can setup libcurl for windows")
	ENDIF()
ENDIF()

FIND_PACKAGE(LIBXML2)
IF(LIBXML2_FOUND)
	MESSAGE(STATUS "LIBXML2_INCLUDE_DIR=${LIBXML2_INCLUDE_DIR}")
	MESSAGE(STATUS "LIBXML2_LIBRARIES=${LIBXML2_LIBRARIES}")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_HAS_LIBXML2")
	IF(WIN32)
		GET_FILENAME_COMPONENT(_path "${LIBXML2_INCLUDE_DIR}/../" ABSOLUTE)
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/bin")
		LIST(APPEND CMAKE_MSVCIDE_RUN_PATH "${_path}/debug/bin")
	ENDIF()
ENDIF()

IF(CMAKE_MSVCIDE_RUN_PATH)
	LIST(REMOVE_DUPLICATES CMAKE_MSVCIDE_RUN_PATH)
ENDIF()
