
#
# uname -a
#
EXECUTE_PROCESS(
	COMMAND uname -a
	OUTPUT_VARIABLE NLIB_UNAME
)
MESSAGE(STATUS "NLIB_UNAME: ${NLIB_UNAME}")
IF("${NLIB_UNAME}" MATCHES "Alpine")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_ALPINE")
	SET(ALPINE 1)
ENDIF()

IF(("${NLIB_UNAME}" MATCHES "Microsoft") AND (EXISTS "/mnt/c"))
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_BASH_UBUNTU_WINDOWS")
	SET(BASH_UBUNTU_WINDOWS 1)
ENDIF()

#
# ccache
#
FIND_PROGRAM(CCACHE_FOUND ccache)
IF(CCACHE_FOUND)
	SET_PROPERTY(GLOBAL PROPERTY RULE_LAUNCH_COMPILE ccache)
	SET_PROPERTY(GLOBAL PROPERTY RULE_LAUNCH_LINK ccache)
ENDIF()

#
# GCC or clang
#
IF (NOT CAT_DIR)
	IF(CYGWIN)
		SET(CAT_DIR "cygwin")
	ELSEIF(CMAKE_CXX_COMPILER_ID STREQUAL "GNU")
		SET(CAT_DIR "gcc")
	ELSEIF(CMAKE_CXX_COMPILER_ID STREQUAL "Clang")
		SET(CAT_DIR "clang")
	ELSEIF(CMAKE_CXX_COMPILER_ID STREQUAL "AppleClang")
		SET(CAT_DIR "clang")
	ELSE()
		MESSAGE(FATAL_ERROR "cannot set CAT_DIR")
	ENDIF()
ENDIF()

IF(CMAKE_CXX_COMPILER_ID STREQUAL "GNU" AND ${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
	MESSAGE(WARNING "Please use clang for FreeBSD")
ENDIF()

IF(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
	IF("${NLIB_UNAME}" MATCHES "Alpine")
		SET(LIBSTDCXX "/usr/lib/libstdc++.so")
	ELSE()
		EXECUTE_PROCESS(COMMAND /sbin/ldconfig -p
						COMMAND grep libstdc++
						OUTPUT_VARIABLE LDCONFIG)
		STRING(REGEX REPLACE "^.+ => \(.+\)\n" "\\1" LIBSTDCXX ${LDCONFIG})
	ENDIF()
	EXECUTE_PROCESS(COMMAND /usr/bin/strings ${LIBSTDCXX}
					COMMAND grep GLIBCXX
					OUTPUT_VARIABLE GLIBCXX_LIST)
	MESSAGE(STATUS "The compatible versions for libstdc++ is:\n ${GLIBCXX_LIST}")
ENDIF()

#
# Compiler options
#   -ffast-math cannot be specified for SQLite
#
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Debug>:DEBUG;_DEBUG>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Release>:NDEBUG;_FORTIFY_SOURCE=2>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Wall;-Wextra;-Wshadow;-fstack-protector")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<CONFIG:Release>:-O3>")

MESSAGE(STATUS "CMAKE_SYSTEM_PROCESSOR ${CMAKE_SYSTEM_PROCESSOR}")

IF (${OSX_ARCHITECTURES} MATCHES "arm")
ELSEIF(NLIB_ARM)
ELSE()
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-msse4.2")
	IF(CMAKE_SIZEOF_VOID_P EQUAL 8)
	ELSE()
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-mfpmath=sse")
	ENDIF()
ENDIF()

SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -fstack-protector")
SET(CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -fstack-protector")
SET(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -fstack-protector")

# gcc specific compiler options
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<CXX_COMPILER_ID:GNU>:>")
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<C_COMPILER_ID:GNU>:>")

# clang compiler options
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<OR:$<CXX_COMPILER_ID:Clang>,$<CXX_COMPILER_ID:AppleClang>>:>")
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<OR:$<C_COMPILER_ID:Clang>,$<C_COMPILER_ID:AppleClang>>:>")

IF(CMAKE_VERSION VERSION_LESS 3.8)
	SET(CMAKE_CXX_STANDARD 14)
ELSE()
	IF(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
		IF(CMAKE_CXX_COMPILER_ID STREQUAL "GNU" OR GLIBCXX_LIST MATCHES "3.4.25")
			# libstdc++ for gcc 7.2.1 cannot be compiled by clang in C++17 mode(optional:1032)
			# https://gcc.gnu.org/onlinedocs/libstdc++/manual/abi.html#abi.versioning
			# GCC 7.2.0: libstdc++.so.6.0.24
			# GCC 8.0.0: libstdc++.so.6.0.25
			SET(CMAKE_CXX_STANDARD 17)
		ELSE()
			SET(CMAKE_CXX_STANDARD 14)
		ENDIF()
	ELSEIF((${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD") AND ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "Clang"))
		SET(CMAKE_CXX_STANDARD 17)
	ELSE()
		SET(CMAKE_CXX_STANDARD 14)
	ENDIF()
ENDIF()
MESSAGE(STATUS "CMAKE_CXX_STANDARD ${CMAKE_CXX_STANDARD}")

IF(CMAKE_CXX_COMPILER_ID STREQUAL "GNU")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Wstack-usage=131072")
ENDIF()

IF(DEFINED ENV{FSANITIZE})
	# export FSANITIZE=address
	# export FSANITIZE=thread
	# export FSANITIZE=memory
	# export FSANITIZE=undefined
	# export FSANITIZE=dataflow
	# export FSANITIZE=cfi
	# export FSANITIZE=safe-stack
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-fsanitize=$ENV{FSANITIZE}")
	# LINK_FLAGS property is not for DIRECTORY
	SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -fsanitize=$ENV{FSANITIZE}")
	IF($ENV{FSANITIZE} MATCHES "address")
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-fno-omit-frame-pointer;-fno-optimize-sibling-calls;-fsanitize-address-use-after-scope")
	ENDIF()
	IF($ENV{FSANITIZE} MATCHES "memory")
		# ubuntu:
		# export MSAN_SYMBOLIZER_PATH=$(which llvm-symbolizer-3.4)
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-fno-omit-frame-pointer;-fno-optimize-sibling-calls;-fsanitize-memory-track-origins")
	ENDIF()
	IF($ENV{FSANITIZE} MATCHES "cfi")
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-flto")
	ENDIF()
ENDIF()

SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "_GNU_SOURCE")
IF(CYGWIN)
	OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." OFF)
	SET(CMAKE_DEBUG_POSTFIX "_d")
ELSEIF(UNIX)
	SET(CMAKE_DEBUG_POSTFIX "_d")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-fPIC")
	IF(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
		OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
	ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
		OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
	ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-stdlib=libc++")
		IF(NOT NLIB_APPLE_DIR_WORKAROUND)
			OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
		ELSE()
			OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." OFF)
		ENDIF()
	ELSEIF(NLIB_ANDX)
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_THREAD_AA_;NLIB_THREAD_EXIT_CPP_THROW_NLIB_EXCEPTION")
		OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
	ELSE()
		MESSAGE(FATAL_ERROR "Unknown environment for nlib")
	ENDIF()
ENDIF()

GET_FILENAME_COMPONENT(NLIB_DIR "${NLIB_RELEASE_ROOT}/lib/cmake/${CAT_DIR}" ABSOLUTE)

FUNCTION(library_default_build_rule target)
	IF(UNIX AND NOT CYGWIN)
		IF(NOT NLIB_APPLE_DIR_WORKAROUND)
			ADD_LIBRARY(${target} ${LIB_SRC} ${LIB_HEADER})
			IF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
				MATH(EXPR BN0 "${NLIB_BUILD_NUMBER} / 1000")
				MATH(EXPR BN1 "${NLIB_BUILD_NUMBER} % 1000")
				SET_PROPERTY(TARGET ${target} PROPERTY VERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}.${BN0}.${BN1}")
			ELSEIF(NOT BASH_UBUNTU_WINDOWS)
				SET_PROPERTY(TARGET ${target} PROPERTY VERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}.${NLIB_BUILD_NUMBER}")
			ENDIF()
			IF(NOT BASH_UBUNTU_WINDOWS)
				SET_PROPERTY(TARGET ${target} PROPERTY SOVERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}")
			ENDIF()
			SET_PROPERTY(TARGET ${target} PROPERTY CXX_VISIBILITY_PRESET hidden)
			SET_PROPERTY(TARGET ${target} PROPERTY C_VISIBILITY_PRESET hidden)
			SET_PROPERTY(TARGET ${target} PROPERTY VISIBILITY_INLINES_HIDDEN ON)
		ELSE()
			ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
		ENDIF()
	ELSE()
		ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	ENDIF()
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE INTERFACE "${NLIB_PKGROOT_DIR}/include")
	IF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
		TARGET_INCLUDE_DIRECTORIES(${target} PRIVATE "/usr/local/include")
	ENDIF()
	target_link_libraries_default(${target})
	IF(BASH_UBUNTU_WINDOWS)
		INSTALL(TARGETS ${target} LIBRARY DESTINATION ${NLIB_DIR} NAMELINK_SKIP)
	ELSEIF(NLIB_APPLE_DIR_WORKAROUND)
		INSTALL(DIRECTORY ${PROJECT_BINARY_DIR}/${CMAKE_BUILD_TYPE}${CMAKE_XCODE_EFFECTIVE_PLATFORMS}/
				DESTINATION ${NLIB_DIR})
	ELSE()
		INSTALL(TARGETS ${target} DESTINATION ${NLIB_DIR})
	ENDIF()
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDFUNCTION()

IF(CMAKE_VERSION VERSION_GREATER "3.8.99")
	CMAKE_POLICY(SET CMP0068 NEW)
ENDIF()

FUNCTION(target_link_libraries_default target)
	IF(APPLE AND CMAKE_MACOSX_BUNDLE)
		SET_PROPERTY(TARGET "${target}" PROPERTY MACOSX_BUNDLE_INFO_PLIST "${NLIB_PKGROOT_DIR}/../misc/samples/apple/Info.plist")
		SET_PROPERTY(TARGET "${target}" APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_OSX_MAINFUNC_NAME=LaunchSample")
		SET_PROPERTY(TARGET "${target}" PROPERTY XCODE_ATTRIBUTE_CODE_SIGN_IDENTITY "${XCODE_CODE_SIGN}")
	ENDIF()
	IF(BUILD_SHARED_LIBS)
		SET_PROPERTY(TARGET ${pgname} PROPERTY SKIP_BUILD_RPATH FALSE)
		SET_PROPERTY(TARGET ${pgname} PROPERTY BUILD_WITH_INSTALL_RPATH FALSE)
		SET_PROPERTY(TARGET ${pgname} PROPERTY INSTALL_RPATH_USE_LINK_PATH TRUE)
		IF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
			#SET_PROPERTY(TARGET ${pgname} PROPERTY BUILD_WITH_INSTALL_NAME_DIR TRUE)
			SET_PROPERTY(TARGET ${pgname} PROPERTY MACOSX_RPATH TRUE)
			SET_PROPERTY(TARGET ${pgname} PROPERTY INSTALL_RPATH "@executable_path/../../../../lib/cmake/${CAT_DIR}")
			#SET_PROPERTY(TARGET ${pgname} APPEND PROPERTY LINK_FLAGS "-Wl,-rpath,${CMAKE_INSTALL_PREFIX}/lib/cmake/${CAT_DIR}")
			#SET_PROPERTY(TARGET ${pgname} PROPERTY INSTALL_NAME_DIR "@executable_path/../../../../lib/cmake/${CAT_DIR}")
		ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
			SET_PROPERTY(TARGET ${pgname} PROPERTY INSTALL_RPATH "$ORIGIN/../../../../lib/cmake/${CAT_DIR}:$ORIGIN/")
		ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
			SET_PROPERTY(TARGET ${pgname} PROPERTY INSTALL_RPATH "${CMAKE_INSTALL_PREFIX}/lib/cmake/clang")
		ENDIF()
	ENDIF()
	IF(APPLE)
		SET_PROPERTY(TARGET "${target}" APPEND PROPERTY COMPILE_DEFINITIONS "CMAKE_CURRENT_BINARY_DIR=\"$<TARGET_FILE_DIR:${target}>\"")
	ENDIF()
ENDFUNCTION()
