﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CONFIG_CLANG_H_
#define INCLUDE_NN_NLIB_CONFIG_CLANG_H_
#ifndef INCLUDE_NN_NLIB_CONFIG_H_
#error do not include directly
#endif

#if defined(__clang__)
#if !defined(__NX__) && ((__clang_major__ == 3 && __clang_minor__ < 4) || __clang_major__ < 3)
#error use clang 3.4 or later
#endif

#if __cplusplus < 201103L
#error Use C++11 or later
#endif

#define NLIB_CXX11_SWAPHEADER

#define NLIB_CXX11_UNIQUEPTR
#define NLIB_CXX11_EXTERN_TEMPLATES
#define NLIB_CXX11_DEFAULT_TEMPLATE_ARGUMENT_FOR_FUNCTION_TEMPLATES

#if __has_feature(cxx_exceptions)
#ifndef __cpp_exceptions
#define __cpp_exceptions 199711L
#endif
#endif

#if __has_feature(cxx_lambdas)
#ifndef __cpp_lambdas
#define __cpp_lambdas 200907L  // N2927
#endif
#endif

#if __has_feature(cxx_rvalue_references)
#ifndef __cpp_rvalue_references
#define __cpp_rvalue_references 200610L  // N2118
#endif
#endif

#if __has_feature(cxx_alias_templates)
#ifndef __cpp_alias_templates
#define __cpp_alias_templates 200704L  // N2258
#endif
#endif

#if __has_feature(cxx_deleted_functions) && __has_feature(cxx_defaulted_functions)
#define NLIB_CXX11_DEFAULTED_AND_DELETED_FUNCTIONS
#endif

#if __has_feature(cxx_explicit_conversions)
#define NLIB_CXX11_EXPLICIT_CONVERSION_OPERATORS
#endif

#if __has_feature(cxx_nullptr)
#define NLIB_CXX11_NULL_POINTER_CONSTANT
#endif

#if __has_feature(cxx_static_assert)
#ifndef __cpp_static_assert
#define __cpp_static_assert 200410L  // N1720
#endif
#endif

#if __has_feature(cxx_range_for)
#ifndef __cpp_range_based_for
#define __cpp_range_based_for 200907L  // N2930
#endif
#endif

#if __has_feature(cxx_override_control)
#define NLIB_CXX11_EXPLICIT_VIRTUAL_OVERRIDES
#endif

#if __has_feature(cxx_relaxed_constexpr)
#ifndef __cpp_constexpr
#define __cpp_constexpr 201304L  // N3652
#endif
#elif __has_feature(cxx_constexpr)
#ifndef __cpp_constexpr
#define __cpp_constexpr 200704L  // N2235
#endif
#endif

#if __has_feature(cxx_alignas)
#define NLIB_CXX11_ALIGNMENT_SUPPORT
#endif

#if __has_feature(cxx_variadic_templates)
#ifndef __cpp_variadic_templates
#define __cpp_variadic_templates 200704L  // N2242
#endif
#endif

#if __has_feature(cxx_delegating_constructors)
#ifndef __cpp_delegating_constructors
#define __cpp_delegating_constructors 200604L  // N1986
#endif
#endif

#if __has_include(<chrono>)
#define NLIB_CXX11_STDLIB_CHRONO
#endif

#if __has_include(<atomic>)
#define NLIB_CXX11_STDLIB_ATOMIC
#endif

#if __has_include(<random>)
#define NLIB_CXX11_STDLIB_RANDOM
#endif

#if __has_include(<array>)
#define NLIB_CXX11_STDLIB_ARRAY
#endif

#if __has_include(<unordered_set>) && __has_include(<unordered_map>)
#define NLIB_CXX11_STDLIB_UNORDERED
#endif

#if __has_include(<tuple>)
#define NLIB_CXX11_STDLIB_TUPLE
#endif

#if __has_feature(cxx_generalized_initializers)
#ifndef __cpp_initializer_lists
#define __cpp_initializer_lists 200806L  // N2672
#endif
#endif
#if __has_feature(cxx_nonstatic_member_init)
#ifndef __cpp_nsdmi
#define __cpp_nsdmi 200809L  // N2756
#endif
#endif
#if __has_feature(cxx_inheriting_constructors)
#ifndef __cpp_inheriting_constructors
#define __cpp_inheriting_constructors 200802L  // N2540
#endif
#endif
#if __has_feature(cxx_reference_qualified_functions)
#ifndef __cpp_ref_qualifiers
#define __cpp_ref_qualifiers 200710L  // N2439
#endif
#endif
#if __cplusplus > 201103L && __clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 7)
// roughly correct(it is complex because clang may use libstdc++)
// __cpp_sized_deallocation defined if -fsized-deallocation specified
#ifndef __cpp_lib_is_null_pointer
#define __cpp_lib_is_null_pointer 201309L  // LWG2247
#endif
#ifndef __cpp_lib_complex_udls
#define __cpp_lib_complex_udls 201309L  // N3779
#endif
#ifndef __cpp_lib_tuple_element_t
#define __cpp_lib_tuple_element_t 201402L  // N3887
#endif
#ifndef __cpp_lib_make_unique
#define __cpp_lib_make_unique 201304L  // N3656
#endif
#ifndef __cpp_lib_shared_timed_mutex
#define __cpp_lib_shared_timed_mutex 201402L  // N3891
#endif
#ifndef __cpp_lib_integer_sequence
#define __cpp_lib_integer_sequence 201304L  // N3658
#endif
#ifndef __cpp_lib_exchange_function
#define __cpp_lib_exchange_function 201304L  // N3668
#endif
#ifndef __cpp_lib_tuples_by_type
#define __cpp_lib_tuples_by_type 201304L  // N3670
#endif
#ifndef __cpp_lib_integral_constant_callable
#define __cpp_lib_integral_constant_callable 201304L  // N3545
#endif
#ifndef __cpp_lib_transparent_operators
#define __cpp_lib_transparent_operators 201210L  // N3421
#endif
#ifndef __cpp_lib_transformation_trait_aliases
#define __cpp_lib_transformation_trait_aliases 201304L  // N3655
#endif
#ifndef __cpp_lib_result_of_sfinae
#define __cpp_lib_result_of_sfinae 201210L  // N3642
#endif
#ifndef __cpp_lib_is_final
#define __cpp_lib_is_final 201402L  // LWG2112
#endif
#ifndef __cpp_lib_chrono_udls
#define __cpp_lib_chrono_udls 201304L  // N3642
#endif
#ifndef __cpp_lib_string_udls
#define __cpp_lib_string_udls 201304L  // N3642
#endif
#ifndef __cpp_lib_generic_associative_lookup
#define __cpp_lib_generic_associative_lookup 201304L  // N3657
#endif
#ifndef __cpp_lib_null_iterators
#define __cpp_lib_null_iterators 201304L  // N3644
#endif
#ifndef __cpp_lib_make_reverse_iterator
#define __cpp_lib_make_reverse_iterator 201402L  // LWG2285
#endif
#ifndef __cpp_lib_robust_nonmodifying_seq_ops
#define __cpp_lib_robust_nonmodifying_seq_ops 201304L  // N3671
#endif
#ifndef __cpp_lib_quoted_string_io
#define __cpp_lib_quoted_string_io 201304L  // N3654
#endif
#endif

#include <cassert>
#ifndef NLIB_ASSERT
#define NLIB_ASSERT assert
#endif

#define NLIB_ALIGNAS(x) __attribute__((aligned(x)))
#define NLIB_ALIGNOF(tp) __alignof__(tp)

#define NLIB_HAS_NATIVE_TYPETRAITS

#endif
#endif  // INCLUDE_NN_NLIB_CONFIG_CLANG_H_
