﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
#define INCLUDE_NN_NLIB_PLATFORM_H_

#ifdef __ILP64__
# error Sorry, ILP64 not supported
#endif

// Note that the C preprocessor # operator must be able to use.
#define NLIB_VERSION_YEAR 2018
#define NLIB_VERSION_YEAR_SHORT 18
#define NLIB_VERSION_DATE 0605
#define NLIB_VERSION 20180605

#ifdef __cplusplus
# ifndef __STDC_FORMAT_MACROS
#  define __STDC_FORMAT_MACROS  // not needed in C11
# endif
#endif
#ifndef __STDC_WANT_LIB_EXT1__
# define __STDC_WANT_LIB_EXT1__ 1
#endif

#ifndef __has_builtin
# define __has_builtin(x) 0
#endif

#ifndef __has_feature
# define __has_feature(x) 0
#endif

#ifndef __has_cpp_attribute
# define __has_cpp_attribute(x) 0
#endif

#ifndef __has_attribute
# define __has_attribute(x) 0
#endif

#ifndef __has_declspec_attribute
# define __has_declspec_attribute(x) 0
#endif

#ifndef __has_include
# define __has_include(x) 0
#endif

#if __has_feature(thread_sanitizer)
# define NLIB_NO_TSAN __attribute__((no_sanitize("thread")))
#else
# define NLIB_NO_TSAN
#endif

#if __has_feature(address_sanitizer)
# define NLIB_NO_ASAN __attribute__((no_sanitize("address")))
#else
# define NLIB_NO_ASAN
#endif

#if __has_feature(memory_sanitizer)
# define NLIB_NO_MSAN __attribute__((no_sanitize("memory")))
#else
# define NLIB_NO_MSAN
#endif

#ifndef NLIB_UNUSED
// #if defined(__cplusplus) && __has_cpp_attribute(maybe_unused)
// # define NLIB_UNUSED(x) [[maybe_unused]]
// #else
# define NLIB_UNUSED(x) (void)(x)
// #endif
#endif

//
// thread safety analysis
// http://clang.llvm.org/docs/ThreadSafetyAnalysis.html
//
#if !defined(NLIB_THREAD_AA_) && defined(__clang__) && \
    (!defined(SWIG)) && __has_attribute(capability)
#define NLIB_THREAD_AA_(x) __attribute__((x))
#else
#ifdef NLIB_THREAD_AA_
#undef NLIB_THREAD_AA_
#endif
#define NLIB_THREAD_AA_(x)
#endif

#define NLIB_CAPABILITY(x) NLIB_THREAD_AA_(capability(x))
#define NLIB_SCOPED_CAPABILITY NLIB_THREAD_AA_(scoped_lockable)
#define NLIB_GUARDED_BY(x) NLIB_THREAD_AA_(guarded_by(x))
#define NLIB_PT_GUARDED_BY(x) NLIB_THREAD_AA_(pt_guarded_by(x))
#define NLIB_ACQUIRED_BEFORE(...) NLIB_THREAD_AA_(acquired_before(__VA_ARGS__))
#define NLIB_ACQUIRED_AFTER(...) NLIB_THREAD_AA_(acquired_after(__VA_ARGS__))
#define NLIB_REQUIRES(...) NLIB_THREAD_AA_(requires_capability(__VA_ARGS__))
#define NLIB_REQUIRES_SHARED(...) NLIB_THREAD_AA_(requires_shared_capability(__VA_ARGS__))
#define NLIB_ACQUIRE(...) NLIB_THREAD_AA_(acquire_capability(__VA_ARGS__))
#define NLIB_ACQUIRE_SHARED(...) NLIB_THREAD_AA_(acquire_shared_capability(__VA_ARGS__))
#define NLIB_RELEASE(...) NLIB_THREAD_AA_(release_capability(__VA_ARGS__))
#define NLIB_RELEASE_SHARED(...) NLIB_THREAD_AA_(release_shared_capability(__VA_ARGS__))
#define NLIB_TRY_ACQUIRE(...) NLIB_THREAD_AA_(try_acquire_capability(__VA_ARGS__))
#define NLIB_TRY_ACQUIRE_SHARED(...) NLIB_THREAD_AA_(try_acquire_shared_capability(__VA_ARGS__))
#define NLIB_EXCLUDES(...) NLIB_THREAD_AA_(locks_excluded(__VA_ARGS__))
#define NLIB_ASSERT_CAPABILITY(x) NLIB_THREAD_AA_(assert_capability(x))
#define NLIB_ASSERT_SHARED_CAPABILITY(x) NLIB_THREAD_AA_(assert_shared_capability(x))
#define NLIB_RETURN_CAPABILITY(x) NLIB_THREAD_AA_(lock_returned(x))
#define NLIB_NO_THREAD_SAFETY_ANALYSIS NLIB_THREAD_AA_(no_thread_safety_analysis)

#include <stddef.h>  // for size_t
#include <stdio.h>   // for SEEK_CUR, SEEK_END, SEEK_SET
#include <stdarg.h>  // for va_list
#include <string.h>  // for memcpy, memmove, memset
#include <time.h>    // for struct timespec
#if (defined(__cplusplus) && __cplusplus >= 201703L) || \
    (defined(_MSVC_LANG) && _MSVC_LANG >= 201703L)
#include <cstddef>   // for std::byte
#endif

#if defined(NLIB_RENAME_CAPI)
#include "nn/nlib/Platform_rename.h"
#else
#ifdef NLIB_CAPI
#undef NLIB_CAPI
#endif
#define NLIB_CAPI(x) x
#endif

#define NLIB_STRINGIFY_(s) #s
#define NLIB_STRINGIFY(s) NLIB_STRINGIFY_(s)

#if defined(_MSC_VER)
# include "nn/nlib/Platform_win32.h"
#elif defined(__linux__)   || \
      defined(__FreeBSD__) || \
      defined(__CYGWIN__)  || \
     (defined(__APPLE__) && defined(__MACH__))
# ifndef NLIB_UNIX
#  define NLIB_UNIX
# endif
# include "nn/nlib/Platform_unix.h"
#elif defined(NN_PLATFORM_CTR)
# include "nn/nlib/Platform_ctr.h"
#elif defined(CAFE)
# include "nn/nlib/Platform_cafe.h"
#elif defined(__NX__)
# include "nn/nlib/Platform_nx.h"
#endif

#if defined(_MSC_VER) && defined(n_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

#ifndef __analysis_assume
# define __analysis_assume(expr)
#endif

#ifndef _Printf_format_string_
# define _Printf_format_string_
#endif

#if defined(__ARM_NEON__) || defined(__aarch64__)
# ifndef NLIB_NEON
#  define NLIB_NEON
# endif
#endif

#ifdef __SSE4_1__
# ifndef NLIB_SSE41
#  define NLIB_SSE41
# endif
#endif

#ifdef __SSE4_2__
# ifndef NLIB_SSE41
#  define NLIB_SSE41
# endif
# ifndef NLIB_SSE42
#  define NLIB_SSE42
# endif
#endif

#if defined(NLIB_SSE41) || defined(NLIB_NEON)
# define NLIB_SIMD
# ifdef NLIB_NEON
#  include <arm_neon.h>
# endif
# ifdef NLIB_SSE41
#  include <smmintrin.h>
# endif
# ifdef NLIB_SSE42
#  include <nmmintrin.h>
# endif
#endif

#if defined(__ARM_ACLE)
#include <arm_acle.h>
#endif

// https://www.jpcert.or.jp/sc-rules/c-int01-c.html
// https://www.securecoding.cert.org/confluence/display/seccode/INT01-C.+Use+rsize_t+or+size_t+for+all+integer+values+representing+the+size+of+an+object
#ifndef RSIZE_MAX
# ifndef NLIB_64BIT
#  define RSIZE_MAX 0x7FFFFFFFL
# else
#  define RSIZE_MAX 0x7FFFFFFFFFFFFFFFLL
# endif
#endif

#ifndef NLIB_WARN
# define NLIB_WARN(exp) ("WARNING: " exp)
// #pragma message NLIB_WARN("your message")
#endif

// NOTE:
// You can use those macros on the environment which printf does not support '%z'
// for size_t.
// "%"PRIuS, sizet_val
// See: https://google-styleguide.googlecode.com/svn/trunk/cppguide.html#64-bit_Portability
// In Japanese: http://www.textdrop.net/google-styleguide-ja/cppguide.xml
#ifndef __PRIS_PREFIX
# define __PRIS_PREFIX "z"
#endif

#ifndef PRIdS
# define PRIdS __PRIS_PREFIX "d"
#endif

#ifndef PRIxS
# define PRIxS __PRIS_PREFIX "x"
#endif

#ifndef PRIuS
# define PRIuS __PRIS_PREFIX "u"
#endif

#ifndef PRIXS
# define PRIXS __PRIS_PREFIX "X"
#endif

#ifndef PRIoS
# define PRIoS __PRIS_PREFIX "o"
#endif

#ifndef NLIB_ASSUME
# define NLIB_ASSUME(cond) switch (0) case 0: default: if (cond) ; else __builtin_unreachable()
#endif

#ifdef NLIB_NONNULL_ENABLED
# define NLIB_EINVAL_IFNULL(p)
#else
# define NLIB_EINVAL_IFNULL(p) if (!p) return EINVAL
#endif

#ifndef NLIB_C_INLINE
# define NLIB_C_INLINE __inline
#endif

//
// Error Type
//
typedef int errno_t;  // TR 24731-1

//
// stdint
//
#include <stdint.h>
#include <inttypes.h>

#ifdef __cplusplus
#ifdef __cpp_unicode_characters
typedef char16_t nlib_utf16_t;
typedef char32_t nlib_utf32_t;
#else
typedef uint16_t nlib_utf16_t;
typedef uint32_t nlib_utf32_t;
#endif
#else
// On some devenv, char16_t, char32_t not typedefed in uchar.h
// #if __has_include( <uchar.h> )
// #include <uchar.h>
// typedef char16_t nlib_utf16_t;
// typedef char32_t nlib_utf32_t;
// #else
typedef uint16_t nlib_utf16_t;
typedef uint32_t nlib_utf32_t;
// #endif
#endif

typedef char nlib_utf8_t;

#if defined(__cplusplus) && !defined(NLIB_DOXYGEN)
#if __cplusplus >= 201703L || (defined(_HAS_STD_BYTE) && _HAS_STD_BYTE != 0)
typedef ::std::byte nlib_byte_t;
#elif __cplusplus >= 201103L || defined(_MSC_VER)
enum class nlib_byte_t : unsigned char {};
#else
typedef unsigned char nlib_byte_t;
#endif
#else
typedef unsigned char nlib_byte_t;
#endif

#if defined(__LP64__) && __LP64__ == 1
// if long, unsinged long are 64 bits long
#define NLIB_LP64
typedef int64_t nlib_long_compatible_t;
typedef uint64_t nlib_ulong_compatible_t;
#else
typedef int32_t nlib_long_compatible_t;
typedef uint32_t nlib_ulong_compatible_t;
#endif

#ifndef NLIB_VIS_PUBLIC_ALT
#define NLIB_VIS_PUBLIC_ALT NLIB_VIS_PUBLIC
#endif

#ifndef NLIB_EXPIMP_TEMPLATE
# define NLIB_EXPIMP_TEMPLATE(x) NLIB_STATIC_ASSERT(sizeof(char) == 1)
#endif

#ifdef __cplusplus
extern "C" {
#endif

NLIB_VIS_PUBLIC const char* nlib_error_string(errno_t e);

//
// Native error
//
NLIB_VIS_PUBLIC unsigned int nlib_get_native_last_error(void);

//
// Version
//

// returns NLIB_VERSION
NLIB_VIS_PUBLIC int nlib_getversion(void);
NLIB_VIS_PUBLIC int nlib_compiler_version(void);

// crc32 = 0 at the beginning
NLIB_VIS_PUBLIC uint32_t nlib_crc32(uint32_t crc32, const void* p, size_t n);
// crc32 = 0 at the beginning
NLIB_VIS_PUBLIC uint32_t nlib_crc32c(uint32_t crc32c, const void* p, size_t n);

//
// Atomic
//
#if defined(__INTELLISENSE__)
#define NLIB_ATOMIC_RELAXED (0)
#define NLIB_ATOMIC_ACQUIRE (1)
#define NLIB_ATOMIC_RELEASE (2)
#define NLIB_ATOMIC_ACQ_REL (3)
#define NLIB_ATOMIC_SEQ_CST (7)

int32_t nlib_atomic_load32(const int32_t* ptr, int memorder);
void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder);
// *target = value, and returns the old value of *target
int32_t nlib_atomic_exchange32(int32_t* ptr,
                               int32_t val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchange32(int32_t* ptr,
                                   int32_t* expected,
                                   int32_t desired,
                                   int weak,
                                   int success_memorder,
                                   int failure_memorder);
// *ptr += val; return *ptr;
int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr -= val; return *ptr;
int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr &= val; return *ptr;
int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr ^= val; return *ptr;
int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr |= val; return *ptr;
int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr += val; return tmp;
int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr -= val; return tmp;
int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr &= val; return tmp;
int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr ^= val; return tmp;
int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr |= val; return tmp;
int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder);

int64_t nlib_atomic_load64(const int64_t* ptr, int memorder);
void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder);
// *target = value, and returns the old value of *target
int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                   int64_t desired, int weak,
                                   int success_memorder, int failure_memorder);
// *ptr += val; return *ptr;
int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr -= val; return *ptr;
int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr &= val; return *ptr;
int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr ^= val; return *ptr;
int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr |= val; return *ptr;
int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr += val; return tmp;
int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr -= val; return tmp;
int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr &= val; return tmp;
int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr ^= val; return tmp;
int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr |= val; return tmp;
int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder);

void* nlib_atomic_loadptr(void* const* ptr, int memorder);
void nlib_atomic_storeptr(void** ptr, void* val, int memorder);
void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchangeptr(void** ptr, void** expected, void* desired,
                                    int weak, int success_memorder, int failure_memorder);

void nlib_atomic_thread_fence(int memorder);
#endif

//
// Time, Duration
//
#ifndef NLIB_TIMESPEC_HAS_NATIVE
struct timespec {
    time_t tv_sec;
    long tv_nsec;
};
#endif

// 100ns => 1, 1970/01/01 == 0
typedef int64_t nlib_time;
// 100ns => 1, 1ms => 10000
typedef int64_t nlib_duration;

// 100ns => 1, 1970/01/01 == 0
NLIB_VIS_PUBLIC errno_t nlib_epochtime(nlib_time* t) NLIB_NONNULL;
// 100ns => 1, boot time == 0, msec = *t / 10000
NLIB_VIS_PUBLIC errno_t nlib_ticktime(nlib_duration* t) NLIB_NONNULL;
// 100ns => 1, sleep 1 msec = nlib_sleep(10000)
NLIB_VIS_PUBLIC errno_t nlib_sleep(nlib_duration t);

#define NLIB_TO_TIMESPEC(tm, t) \
    (tm)->tv_sec = (time_t)((t) / (1000 * 10000)); \
    (tm)->tv_nsec = ((long)((t) % (1000 * 10000)) * 100)

#define NLIB_FROM_TIMESPEC(tm, t) \
    ((t) = (nlib_time)((tm)->tv_sec) * (1000 * 10000) + ((tm)->tv_nsec / 100))

#define NLIB_TO_SEC_NSEC(sec, nsec, t) \
    (sec) = (int64_t)((t) / (1000 * 10000)); \
    (nsec) = ((uint32_t)((t) % (1000 * 10000)) * 100)

#define NLIB_FROM_SEC_NSEC(sec, nsec, t) \
    ((t) = (nlib_time)(sec) * (1000 * 10000) + ((nsec) / 100))

static NLIB_C_INLINE errno_t nlib_epochtime_timespec(struct timespec* tm) {
    nlib_time t;
    errno_t e = nlib_epochtime(&t);
    if (NLIB_UNLIKELY(e != 0)) return e;
    NLIB_TO_TIMESPEC(tm, t);
    return 0;
}

static NLIB_C_INLINE errno_t nlib_ticktime_timespec(struct timespec* tm) {
    nlib_duration d;
    errno_t e = nlib_ticktime(&d);
    if (NLIB_UNLIKELY(e != 0)) return e;
    NLIB_TO_TIMESPEC(tm, d);
    return 0;
}

static NLIB_C_INLINE errno_t nlib_sleep_timespec(const struct timespec* tm) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_sleep(d);
}

#if !defined(NLIB_TIMER_HAS_NATIVE)
typedef uint32_t nlib_timer;
#elif defined(_MSC_VER)
typedef HANDLE nlib_timer;
#elif defined(__linux__)
typedef int nlib_timer;
#else
# error sorry
#endif
typedef void (*nlib_timer_callback)(nlib_timer timer, void* param);
struct nlib_timerspec_ {
    nlib_duration due_time;
    nlib_duration interval;
};
typedef struct nlib_timerspec_ nlib_timerspec;
#ifdef NLIB_DOXYGEN
struct nlib_timerspec {
    nlib_duration due_time;
    nlib_duration interval;
};
#endif
NLIB_VIS_PUBLIC errno_t nlib_timer_create(nlib_timer* timer, nlib_timer_callback callback,
                                          void* param, uint32_t flags);
NLIB_VIS_PUBLIC errno_t nlib_timer_settime(nlib_timer timer, const nlib_timerspec* new_value,
                                           nlib_timerspec* old_value);
NLIB_VIS_PUBLIC errno_t nlib_timer_gettime(nlib_timer timer, nlib_timerspec* curr_value);
NLIB_VIS_PUBLIC errno_t nlib_timer_delete(nlib_timer timer,
                                          int wait_completion,
                                          nlib_timer_callback completion_callback);
#define NLIB_TIMER_SHORTTERM_TASK 0x00000001
#define NLIB_TIMER_LONGTERM_TASK 0x00000002

//
// Random
//

// Store 'size' bytes of random values on 'buf'
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_gen_random(void* buf, size_t size) NLIB_NONNULL;

//
// Virtual Memory, Physical Memory
//
NLIB_VIS_PUBLIC errno_t nlib_mempagesize(size_t* size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_virtual_alloc(void** ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_virtual_free(void* ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_physical_alloc(void* ptr, size_t size, int prot) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_physical_free(void* ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mprotect(void* ptr, size_t size, int prot) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mlock(void* addr, size_t len) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_munlock(void* addr, size_t len) NLIB_NONNULL;

#define NLIB_PHYSICAL_ALLOC_PROT_NONE 0
#define NLIB_PHYSICAL_ALLOC_PROT_READ 1
#define NLIB_PHYSICAL_ALLOC_PROT_WRITE 2
#define NLIB_PHYSICAL_ALLOC_PROT_EXEC 4

//
// TLS
//
typedef void (*nlib_tls_destructor)(void* tls_value);
#define NLIB_TLS_INVALID (nlib_tls)(-1)

NLIB_CHECK_RESULT
#ifdef NLIB_PTHREAD_nlib_tls_alloc
static
#else
NLIB_VIS_PUBLIC
#endif
// code snippets:
//   # map tls_key on thread local storage
//     nlib_tls tls_key;
//     e = nlib_tls_alloc(&tls_key, NULL);  # no dtor invoked if destr is NULL
//     if (e != 0) { error ... }
//   # unmap tls_key
//     nlib_tls_free(tls_key);
//   # access from a thread
//     void* thread_local_value;
//     nlib_tls_getvalue(tls_key, &thread_local_value);
//     # use and update thread_local_value
//     nlib_tls_setvalue(tls_key, thread_local_value);
errno_t nlib_tls_alloc(nlib_tls* tls, nlib_tls_destructor destr) NLIB_NONNULL_1;
#ifdef NLIB_PTHREAD_nlib_tls_alloc
static NLIB_C_INLINE errno_t nlib_tls_alloc(nlib_tls* tls, nlib_tls_destructor destr) {
#ifndef __NX__
    return pthread_key_create(tls, destr);
#else
    errno_t e = pthread_key_create(tls, destr);
    if (e == 0) {
        // NINTENDOSDK-5089 workaround
        (void)pthread_setspecific(*tls, NULL);
    }
    return e;
#endif
}
#endif
#ifdef NLIB_PTHREAD_nlib_tls_free
static NLIB_C_INLINE errno_t nlib_tls_free(nlib_tls tls) {
    return pthread_key_delete(tls);
}
#else
NLIB_VIS_PUBLIC errno_t nlib_tls_free(nlib_tls tls);
#endif
#ifdef NLIB_PTHREAD_nlib_tls_setvalue
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_tls_setvalue(nlib_tls tls, const void* value);
#ifdef NLIB_PTHREAD_nlib_tls_setvalue
static NLIB_C_INLINE errno_t nlib_tls_setvalue(nlib_tls tls, const void* value) {
    return pthread_setspecific(tls, value);
}
#endif

#ifdef NLIB_PTHREAD_nlib_tls_getvalue
static
#else
NLIB_VIS_PUBLIC
#endif

errno_t nlib_tls_getvalue(nlib_tls tls, void** value) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_tls_getvalue
static NLIB_C_INLINE errno_t nlib_tls_getvalue(nlib_tls tls, void** value) {
    *value = pthread_getspecific(tls);
    return 0;
}
#endif

//
// Mutex
//

#ifdef NLIB_PTHREAD_nlib_mutex_init
static
#else
NLIB_VIS_PUBLIC
#endif

// you can use NLIB_MUTEX_INITIALIZER static initializer
errno_t nlib_mutex_init(nlib_mutex* mutex) NLIB_NONNULL NLIB_EXCLUDES(*mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_init
static NLIB_C_INLINE errno_t nlib_mutex_init(nlib_mutex* mutex) NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_mutex_init(mutex, NULL);
}
#endif

// you can use NLIB_RECURSIVE_MUTEX_INITIALIZER static initializer
NLIB_VIS_PUBLIC errno_t nlib_mutex_recursive_init(nlib_mutex* mutex)
    NLIB_NONNULL NLIB_EXCLUDES(*mutex);
// you can use NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER static initializer
NLIB_VIS_PUBLIC errno_t nlib_mutex_recursive_timed_init(nlib_mutex* mutex)
    NLIB_NONNULL NLIB_EXCLUDES(*mutex);

#ifdef NLIB_PTHREAD_nlib_mutex_lock
static
#else
NLIB_VIS_PUBLIC
#endif
// code snippets:
//   nlib_mutex m;
//   if (nlib_mutex_init(&m) != 0) { ... } # always returns 0 on almost all platforms?
//   nlib_mutex_lock(&m);
//      ....
//   nlib_mutex_unlock(&m);
//   nlib_mutex_destroy(&m);
errno_t nlib_mutex_lock(nlib_mutex* mutex) NLIB_NONNULL NLIB_ACQUIRE(*mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_lock
static NLIB_C_INLINE errno_t nlib_mutex_lock(nlib_mutex* mutex) NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_mutex_lock(mutex);
}
#endif

// returns EBUSY if a lock cannot be acquired
NLIB_CHECK_RESULT
#ifdef NLIB_PTHREAD_nlib_mutex_trylock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_mutex_trylock(nlib_mutex* mutex) NLIB_NONNULL NLIB_TRY_ACQUIRE(0, *mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_trylock
static NLIB_C_INLINE errno_t nlib_mutex_trylock(nlib_mutex* mutex) NLIB_TRY_ACQUIRE(0, *mutex) {
    return pthread_mutex_trylock(mutex);
}
#endif
// returns ETIMEDOUT if timeout
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mutex_trylock_for(nlib_mutex* mutex,
                               nlib_duration delta) NLIB_NONNULL NLIB_TRY_ACQUIRE(0, *mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_unlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_mutex_unlock(nlib_mutex* mutex) NLIB_NONNULL NLIB_RELEASE(*mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_unlock
static NLIB_C_INLINE errno_t nlib_mutex_unlock(nlib_mutex* mutex) NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_mutex_unlock(mutex);
}
#endif

#ifdef NLIB_PTHREAD_nlib_mutex_destroy
static
#else
NLIB_VIS_PUBLIC
#endif
// don't forget to write this, some platforms require this called.
errno_t nlib_mutex_destroy(nlib_mutex* mutex) NLIB_NONNULL NLIB_EXCLUDES(*mutex);
#ifdef NLIB_PTHREAD_nlib_mutex_destroy
static NLIB_C_INLINE errno_t nlib_mutex_destroy(nlib_mutex* mutex) NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_mutex_destroy(mutex);
}
#endif

static NLIB_C_INLINE errno_t nlib_mutex_trylock_for_timespec(nlib_mutex* mutex,
                                                             const struct timespec* tm)
    NLIB_TRY_ACQUIRE(0, *mutex) {
    nlib_duration delta;
    NLIB_FROM_TIMESPEC(tm, delta);
    return nlib_mutex_trylock_for(mutex, delta);
}

//
// Semaphore
//

NLIB_VIS_PUBLIC errno_t nlib_semaphore_init(nlib_semaphore* sem, int initial_count) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_wait(nlib_semaphore* sem) NLIB_NONNULL;
// returns EAGAIN if semaphore cannot be acquired
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_semaphore_trywait(nlib_semaphore* sem) NLIB_NONNULL;
// returns ETIMEDOUT if timeout
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_semaphore_trywait_for(
    nlib_semaphore* sem, nlib_duration duration) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_post(nlib_semaphore* __restrict sem,
                                            int* __restrict previous_count) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_post_ex(nlib_semaphore* __restrict sem, int release_count,
                                               int* __restrict previous_count) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_destroy(nlib_semaphore* sem) NLIB_NONNULL;

static NLIB_C_INLINE errno_t nlib_semaphore_trywait_for_timespec(
    nlib_semaphore* sem, const struct timespec* tm) {
    nlib_duration duration;
    NLIB_FROM_TIMESPEC(tm, duration);
    return nlib_semaphore_trywait_for(sem, duration);
}

//
// Condition Variable
//

#ifdef NLIB_PTHREAD_nlib_cond_init
static
#else
NLIB_VIS_PUBLIC
#endif

// you can use NLIB_COND_INITIALIZER for static initializer
errno_t nlib_cond_init(nlib_cond* cond) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_cond_init
static NLIB_C_INLINE errno_t nlib_cond_init(nlib_cond* cond) {
    return pthread_cond_init(cond, NULL);
}
#endif

#ifdef NLIB_PTHREAD_nlib_cond_signal
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_cond_signal(nlib_cond* cond) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_cond_signal
static NLIB_C_INLINE errno_t nlib_cond_signal(nlib_cond* cond) {
    return pthread_cond_signal(cond);
}
#endif

#ifdef NLIB_PTHREAD_nlib_cond_broadcast
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_cond_broadcast(nlib_cond* cond) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_cond_broadcast
static NLIB_C_INLINE errno_t nlib_cond_broadcast(nlib_cond* cond) {
    return pthread_cond_broadcast(cond);
}
#endif

#ifdef NLIB_PTHREAD_nlib_cond_wait
static
#else
NLIB_VIS_PUBLIC
#endif
// code snippets:
//   Initialization:
//     bool flag = false;
//     nlib_mutex m;
//     nlib_cond cond;
//     nlib_mutex_init(&m);
//     nlib_cond_init(&cond);
//   Thread1:
//     nlib_mutex_lock(&m);
//     while (!flag)
//       e = nlib_cond_wait(&cond, &m);  # m to be unlocked in nlib_cond_wait
//       # note that nlib_cond_wait may return without signal notified
//       if (e != 0) { error .... }
//     # do job and reset flag
//     flag = false;
//     nlib_mutex_unlock(&m);
//   Thread2:
//     nlib_mutex_lock(&m);
//     flag = true;
//     nlib_cond_broadcast(&cond);
//     nlib_mutex_unlock(&m);
errno_t nlib_cond_wait(nlib_cond* __restrict cond, nlib_mutex* __restrict mutex)
    NLIB_NONNULL NLIB_REQUIRES(*mutex);
#ifdef NLIB_PTHREAD_nlib_cond_wait
static NLIB_C_INLINE
errno_t nlib_cond_wait(nlib_cond* __restrict cond, nlib_mutex* __restrict mutex) {
    return pthread_cond_wait(cond, mutex);
}
#endif

// returns ETIMEDOUT if timeout, and see baloon for nlib_cond_wait()
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_cond_wait_for(nlib_cond* __restrict cond,
                           nlib_mutex* __restrict mutex,
                           nlib_duration duration) NLIB_NONNULL NLIB_REQUIRES(*mutex);
// returns ETIMEDOUT if timeout, and see baloon for nlib_cond_wait()
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_cond_wait_until(nlib_cond* __restrict cond,
                             nlib_mutex* __restrict mutex,
                             nlib_time abstime) NLIB_NONNULL NLIB_REQUIRES(*mutex);

#ifdef NLIB_PTHREAD_nlib_cond_destroy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_cond_destroy(nlib_cond* cond) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_cond_destroy
static NLIB_C_INLINE errno_t nlib_cond_destroy(nlib_cond* cond) {
    return pthread_cond_destroy(cond);
}
#endif

static NLIB_C_INLINE
errno_t nlib_cond_wait_for_timespec(nlib_cond* cond, nlib_mutex* mutex, const struct timespec* tm)
    NLIB_REQUIRES(*mutex) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_cond_wait_for(cond, mutex, d);
}

static NLIB_C_INLINE
errno_t nlib_cond_wait_until_timespec(nlib_cond* cond, nlib_mutex* mutex,
                                      const struct timespec* tm) NLIB_REQUIRES(*mutex) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_cond_wait_until(cond, mutex, d);
}

//
// Read/Write lock
//
#ifndef NLIB_RWLOCK_HAS_NATIVE
struct nlib_rwlock_ {
    int32_t _0[3];
    nlib_mutex _1[2];
    nlib_cond _2;
};
NLIB_CAPABILITY("mutex")
typedef struct nlib_rwlock_ nlib_rwlock;

#define NLIB_RWLOCK_INITIALIZER {                                                       \
    { 0, 0, 0 },                                                                        \
    { NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER, NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER }, \
      NLIB_COND_INITIALIZER }
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_init
static
#else
NLIB_VIS_PUBLIC
#endif

errno_t nlib_rwlock_init(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_EXCLUDES(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_init
static NLIB_C_INLINE errno_t nlib_rwlock_init(nlib_rwlock* rwlock) {
    return pthread_rwlock_init(rwlock, NULL);
}
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_destroy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_destroy(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_EXCLUDES(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_destroy
static NLIB_C_INLINE errno_t nlib_rwlock_destroy(nlib_rwlock* rwlock) {
    return pthread_rwlock_destroy(rwlock);
}
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_rdlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_rdlock(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_ACQUIRE_SHARED(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_rdlock
static NLIB_C_INLINE errno_t nlib_rwlock_rdlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_rdlock(rwlock);
}
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_tryrdlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_tryrdlock(nlib_rwlock* rwlock)
    NLIB_NONNULL NLIB_TRY_ACQUIRE_SHARED(0, *rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_tryrdlock
static NLIB_C_INLINE errno_t nlib_rwlock_tryrdlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_tryrdlock(rwlock);
}
#endif

NLIB_VIS_PUBLIC
errno_t nlib_rwlock_tryrdlock_for(nlib_rwlock* rwlock, nlib_duration duration)
    NLIB_NONNULL NLIB_TRY_ACQUIRE_SHARED(0, *rwlock);
NLIB_VIS_PUBLIC
errno_t nlib_rwlock_tryrdlock_until(nlib_rwlock* rwlock, nlib_time abstime)
    NLIB_NONNULL NLIB_TRY_ACQUIRE_SHARED(0, *rwlock);

#ifdef NLIB_PTHREAD_nlib_rwlock_rdunlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_rdunlock(nlib_rwlock* rwlock)
    NLIB_NONNULL NLIB_RELEASE_SHARED(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_rdunlock
static NLIB_C_INLINE errno_t nlib_rwlock_rdunlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_unlock(rwlock);
}
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_wrlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_wrlock(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_ACQUIRE(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_wrlock
static NLIB_C_INLINE errno_t nlib_rwlock_wrlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_wrlock(rwlock);
}
#endif

#ifdef NLIB_PTHREAD_nlib_rwlock_trywrlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_trywrlock(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_TRY_ACQUIRE(0, *rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_trywrlock
static NLIB_C_INLINE errno_t nlib_rwlock_trywrlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_trywrlock(rwlock);
}
#endif

NLIB_VIS_PUBLIC
errno_t nlib_rwlock_trywrlock_for(nlib_rwlock* rwlock, nlib_duration duration)
    NLIB_NONNULL NLIB_TRY_ACQUIRE(0, *rwlock);
NLIB_VIS_PUBLIC
errno_t nlib_rwlock_trywrlock_until(nlib_rwlock* rwlock, nlib_time abstime)
    NLIB_NONNULL NLIB_TRY_ACQUIRE(0, *rwlock);

#ifdef NLIB_PTHREAD_nlib_rwlock_wrunlock
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_rwlock_wrunlock(nlib_rwlock* rwlock) NLIB_NONNULL NLIB_RELEASE(*rwlock);
#ifdef NLIB_PTHREAD_nlib_rwlock_wrunlock
static NLIB_C_INLINE errno_t nlib_rwlock_wrunlock(nlib_rwlock* rwlock)
    NLIB_NO_THREAD_SAFETY_ANALYSIS {
    return pthread_rwlock_unlock(rwlock);
}
#endif

static NLIB_C_INLINE
errno_t nlib_rwlock_tryrdlock_for_timespec(nlib_rwlock* rwlock, const struct timespec* tm)
    NLIB_TRY_ACQUIRE_SHARED(0, *rwlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_rwlock_tryrdlock_for(rwlock, d);
}

static NLIB_C_INLINE
errno_t nlib_rwlock_tryrdlock_until_timespec(nlib_rwlock* rwlock, const struct timespec* tm)
    NLIB_TRY_ACQUIRE_SHARED(0, *rwlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_rwlock_tryrdlock_until(rwlock, d);
}

static NLIB_C_INLINE
errno_t nlib_rwlock_trywrlock_for_timespec(nlib_rwlock* rwlock, const struct timespec* tm)
    NLIB_TRY_ACQUIRE(0, *rwlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_rwlock_trywrlock_for(rwlock, d);
}

static NLIB_C_INLINE
errno_t nlib_rwlock_trywrlock_until_timespec(nlib_rwlock* rwlock, const struct timespec* tm)
    NLIB_TRY_ACQUIRE(0, *rwlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_rwlock_trywrlock_until(rwlock, d);
}


#if defined(_MSC_VER) && defined(NLIB_RWLOCK_HAS_NATIVE)
typedef struct nlib_condrwlock_ {
    CONDITION_VARIABLE cond;
} nlib_condrwlock;
#define NLIB_CONDRWLOCK_INITIALIZER { CONDITION_VARIABLE_INIT }
#else
typedef struct nlib_condrwlock_ {
    nlib_cond cond;
    nlib_mutex mutex;
} nlib_condrwlock;
#define NLIB_CONDRWLOCK_INITIALIZER { NLIB_COND_INITIALIZER, NLIB_MUTEX_INITIALIZER }
#endif

NLIB_VIS_PUBLIC errno_t nlib_condrwlock_init(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_destroy(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_signal(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_broadcast(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait(nlib_condrwlock* __restrict cond,
                                             nlib_rwlock* __restrict rwlock,
                                             int rdlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait_for(nlib_condrwlock* __restrict cond,
                                                 nlib_rwlock* __restrict rwlock,
                                                 nlib_duration duration,
                                                 int rdlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait_until(nlib_condrwlock* __restrict cond,
                                                   nlib_rwlock* __restrict rwlock,
                                                   nlib_time abstime,
                                                   int rdlock) NLIB_NONNULL;

static NLIB_C_INLINE errno_t nlib_condrwlock_wait_for_timespec(nlib_condrwlock* cond,
                                                               nlib_rwlock* rwlock,
                                                               const struct timespec* tm,
                                                               int rdlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_condrwlock_wait_for(cond, rwlock, d, rdlock);
}

static NLIB_C_INLINE errno_t nlib_condrwlock_wait_until_timespec(nlib_condrwlock* cond,
                                                                 nlib_rwlock* rwlock,
                                                                 const struct timespec* tm,
                                                                 int rdlock) {
    nlib_duration d;
    NLIB_FROM_TIMESPEC(tm, d);
    return nlib_condrwlock_wait_until(cond, rwlock, d, rdlock);
}

//
// Barrier
//
#ifndef NLIB_BARRIER_HAS_NATIVE
struct nlib_barrier_ {
    nlib_mutex _0;
    nlib_cond _1;
    unsigned int _2[3];
};
typedef struct nlib_barrier_ nlib_barrier;
#endif

#ifdef NLIB_PTHREAD_nlib_barrier_init
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_barrier_init(nlib_barrier* barrier, unsigned int count) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_barrier_init
static NLIB_C_INLINE errno_t nlib_barrier_init(nlib_barrier* barrier, unsigned int count) {
    return pthread_barrier_init(barrier, NULL, count);
}
#endif

#ifdef NLIB_PTHREAD_nlib_barrier_destroy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_barrier_destroy(nlib_barrier* barrier) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_barrier_destroy
static NLIB_C_INLINE errno_t nlib_barrier_destroy(nlib_barrier* barrier) {
    return pthread_barrier_destroy(barrier);
}
#endif

NLIB_VIS_PUBLIC errno_t nlib_barrier_wait(nlib_barrier* barrier) NLIB_NONNULL;

//
// Once
//
#ifndef NLIB_ONCE_HAS_NATIVE
struct nlib_onceflag_ {
    int status;
};
typedef struct nlib_onceflag_ nlib_onceflag;
#define NLIB_ONCE_INIT { 0 }
typedef void (*nlib_oncefunc)(void);

// code snippets
//   void OnceFunc() { .... }
//   nlib_onceflag flag = NLIB_ONCE_INIT;  // should be static initialized
//   nlib_once(&flag, OnceFunc);  // OnceFunc executes only once
NLIB_VIS_PUBLIC errno_t nlib_once(nlib_onceflag* flag,
                                  nlib_oncefunc func) NLIB_NONNULL;
#elif defined(_MSC_VER)
typedef INIT_ONCE nlib_onceflag;
#define NLIB_ONCE_INIT INIT_ONCE_STATIC_INIT
typedef void (*nlib_oncefunc)(void);
NLIB_VIS_PUBLIC errno_t nlib_once(nlib_onceflag* flag, nlib_oncefunc func);
#elif defined(__APPLE__) || defined(__FreeBSD__)
typedef dispatch_once_t nlib_onceflag;
#define NLIB_ONCE_INIT 0
typedef void (*nlib_oncefunc)(void);
NLIB_VIS_PUBLIC errno_t nlib_once(nlib_onceflag* flag, nlib_oncefunc func) NLIB_NONNULL;
#else
typedef pthread_once_t nlib_onceflag;
#define NLIB_ONCE_INIT PTHREAD_ONCE_INIT
typedef void (*nlib_oncefunc)(void);
static errno_t nlib_once(nlib_onceflag* flag, nlib_oncefunc func) NLIB_NONNULL;
static NLIB_C_INLINE errno_t nlib_once(nlib_onceflag* flag, nlib_oncefunc func) {
    return pthread_once(flag, func);
}
#endif

//
// Message Queue
//
#ifdef NLIB_DOXYGEN
typedef int32_t nlib_mq;
#else
typedef struct nlib_mq_ {
    int32_t raw_handle;  // 0 for invalid handle
} nlib_mq;
#endif
typedef void* nlib_mq_msg;

#define NLIB_MQ_BLOCK 0
#define NLIB_MQ_NONBLOCK 1
#define NLIB_MQ_LOCKFREE 2

typedef void (*nlib_mq_msg_destructor)(nlib_mq_msg);
#ifdef NLIB_DOXYGEN
struct nlib_mq_attr {
    int32_t flag;
    int32_t max_msg;
    int32_t cur_msg;
    nlib_mq_msg_destructor destructor;
};
#else
typedef struct nlib_mq_attr_ {
    int32_t flag;  // NLIB_MQ_BLOCK / NLIB_MQ_NONBLOCK
    int32_t max_msg;  // 0 for 128
    int32_t cur_msg;  // blocking mode only
    nlib_mq_msg_destructor destructor;
} nlib_mq_attr;
#endif

NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_open(nlib_mq* mq, const nlib_mq_attr* attr) NLIB_NONNULL;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_getattr(nlib_mq mq, nlib_mq_attr* attr) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mq_close(nlib_mq mq);
NLIB_VIS_PUBLIC errno_t nlib_mq_readonly(nlib_mq mq);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_mq_send(nlib_mq mq, nlib_mq_msg msg, int prio);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_send_until(nlib_mq mq, nlib_mq_msg msg, int prio, nlib_time abstime);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_receive(nlib_mq mq, nlib_mq_msg* msg, int* prio) NLIB_NONNULL_2;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_receive_until(nlib_mq mq, nlib_mq_msg* msg, int* prio, nlib_time abstime)
    NLIB_NONNULL_2;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_mq_drop(nlib_mq mq, nlib_mq_msg* msg, int* prio) NLIB_NONNULL_2;

//
// Thread
//
NLIB_VIS_PUBLIC errno_t nlib_yield(void);
static NLIB_ALWAYS_INLINE void nlib_pause(void) {
#if defined(__x86_64__) || defined(__i386__) || defined(_M_IX86) || defined(_M_AMD64)
    _mm_pause();
#elif defined(__ARM_ACLE)
    __yield();
#else
    (void)nlib_yield();
#endif
}

#define NLIB_THREAD_INVALID (nlib_thread)(0)

#ifndef NLIB_SPINLOCK_HAS_NATIVE
typedef int32_t nlib_spinlock;
#endif

#ifndef NLIB_THREAD_ATTR_HAS_NATIVE
struct nlib_thread_attr_ {
    nlib_spinlock spin;
    int detach_state;
    int explicit_sched;
    int priority;
    uint32_t affinity;
    void* stack_addr;
    size_t stack_size;
};
typedef struct nlib_thread_attr_ nlib_thread_attr;
#else
struct nlib_thread_attr_ {
    pthread_attr_t attr;
    uint32_t affinity;
};
typedef struct nlib_thread_attr_ nlib_thread_attr;
#endif
typedef void (*nlib_thread_func)(void* arg);
// -1 for invalid
typedef int nlib_thread_id;

// code snippets:
//   nlib_thread th;
//   if ((e = nlib_thread_create(&th, NULL, myfunc, myarg)) != 0) { error ... }
//   nlib_thread_join(th); # or nlib_thread_detach(th);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_thread_create(nlib_thread* __restrict thread, const nlib_thread_attr* __restrict attr,
                           nlib_thread_func func, void* __restrict arg)
    NLIB_NONNULL_1 NLIB_NONNULL_3;
#ifdef NLIB_PTHREAD_nlib_thread_join
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_thread_join(nlib_thread thread);
#ifdef NLIB_PTHREAD_nlib_thread_join
static NLIB_C_INLINE errno_t nlib_thread_join(nlib_thread thread) {
    return pthread_join(thread, NULL);
}
#endif

#ifdef NLIB_PTHREAD_nlib_thread_detach
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_thread_detach(nlib_thread thread);
#ifdef NLIB_PTHREAD_nlib_thread_detach
static NLIB_C_INLINE errno_t nlib_thread_detach(nlib_thread thread) {
    return pthread_detach(thread);
}
#endif

#ifdef NLIB_PTHREAD_nlib_thread_self
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_thread_self(nlib_thread* thread) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_thread_self
static NLIB_C_INLINE errno_t nlib_thread_self(nlib_thread* thread) {
    *thread = pthread_self();
    return 0;
}
#endif

NLIB_VIS_PUBLIC errno_t nlib_thread_getconcurrency(unsigned int* num_cpu) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_getid(nlib_thread_id* id) NLIB_NONNULL;

#ifdef NLIB_PTHREAD_nlib_thread_equal
static
#else
NLIB_VIS_PUBLIC
#endif
int nlib_thread_equal(nlib_thread th1, nlib_thread th2);
#ifdef NLIB_PTHREAD_nlib_thread_equal
static NLIB_C_INLINE int nlib_thread_equal(nlib_thread th1, nlib_thread th2) {
    return pthread_equal(th1, th2);
}
#endif

NLIB_VIS_PUBLIC errno_t nlib_thread_getcpu(int* result) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_setaffinity(nlib_thread thread, uint32_t affinity);
NLIB_VIS_PUBLIC errno_t nlib_thread_setname(nlib_thread thread, const char* name) NLIB_NONNULL;

#ifdef NLIB_PTHREAD_nlib_thread_getname
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_thread_getname(nlib_thread thread, char* name, size_t len) NLIB_NONNULL;
#ifdef NLIB_PTHREAD_nlib_thread_getname
static NLIB_C_INLINE errno_t nlib_thread_getname(nlib_thread thread, char* name, size_t len) {
    return pthread_getname_np(thread, name, len);
}
#endif

// NOTE:
// win32 does not have GetThreadAffinityMask()
// errno_t nlib_thread_get_affinify(nlib_thread thread, uint32_t* affinity);

NLIB_VIS_PUBLIC errno_t nlib_thread_attr_init(nlib_thread_attr* attr) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_setint(nlib_thread_attr* attr, int key, int value) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_getint(const nlib_thread_attr* __restrict attr, int key,
                                int* __restrict value) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_setptr(nlib_thread_attr* __restrict attr, int key,
                                void* __restrict value) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_getptr(const nlib_thread_attr* __restrict attr, int key,
                                void** __restrict value) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_setstack(nlib_thread_attr* __restrict attr, void* __restrict stack_addr,
                                  size_t stack_size) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_thread_attr_getstack(const nlib_thread_attr* __restrict attr,
                                  void** __restrict stack_addr, size_t* __restrict stack_size)
    NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_destroy(nlib_thread_attr* attr) NLIB_NONNULL;

#define NLIB_THREAD_ATTR_KEY_DETACHSTATE    (1)
#define NLIB_THREAD_ATTR_KEY_STACKSIZE      (2)
#define NLIB_THREAD_ATTR_KEY_PRIORITY       (4)
#define NLIB_THREAD_ATTR_KEY_AFFINITY       (5)
#define NLIB_THREAD_ATTR_KEY_EXPLICIT_SCHED (6)

NLIB_VIS_PUBLIC errno_t nlib_thread_getpriority(nlib_thread thread, int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_setpriority(nlib_thread thread, int priority);
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_min(int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_max(int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_default(int* priority) NLIB_NONNULL;

#ifndef NN_PLATFORM_CTR
// See also nlib_thread_exit_cpp();
NLIB_NORETURN NLIB_VIS_PUBLIC void nlib_thread_exit(void);
#endif

#ifdef NLIB_DOXYGEN
void nlib_thread_cleanup_push(void (*fn)(void*), void* arg);
void nlib_thread_cleanup_pop(int exec);
#elif defined(pthread_cleanup_push)
# define nlib_thread_cleanup_push(fn, arg) pthread_cleanup_push(fn, arg)
# define nlib_thread_cleanup_pop(exec) pthread_cleanup_pop(exec)
#elif !defined(NN_PLATFORM_CTR)
struct nlib_thread_cleanup_handler_ {
    void (*func)(void*);
    void* arg;
    struct nlib_thread_cleanup_handler_* next;
};
#define nlib_thread_cleanup_push(fn, arg) switch (0) case 0: default: { \
    struct nlib_thread_cleanup_handler_ _thread_cleanup_handler = { fn, arg, NULL }; \
    nlib_thread_cleanup_push_(&_thread_cleanup_handler)
#define nlib_thread_cleanup_pop(exec) nlib_thread_cleanup_pop_(exec); }

NLIB_VIS_PUBLIC void nlib_thread_cleanup_push_(struct nlib_thread_cleanup_handler_* handler);
NLIB_VIS_PUBLIC void nlib_thread_cleanup_pop_(int exec);
#endif

//
// Console/Debug
//

// note that buf is not null terminated
NLIB_VIS_PUBLIC
errno_t nlib_write_stdout(size_t* __restrict result, const void* __restrict buf, size_t count)
    NLIB_NONNULL;
// note that buf is not null terminated
NLIB_VIS_PUBLIC
errno_t nlib_write_stderr(size_t* __restrict result, const void* __restrict buf, size_t count)
    NLIB_NONNULL;
NLIB_VIS_PUBLIC void nlib_debug_break(void);
NLIB_VIS_PUBLIC
errno_t nlib_debug_backtrace(size_t* __restrict result, void** __restrict buffer, size_t count)
    NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_debug_backtrace_gettext(char* __restrict str, size_t strbufsize,
                                     void* const* __restrict buf, size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_getenv(size_t* __restrict result, char* __restrict buf, size_t bufsize,
                    const char* __restrict varname) NLIB_NONNULL_1 NLIB_NONNULL_4;

typedef enum nlib_log_priority {
    kNlibLogUnknown = 0,
    kNlibLogDefault,
    kNlibLogVerbose,
    kNlibLogDebug,
    kNlibLogInfo,
    kNlibLogWarn,
    kNlibLogError,
    kNlibLogFatal,
    kNlibLogSilent,
    kNlibLogLevelEqualOrAbove = 0x10,
    kNlibLogLevelEqualOrBelow = 0x20,
    kNlibLogLevelAll = 0x30
} nlib_log_priority;

typedef enum nlib_log_key {
    kNlibLogAttrUnknown = 0,
    kNlibLogAttrStdout,
    kNlibLogAttrStderr,
    kNlibLogAttrMsvcTrace,
    kNlibLogAttrSyslog,
    kNlibLogAttrNlibFd,
    kNlibLogAttrMax
} nlib_log_key;

#ifndef NLIB_ATTRIBUTE_PRINTF
# define NLIB_ATTRIBUTE_PRINTF(x, y) __attribute__((format(printf, x, y)))
#endif

NLIB_VIS_PUBLIC int nlib_log_print(int prio, _Printf_format_string_ const char* __restrict tag,
                                   const char* __restrict fmt, ...)
                                   NLIB_ATTRIBUTE_PRINTF(3, 4) NLIB_NONNULL;
NLIB_VIS_PUBLIC int nlib_log_vprint(int prio, _Printf_format_string_ const char* __restrict tag,
                                    const char* __restrict fmt, va_list ap) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_log_attr_setint(int prio, int key, int value);

//
// File Access
//
#ifndef NLIB_FD_O_RDONLY
# ifndef O_RDONLY
#  error
# endif
# define NLIB_FD_O_RDONLY O_RDONLY
#endif

#ifndef NLIB_FD_O_WRONLY
# ifndef O_WRONLY
#  error
# endif
# define NLIB_FD_O_WRONLY O_WRONLY
#endif

#ifndef NLIB_FD_O_RDWR
# ifndef O_RDWR
#  error
# endif
# define NLIB_FD_O_RDWR   O_RDWR
#endif

#ifndef NLIB_FD_O_APPEND
# ifndef O_APPEND
#  error
# endif
# define NLIB_FD_O_APPEND O_APPEND
#endif

#ifndef NLIB_FD_O_CREAT
# ifndef O_CREAT
#  error
# endif
# define NLIB_FD_O_CREAT  O_CREAT
#endif

#ifndef NLIB_FD_O_TRUNC
# ifndef O_TRUNC
#  error
# endif
# define NLIB_FD_O_TRUNC  O_TRUNC
#endif

#ifndef NLIB_FD_O_EXCL
# ifndef O_EXCL
#  error
# endif
# define NLIB_FD_O_EXCL   O_EXCL
#endif

#ifndef NLIB_SEEK_SET
# ifndef SEEK_SET
#  error
# endif
# define NLIB_SEEK_SET SEEK_SET
#endif

#ifndef NLIB_SEEK_CUR
# ifndef SEEK_CUR
#  error
# endif
# define NLIB_SEEK_CUR SEEK_CUR
#endif

// NOTE:
// SEEK_END not supported(because of FIO19-C)

typedef int64_t nlib_offset;
typedef int nlib_fd;
#define NLIB_FD_INVALID (-1)

#ifdef NLIB_DOXYGEN
errno_t nlib_fd_open(nlib_fd* fd, const char* native_path, unsigned int flags);
errno_t nlib_fd_open(nlib_fd* fd, const char* native_path, unsigned int flags, int mode);
#else
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_open(nlib_fd* fd, const char* native_path, unsigned int flags, ...) NLIB_NONNULL_1;
#endif
NLIB_CHECK_RESULT static NLIB_C_INLINE
errno_t nlib_fd_creat(nlib_fd* fd, const char* native_path, int mode) {
    return nlib_fd_open(fd, native_path,
        NLIB_FD_O_CREAT | NLIB_FD_O_WRONLY | NLIB_FD_O_EXCL, mode);
}
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_fd_close(nlib_fd fd);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_read(size_t* __restrict result, nlib_fd fd, void* __restrict buf, size_t count)
    NLIB_NONNULL_1;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_write(size_t* __restrict result, nlib_fd fd, const void* __restrict buf,
                      size_t count) NLIB_NONNULL_1;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_seek(nlib_offset* result, nlib_fd fd, nlib_offset offset, int whence)
     NLIB_NONNULL_1;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_pread(size_t* __restrict result, nlib_fd fd, void* __restrict buf,
                      size_t count, nlib_offset offset) NLIB_NONNULL_1;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_pwrite(size_t* __restrict result, nlib_fd fd, const void* __restrict buf,
                       size_t count, nlib_offset offset) NLIB_NONNULL_1;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_fd_truncate(nlib_fd fd, nlib_offset length);

NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_getsize(nlib_offset* size, nlib_fd fd) NLIB_NONNULL;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_fd_flush(nlib_fd fd);
NLIB_VIS_PUBLIC errno_t nlib_fd_sync(nlib_fd fd);
NLIB_VIS_PUBLIC errno_t nlib_fd_native_handle(void** native_handle, nlib_fd fd) NLIB_NONNULL;
// errno_t nlib_fd_fcntl_getflag(unsigned int* flags, nlib_fd fd);
// errno_t nlib_fd_fcntl_setflag(nlib_fd fd, unsigned int flags);

// Scatter/Gather buffer
#if !defined(NLIB_IOVEC_HAS_NATIVE)
struct nlib_fd_iovec_ {
    void* iov_base;
    size_t iov_len;
};
typedef struct nlib_fd_iovec_ nlib_fd_iovec;
#else
typedef struct iovec nlib_fd_iovec;
#endif
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_readv(size_t* __restrict result, nlib_fd fd, const nlib_fd_iovec* __restrict iov,
                      int iovcnt) NLIB_NONNULL;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_writev(size_t* __restrict result, nlib_fd fd, const nlib_fd_iovec* __restrict iov,
                       int iovcnt) NLIB_NONNULL;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_preadv(size_t* __restrict result, nlib_fd fd, const nlib_fd_iovec* __restrict iov,
                       int iovcnt, nlib_offset offset) NLIB_NONNULL;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_fd_pwritev(size_t* __restrict result, nlib_fd fd, const nlib_fd_iovec* __restrict iov,
                        int iovcnt, nlib_offset offset) NLIB_NONNULL;
// errno_t nlib_fd_stat(stat* stat, nlib_fd fd);

NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_unlink(const char* native_path);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_mkdir(const char* native_path,
                                                     unsigned int flags);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_rmdir(const char* native_path);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t nlib_remove(const char* native_path);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_rename(const char* __restrict old_path, const char* __restrict new_path);

struct nlib_dir_ {
    void* ptr;
};
typedef struct nlib_dir_ nlib_dir;
typedef struct nlib_dirent_ {
    uint32_t flags;  // 0: file, 1: directory
    char name[768];
} nlib_dirent;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_dir_open(nlib_dir* __restrict dir, const char* __restrict native_path) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_dir_close(nlib_dir dir);
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_dir_read(nlib_dirent* ent, nlib_dir dir) NLIB_NONNULL;

// CTR does not have getcwd.....
// errno_t nlib_getcwd(char* buf, size_t bufsize);
// errno_t nlib_chdir(const char* path);

NLIB_VIS_PUBLIC
errno_t nlib_is_dir(int* __restrict result, const char* __restrict native_path) NLIB_NONNULL_1;
// *result != 0 if exists
NLIB_VIS_PUBLIC
errno_t nlib_exist_path(int* __restrict result, const char* __restrict native_path) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC
errno_t nlib_disk_freespace(const char* __restrict native_path,
                            uint64_t* __restrict free_bytes_available,
                            uint64_t* __restrict total_bytes,
                            uint64_t* __restrict total_free_bytes);

NLIB_VIS_PUBLIC const char* nlib_basename(const char* path) NLIB_NONNULL;
NLIB_VIS_PUBLIC const char* nlib_dirname(size_t* len, const char* path) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mkostemps(nlib_fd* fd, char* templ, int suffixlen, int flags);

struct nlib_fileid_ {
#if defined(_MSC_VER)
    FILE_ID_INFO _;
#elif defined(NLIB_UNIX)
    dev_t _0;
    ino_t _1;
#else
    uint64_t _;
#endif
};
typedef struct nlib_fileid_ nlib_fileid;
NLIB_VIS_PUBLIC errno_t nlib_fd_fileid(nlib_fileid* result, nlib_fd fd);
NLIB_VIS_PUBLIC errno_t nlib_readlink(size_t* len, const char* native_path, char* buf,
                                      size_t bufsize);

//
// Socket(Win32/Linux/Cygwin only)
//
#if defined(_MSC_VER) || defined(NLIB_UNIX)
#include "nn/nlib/Platform_socket.h"
#endif

//
// errno workaround
//

// PLEASE DO NOT DEFINE POSIX.1-2008 errno macros in your code
// http://pubs.opengroup.org/onlinepubs/9699919799/
#ifdef _MSC_VER
#  ifdef EDQUOT
#   warning  Do not define EDQUOT. nlib may not work correctly.
#  endif
#endif

#ifndef NLIB_SKIP_ERRNO_DEFINE
#include <errno.h>  // for POSIX error values
#if !defined(__CYGWIN__) && !defined(__linux__) && !defined(__FreeBSD__) && !defined(__NX__)
#include "nn/nlib/Platform_errno.h"
#endif
#endif

//
// nlib_spinlock
//
#ifndef NLIB_SPINLOCK_HAS_NATIVE
#define NLIB_SPINLOCK_INITIALIZER (0)
NLIB_VIS_PUBLIC void nlib_spinlock_lock_(nlib_spinlock* lock) NLIB_NONNULL;
#ifndef NN_PLATFORM_CTR
NLIB_VIS_PUBLIC void nlib_spinlock_unlock_(nlib_spinlock* lock) NLIB_NONNULL;
#endif

static void nlib_spinlock_init(nlib_spinlock* lock) NLIB_NONNULL;
static void nlib_spinlock_lock(nlib_spinlock* lock) NLIB_NONNULL;
static errno_t nlib_spinlock_trylock(nlib_spinlock* lock) NLIB_NONNULL;
static void nlib_spinlock_unlock(nlib_spinlock* lock) NLIB_NONNULL;

static NLIB_ALWAYS_INLINE void nlib_spinlock_init(nlib_spinlock* lock) {
    *lock = 0;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_lock(nlib_spinlock* lock) {
#if defined(__arm__)
#if __has_builtin(__builtin_arm_ldrex)
    int R5 = __builtin_arm_ldrex(lock);
#else
    int R5 = __ldrex(lock);
#endif
    if (R5 == 0) {
#if __has_builtin(__builtin_arm_strex)
        if (__builtin_arm_strex(1, lock) == 0) {
#else
        if (__strex(1, lock) == 0) {
#endif
#if !defined(NN_PLATFORM_CTR)
            __dmb(0xf);
#else
            nlib_ctr_barrier();
#endif
        }
        return;
    }
    nlib_spinlock_lock_(lock);
#else
    int32_t expected = 0;
    if (!nlib_atomic_compare_exchange32(lock, &expected, 1, 1,
                                        NLIB_ATOMIC_ACQUIRE, NLIB_ATOMIC_RELAXED)) {
        nlib_spinlock_lock_(lock);
    }
#endif
}
static NLIB_ALWAYS_INLINE errno_t nlib_spinlock_trylock(nlib_spinlock* lock) {
#if defined(__arm__)
#if __has_builtin(__builtin_arm_ldrex)
    int R5 = __builtin_arm_ldrex(lock);
#else
    int R5 = __ldrex(lock);
#endif
    if (R5 == 0) {
#if __has_builtin(__builtin_arm_strex)
        if (__builtin_arm_strex(1, lock) == 0) {
#else
        if (__strex(1, lock) == 0) {
#endif
#if !defined(NN_PLATFORM_CTR)
            __dmb(0xf);
#else
            nlib_ctr_barrier();
#endif
            return 0;
        }
    }
    return EBUSY;
#else
    int32_t expected = 0;
    if (nlib_atomic_compare_exchange32(lock, &expected, 1, 0,
        NLIB_ATOMIC_ACQUIRE, NLIB_ATOMIC_RELAXED))
        return 0;
    else
        return EBUSY;
#endif
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_unlock(nlib_spinlock* lock) {
#if defined(NN_PLATFORM_CTR)
    nlib_atomic_store32(lock, 0, NLIB_ATOMIC_RELEASE);
#else
    int32_t expected = 1;
    if (!nlib_atomic_compare_exchange32(lock, &expected, 0, 0,
        NLIB_ATOMIC_RELEASE, NLIB_ATOMIC_RELAXED)) {
        NLIB_ASSUME(expected == 2);
        nlib_spinlock_unlock_(lock);
    }
#if defined(__arm__)
    __sev();
#endif
#endif
}
#endif

//
// Utilities(Safer style functions)
//
NLIB_VIS_PUBLIC
errno_t nlib_vsnprintf(size_t* __restrict count, char* __restrict buf, size_t size,
                       _Printf_format_string_ const char* __restrict fmt, va_list args)
    NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_snprintf(size_t* __restrict count, char* __restrict buf, size_t size,
                      _Printf_format_string_ const char* __restrict fmt, ...)
    NLIB_ATTRIBUTE_PRINTF(4, 5) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_vdprintf(nlib_fd fd, size_t* __restrict count,
                      _Printf_format_string_ const char* __restrict fmt, va_list args)
    NLIB_NONNULL_3;
NLIB_VIS_PUBLIC
errno_t nlib_dprintf(nlib_fd fd, size_t* __restrict count,
                     _Printf_format_string_ const char* __restrict fmt, ...)
    NLIB_ATTRIBUTE_PRINTF(3, 4) NLIB_NONNULL_3;
// fmt and string must be in UTF-8 even if you use Visual Studio
NLIB_VIS_PUBLIC
int nlib_printf(_Printf_format_string_ const char* fmt, ...)
    NLIB_ATTRIBUTE_PRINTF(1, 2) NLIB_NONNULL_1;

NLIB_VIS_PUBLIC
errno_t nlib_vsnwprintf(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                        _Printf_format_string_ const wchar_t* __restrict fmt, va_list args)
    NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_snwprintf(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                       _Printf_format_string_ const wchar_t* __restrict fmt, ...)
    NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_vdwprintf(nlib_fd fd, size_t* __restrict count,
                       _Printf_format_string_ const wchar_t* __restrict fmt, va_list args)
    NLIB_NONNULL_3;
NLIB_VIS_PUBLIC
errno_t nlib_dwprintf(nlib_fd fd, size_t* __restrict count,
                      _Printf_format_string_ const wchar_t* __restrict fmt, ...) NLIB_NONNULL_3;
NLIB_VIS_PUBLIC
int nlib_wprintf(_Printf_format_string_ const wchar_t* fmt, ...) NLIB_NONNULL_1;

NLIB_VIS_PUBLIC
errno_t nlib_vsnprintf_fallback(size_t* __restrict count, char* __restrict buf, size_t size,
                                _Printf_format_string_ const char* __restrict fmt, va_list args)
    NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_snprintf_fallback(size_t* __restrict count, char* __restrict buf,
                               size_t size, _Printf_format_string_ const char* __restrict fmt, ...)
    NLIB_ATTRIBUTE_PRINTF(4, 5) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_vsnwprintf_fallback(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                                 _Printf_format_string_ const wchar_t* __restrict fmt, va_list args)
    NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_snwprintf_fallback(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                                _Printf_format_string_ const wchar_t* __restrict fmt, ...)
    NLIB_NONNULL_4;

// http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1078.pdf
// http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1079.htm
// a bit different from Microsoft's strcpy_s or etc.
// returns ERANGE instead of EINVAL

static errno_t nlib_memcpy(void* __restrict s1, size_t s1max, const void* __restrict s2, size_t n)
    NLIB_NONNULL;
static errno_t nlib_memmove(void* s1, size_t s1max, const void* s2, size_t n) NLIB_NONNULL;
static errno_t nlib_memset(void* buf, int ch, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC
void* nlib_memccpy(void* __restrict dest, size_t dest_size, const void* __restrict src,
                   size_t src_size, int c) NLIB_NONNULL;

#ifdef NLIB_LIBC_nlib_memcmp
static
#else
NLIB_VIS_PUBLIC_ALT
#endif
int nlib_memcmp(const void* buf1, const void* buf2, size_t n) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_memcmp
static NLIB_C_INLINE int nlib_memcmp(const void* buf1, const void* buf2, size_t n) {
    return memcmp(buf1, buf2, n);
}
#endif

NLIB_VIS_PUBLIC_ALT const void* nlib_memchr(const void* s, int c, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC_ALT const void* nlib_memrchr(const void* s, int c, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_not(const void* s, int c, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC_ALT
const void* nlib_memchr_range_not(const void* __restrict s, const char* __restrict range,
                                  size_t n) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_lt(const void* s, int c, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_gt(const void* s, int c, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
// find (c & 0x80) != 0
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_mb(const void* s, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC size_t nlib_memspn(const void* __restrict buf, size_t len,
                                   const char* __restrict set, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
NLIB_VIS_PUBLIC size_t nlib_memcspn(const void* __restrict buf, size_t len,
                                    const char* __restrict set, size_t n)
    NLIB_NONNULL NLIB_ATTRIBUTE_PURE;

// ' ', CR, LF, HT are skipped
NLIB_VIS_PUBLIC_ALT
const char* nlib_skipws(size_t* __restrict cnt_lf, const char** __restrict last_lf,
                        const char* __restrict s, size_t n) NLIB_NONNULL_2;

#ifdef NLIB_LIBC_nlib_strlen
static
#else
NLIB_VIS_PUBLIC_ALT
#endif
size_t nlib_strlen(const char* s) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_strlen
static NLIB_C_INLINE size_t nlib_strlen(const char* s) { return strlen(s); }
#endif

#ifdef NLIB_LIBC_nlib_strnlen
static
#else
NLIB_VIS_PUBLIC_ALT
#endif
size_t nlib_strnlen(const char* s, size_t maxsize) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_strnlen
static NLIB_C_INLINE size_t nlib_strnlen(const char* s, size_t maxsize) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return strnlen_s(s, maxsize);
#else
    return strnlen(s, maxsize);
#endif
}
#endif

#ifdef NLIB_LIBC_nlib_strcpy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_strcpy(char* __restrict s1, size_t s1max, const char* __restrict s2) NLIB_NONNULL;
#ifdef NLIB_LIBC_nlib_strcpy
static NLIB_C_INLINE
errno_t nlib_strcpy(char*  __restrict s1, size_t s1max, const char* __restrict s2) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return strcpy_s(s1, s1max, s2);
#else
# error
#endif
}
#endif

#ifdef NLIB_LIBC_nlib_strncpy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_strncpy(char* __restrict s1, size_t s1max, const char* __restrict s2, size_t n)
    NLIB_NONNULL;
#ifdef NLIB_LIBC_nlib_strncpy
static NLIB_C_INLINE
errno_t nlib_strncpy(char* __restrict s1, size_t s1max, const char* __restrict s2, size_t n) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return strncpy_s(s1, s1max, s2, n);
#else
# error
#endif
}
#endif

#ifdef NLIB_LIBC_nlib_strchr
static
#else
NLIB_VIS_PUBLIC_ALT
#endif
const char* nlib_strchr(const char* s, int c) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_strchr
static NLIB_C_INLINE const char* nlib_strchr(const char* s, int c) { return strchr(s, c); }
#endif

#ifdef NLIB_LIBC_nlib_strrchr
static
#else
NLIB_VIS_PUBLIC_ALT
#endif
const char* nlib_strrchr(const char* s, int c) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_strrchr
static NLIB_C_INLINE const char* nlib_strrchr(const char* s, int c) { return strrchr(s, c); }
#endif

// find (c & 0x80) != 0, used for skipping ASCII chars
static const char* nlib_strchr_mb(const char* s) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
static NLIB_C_INLINE const char* nlib_strchr_mb(const char* s) {
    size_t n = nlib_strlen(s);
    const void* p = nlib_memchr_mb(s, n);
    if (p) {
        return (const char*)p;
    } else {
        return s + n;
    }
}

#ifdef NLIB_LIBC_nlib_wcslen
static
#else
NLIB_VIS_PUBLIC
#endif
size_t nlib_wcslen(const wchar_t* s) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_wcslen
static NLIB_C_INLINE size_t nlib_wcslen(const wchar_t* s) { return wcslen(s); }
#endif

#ifdef NLIB_LIBC_nlib_wcsnlen
static
#else
NLIB_VIS_PUBLIC
#endif
size_t nlib_wcsnlen(const wchar_t* s, size_t maxsize) NLIB_NONNULL NLIB_ATTRIBUTE_PURE;
#ifdef NLIB_LIBC_nlib_wcsnlen
static NLIB_C_INLINE size_t nlib_wcsnlen(const wchar_t* s, size_t maxsize) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return wcsnlen_s(s, maxsize);
#else
    return wcsnlen(s, maxsize);
#endif
}
#endif

#ifdef NLIB_LIBC_nlib_wcscpy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_wcscpy(wchar_t* __restrict s1, size_t s1max, const wchar_t* __restrict s2)
    NLIB_NONNULL;
#ifdef NLIB_LIBC_nlib_wcscpy
static NLIB_C_INLINE
errno_t nlib_wcscpy(wchar_t* __restrict s1, size_t s1max, const wchar_t* __restrict s2) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return wcscpy_s(s1, s1max, s2);
#else
# error
#endif
}
#endif

#ifdef NLIB_LIBC_nlib_wcsncpy
static
#else
NLIB_VIS_PUBLIC
#endif
errno_t nlib_wcsncpy(wchar_t* __restrict s1, size_t s1max, const wchar_t* __restrict s2, size_t n)
    NLIB_NONNULL;
#ifdef NLIB_LIBC_nlib_wcsncpy
static NLIB_C_INLINE
errno_t nlib_wcsncpy(wchar_t* __restrict s1, size_t s1max, const wchar_t* __restrict s2, size_t n) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return wcsncpy_s(s1, s1max, s2, n);
#else
# error
#endif
}
#endif

NLIB_VIS_PUBLIC
errno_t nlib_strto_int32(int32_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_int64(int64_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_uint32(uint32_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_uint64(uint64_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_double(double* result, const char* nptr, char** endptr)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_float(float* result, const char* nptr, char** endptr)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_int32_fallback(int32_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_int64_fallback(int64_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_uint32_fallback(uint32_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_uint64_fallback(uint64_t* result, const char* nptr, char** endptr, int base)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_double_fallback(double* result, const char* nptr, char** endptr)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC
errno_t nlib_strto_float_fallback(float* result, const char* nptr, char** endptr)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
static NLIB_C_INLINE
errno_t nlib_strto_int8(int8_t* result, const char* nptr, char** endptr, int base) {
    int32_t tmp;
    errno_t e;
    e = nlib_strto_int32(&tmp, nptr, endptr, base);
    if (e != 0 && e != ERANGE) return e;
    if (tmp > 127 || tmp < -128) {
        *result = tmp < 0 ? -128 : 127;
        return ERANGE;
    }
    *result = (int8_t)tmp;
    return e;
}
static NLIB_C_INLINE
errno_t nlib_strto_int16(int16_t* result, const char* nptr, char** endptr, int base) {
    int32_t tmp;
    errno_t e;
    e = nlib_strto_int32(&tmp, nptr, endptr, base);
    if (e != 0 && e != ERANGE) return e;
    if (tmp > 32767 || tmp < -32768) {
        *result = tmp < 0 ? -32768 : 32767;
        return ERANGE;
    }
    *result = (int16_t)tmp;
    return e;
}
static NLIB_C_INLINE
errno_t nlib_strto_uint8(uint8_t* result, const char* nptr, char** endptr, int base) {
    uint32_t tmp;
    errno_t e;
    e = nlib_strto_uint32(&tmp, nptr, endptr, base);
    if (e != 0 && e != ERANGE) return e;
    if (tmp > 255) {
        *result = 255;
        return ERANGE;
    }
    *result = (uint8_t)tmp;
    return e;
}
static NLIB_C_INLINE
errno_t nlib_strto_uint16(uint16_t* result, const char* nptr, char** endptr, int base) {
    uint32_t tmp;
    errno_t e;
    e = nlib_strto_uint32(&tmp, nptr, endptr, base);
    if (e != 0 && e != ERANGE) return e;
    if (tmp > 65535) {
        *result = 65535;
        return ERANGE;
    }
    *result = (uint16_t)tmp;
    return e;
}
NLIB_VIS_PUBLIC
errno_t nlib_int8_from_chars(int8_t* result, const char** endptr,
                             const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_int16_from_chars(int16_t* result, const char** endptr,
                              const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_int32_from_chars(int32_t* result, const char** endptr,
                              const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_int64_from_chars(int64_t* result, const char** endptr,
                              const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_uint8_from_chars(uint8_t* result, const char** endptr,
                              const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_uint16_from_chars(uint16_t* result, const char** endptr,
                               const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_uint32_from_chars(uint32_t* result, const char** endptr,
                               const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_uint64_from_chars(uint64_t* result, const char** endptr,
                               const char* first, const char* last, int base);
NLIB_VIS_PUBLIC
errno_t nlib_double_from_chars(double* result, const char** endptr,
                               const char* first, const char* last);
NLIB_VIS_PUBLIC
errno_t nlib_float_from_chars(float* result, const char** endptr,
                              const char* first, const char* last);

NLIB_VIS_PUBLIC
errno_t nlib_wide_to_utf8(size_t* __restrict utf8count, nlib_utf8_t* __restrict utf8,
                          size_t buflen, const wchar_t* __restrict wcstr) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_utf8_to_wide(size_t* __restrict wccount, wchar_t* __restrict wcstr,
                          size_t buflen, const nlib_utf8_t* __restrict utf8) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_memwide_to_utf8(size_t* __restrict to_count, size_t* __restrict from_count,
                             nlib_utf8_t* __restrict to, size_t to_size,
                             const wchar_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_memutf8_to_wide(size_t* __restrict to_count, size_t* __restrict from_count,
                             wchar_t* __restrict to, size_t to_size,
                             const nlib_utf8_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;

NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_wcscplen(size_t* __restrict count, const wchar_t* __restrict str) NLIB_NONNULL_2;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_strcplen(size_t* __restrict codepoint_count,
                      size_t* __restrict supplementary_codepoint_count,
                      size_t* __restrict len,
                      const nlib_utf8_t* __restrict str) NLIB_NONNULL_4;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_memcplen(size_t* __restrict codepoint_count,
                      size_t* __restrict supplementary_codepoint_count,
                      size_t* __restrict from_read,
                      const nlib_utf8_t* __restrict from,
                      size_t from_size) NLIB_NONNULL_3 NLIB_NONNULL_4;

// 0 if error
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
int nlib_utf16_to_utf32char(nlib_utf32_t* utf32, nlib_utf16_t upper,
                            nlib_utf16_t lower) NLIB_NONNULL;
// 0 if error
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
int nlib_utf32char_to_utf16(nlib_utf16_t* upper, nlib_utf16_t* lower,
                            nlib_utf32_t utf32) NLIB_NONNULL;
// 0 if error
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
int nlib_utf8_to_utf32char(nlib_utf32_t* utf32, const nlib_utf8_t* utf8) NLIB_NONNULL;
#ifdef __cplusplus
// 0 if error
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
int nlib_utf32char_to_utf8(nlib_utf8_t (&utf8)[4], nlib_utf32_t utf32);
#endif

NLIB_VIS_PUBLIC
errno_t nlib_utf16_to_utf8(size_t* utf8count, nlib_utf8_t* utf8, size_t buflen,
                           const nlib_utf16_t* utf16) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_utf8_to_utf16(size_t* utf16count, nlib_utf16_t* utf16, size_t buflen,
                           const nlib_utf8_t* utf8) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_utf32_to_utf8(size_t* utf8count, nlib_utf8_t* utf8, size_t buflen,
                           const nlib_utf32_t* utf32) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC
errno_t nlib_utf8_to_utf32(size_t* utf32count, nlib_utf32_t* utf32, size_t buflen,
                           const nlib_utf8_t* utf8) NLIB_NONNULL_4;

NLIB_VIS_PUBLIC
errno_t nlib_memutf16_to_utf8(size_t* __restrict to_count, size_t* __restrict from_count,
                              nlib_utf8_t* __restrict to, size_t to_size,
                              const nlib_utf16_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_memutf8_to_utf16(size_t* __restrict to_count, size_t* __restrict from_count,
                              nlib_utf16_t* __restrict to, size_t to_size,
                              const nlib_utf8_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_memutf32_to_utf8(size_t* __restrict to_count, size_t* __restrict from_count,
                              nlib_utf8_t* __restrict to, size_t to_size,
                              const nlib_utf32_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_memutf8_to_utf32(size_t* __restrict to_count, size_t* __restrict from_count,
                              nlib_utf32_t* __restrict to, size_t to_size,
                              const nlib_utf8_t* __restrict from, size_t from_size)
    NLIB_NONNULL_1 NLIB_NONNULL_2 NLIB_NONNULL_5;
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_memutf8_to_utf32char(size_t* count, nlib_utf32_t* utf32, const nlib_utf8_t* utf8,
                                  size_t utf8_size) NLIB_NONNULL;

NLIB_VIS_PUBLIC_ALT
size_t nlib_utf16len_(const uint16_t* str) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT
size_t nlib_utf16nlen_(const uint16_t* str, size_t maxsize) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_utf16cpy_(uint16_t* s1, size_t s1max, const uint16_t* s2) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_utf16ncpy_(uint16_t* s1, size_t s1max, const uint16_t* s2, size_t n) NLIB_NONNULL;
NLIB_CHECK_RESULT static NLIB_ALWAYS_INLINE
size_t nlib_utf16len(const nlib_utf16_t* str) {
    return nlib_utf16len_((const uint16_t*)str);
}
NLIB_CHECK_RESULT static NLIB_ALWAYS_INLINE
size_t nlib_utf16nlen(const nlib_utf16_t* str, size_t maxsize) {
    return nlib_utf16nlen_((const uint16_t*)str, maxsize);
}
static NLIB_ALWAYS_INLINE
errno_t nlib_utf16cpy(nlib_utf16_t* s1, size_t s1max, const nlib_utf16_t* s2) {
    return nlib_utf16cpy_((uint16_t*)s1, s1max, (const uint16_t*)s2);
}
static NLIB_ALWAYS_INLINE
errno_t nlib_utf16ncpy(nlib_utf16_t* s1, size_t s1max, const nlib_utf16_t* s2, size_t n) {
    return nlib_utf16ncpy_((uint16_t*)s1, s1max, (const uint16_t*)s2, n);
}

NLIB_VIS_PUBLIC_ALT
size_t nlib_utf32len_(const uint32_t* str) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT
size_t nlib_utf32nlen_(const uint32_t* str, size_t maxsize) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_utf32cpy_(uint32_t* s1, size_t s1max, const uint32_t* s2) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_utf32ncpy_(uint32_t* s1, size_t s1max, const uint32_t* s2, size_t n) NLIB_NONNULL;
NLIB_CHECK_RESULT static NLIB_ALWAYS_INLINE
size_t nlib_utf32len(const nlib_utf32_t* str) {
    return nlib_utf32len_((const uint32_t*)str);
}
NLIB_CHECK_RESULT static NLIB_ALWAYS_INLINE
size_t nlib_utf32nlen(const nlib_utf32_t* str, size_t maxsize) {
    return nlib_utf32nlen_((const uint32_t*)str, maxsize);
}
static NLIB_ALWAYS_INLINE
errno_t nlib_utf32cpy(nlib_utf32_t* s1, size_t s1max, const nlib_utf32_t* s2) {
    return nlib_utf32cpy_((uint32_t*)s1, s1max, (const uint32_t*)s2);
}
static NLIB_ALWAYS_INLINE
errno_t nlib_utf32ncpy(nlib_utf32_t* s1, size_t s1max, const nlib_utf32_t* s2, size_t n) {
    return nlib_utf32ncpy_((uint32_t*)s1, s1max, (const uint32_t*)s2, n);
}

NLIB_CHECK_RESULT NLIB_VIS_PUBLIC_ALT
errno_t nlib_utf16cplen_ex_(size_t* count, size_t* len, const uint16_t* str) NLIB_NONNULL_3;

static NLIB_ALWAYS_INLINE
errno_t nlib_utf16cplen(size_t* count, const nlib_utf16_t* str) {
    return nlib_utf16cplen_ex_(count, NULL, (const uint16_t*)str);
}
static NLIB_ALWAYS_INLINE
errno_t nlib_utf16cplen_ex(size_t* count, size_t* len, const nlib_utf16_t* str) {
    return nlib_utf16cplen_ex_(count, len, (const uint16_t*)str);
}
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC
errno_t nlib_utf32cplen(size_t* count, const nlib_utf32_t* str) NLIB_NONNULL_2;

#if defined(CAFE) || defined(NN_PLATFORM_CTR)
static NLIB_ALWAYS_INLINE uint16_t nlib_bswap16(uint16_t x) {
    return ((x & 0xFF) << 8) | ((x >> 8) & 0xFF);
}
static NLIB_ALWAYS_INLINE uint32_t nlib_bswap32(uint32_t x) {
    return (x << 24) | ((x & 0xFF00U) << 8) | ((x >> 8) & 0xFF00U) | (x >> 24);
}
static NLIB_ALWAYS_INLINE uint64_t nlib_bswap64(uint64_t x) {
    return (x << 56) |
            ((x & 0xFF00U) << 40) |
            ((x & 0xFF0000U) << 24) |
            ((x & 0xFF000000U) << 8) |
            ((x >> 8) & 0xFF000000U) |
            ((x >> 24) & 0xFF0000U) |
            ((x >> 40) & 0xFF00U) |
            (x >> 56);
}
#elif defined(_MSC_VER)
static NLIB_ALWAYS_INLINE uint16_t nlib_bswap16(uint16_t x) { return _byteswap_ushort(x); }
static NLIB_ALWAYS_INLINE uint32_t nlib_bswap32(uint32_t x) { return _byteswap_ulong(x); }
static NLIB_ALWAYS_INLINE uint64_t nlib_bswap64(uint64_t x) { return _byteswap_uint64(x); }
#else
static NLIB_ALWAYS_INLINE uint16_t nlib_bswap16(uint16_t x) { return __builtin_bswap16(x); }
static NLIB_ALWAYS_INLINE uint32_t nlib_bswap32(uint32_t x) { return __builtin_bswap32(x); }
static NLIB_ALWAYS_INLINE uint64_t nlib_bswap64(uint64_t x) { return __builtin_bswap64(x); }
#endif

typedef enum nlib_unicode_char_category {
    kUnicodeCharCategoryLu = 0,
    kUnicodeCharCategoryLl = 1,
    kUnicodeCharCategoryLt = 2,
    kUnicodeCharCategoryLm = 3,
    kUnicodeCharCategoryLo = 4,
    kUnicodeCharCategoryMn = 5,
    kUnicodeCharCategoryMc = 6,
    kUnicodeCharCategoryMe = 7,
    kUnicodeCharCategoryNd = 8,
    kUnicodeCharCategoryNl = 9,
    kUnicodeCharCategoryNo = 10,
    kUnicodeCharCategoryPc = 11,
    kUnicodeCharCategoryPd = 12,
    kUnicodeCharCategoryPs = 13,
    kUnicodeCharCategoryPe = 14,
    kUnicodeCharCategoryPi = 15,
    kUnicodeCharCategoryPf = 16,
    kUnicodeCharCategoryPo = 17,
    kUnicodeCharCategorySm = 18,
    kUnicodeCharCategorySc = 19,
    kUnicodeCharCategorySk = 20,
    kUnicodeCharCategorySo = 21,
    kUnicodeCharCategoryZs = 22,
    kUnicodeCharCategoryZl = 23,
    kUnicodeCharCategoryZp = 24,
    kUnicodeCharCategoryCc = 25,
    kUnicodeCharCategoryCf = 26,
    kUnicodeCharCategoryCs = 27,
    kUnicodeCharCategoryCo = 28,
    kUnicodeCharCategoryCn = 29
} nlib_unicode_char_category;
typedef enum nlib_case_mapping_option {
    kCaseMappingUpperCase = 0,
    kCaseMappingLowerCase
    // kCaseMappingTitleCase
} nlib_case_mapping_option;
typedef enum nlib_unicode_char_property {
    kUnicodeCharPropertyCategory = 0,
    kUnicodeCharPropertyCombiningClass
} nlib_unicode_char_property;
typedef enum nlib_case_folding_option {
    kCaseFoldingDefault = 0,
    kCaseFoldingExcludeSpecialI
} nlib_case_folding_option;
typedef enum nlib_nfkc_option {
    kNfkcStop = 0,
    kNfkcContinue,
    kNfkcDefault = 0
} nlib_nfkc_option;
typedef struct nlib_utf8_convert_info {
    size_t written;
    const nlib_utf8_t* cur;
} nlib_utf8_convert_info;
typedef enum nlib_unicode_break_property {
    kUnicodeBreakPropertyGrapheme = 0,
    kUnicodeBreakPropertyWord,
    kUnicodeBreakPropertySentence,
    kUnicodeBreakPropertyLine
} nlib_unicode_break_property;
typedef enum nlib_unicode_break_option {
    kUnicodeBreakStop = 0,
    kUnicodeBreakContinue,
    kUnicodeBreakDefault = 0
} nlib_unicode_break_option;

NLIB_VIS_PUBLIC
errno_t nlib_get_unicode_char_property(void* value,
    const nlib_utf8_t* first, const nlib_utf8_t* last,
    nlib_unicode_char_property property) NLIB_NONNULL;
NLIB_VIS_PUBLIC
errno_t nlib_case_mapping(nlib_utf8_convert_info* __restrict result,
    nlib_utf8_t* __restrict buf, size_t n,
    const nlib_utf8_t* __restrict first, const nlib_utf8_t* last,
    nlib_case_mapping_option option) NLIB_NONNULL_1 NLIB_NONNULL_4 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_case_folding(nlib_utf8_convert_info* __restrict result,
    nlib_utf8_t* __restrict buf, size_t n,
    const nlib_utf8_t* __restrict first, const nlib_utf8_t* last,
    nlib_case_folding_option option) NLIB_NONNULL_1 NLIB_NONNULL_4 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_nfkc(nlib_utf8_convert_info* __restrict result,
    nlib_utf8_t* __restrict buf, size_t n,
    const nlib_utf8_t* __restrict first, const nlib_utf8_t* last,
    nlib_nfkc_option option) NLIB_NONNULL_1 NLIB_NONNULL_4 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_nfkc_case_folding(nlib_utf8_convert_info* __restrict result,
    nlib_utf8_t* __restrict buf, size_t n,
    const nlib_utf8_t* __restrict first, const nlib_utf8_t* last,
    nlib_nfkc_option nfkc_option,
    nlib_case_folding_option case_folding_option) NLIB_NONNULL_1 NLIB_NONNULL_4 NLIB_NONNULL_5;
NLIB_VIS_PUBLIC
errno_t nlib_find_break(const nlib_utf8_t** pos, const nlib_utf8_t* first, const nlib_utf8_t* last,
    nlib_unicode_break_property property, nlib_unicode_break_option option) NLIB_NONNULL;

// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_16(uint16_t* p, size_t count) NLIB_NONNULL;
// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_32(uint32_t* p, size_t count) NLIB_NONNULL;
// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_64(uint64_t* p, size_t count) NLIB_NONNULL;

//
// malloc functions which nlib uses
// You can redefine them.
// See sample replace_malloc.cpp of nlibnx_heap.a library.
//

// weak function
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC_ALT
void* nlib_malloc(size_t size) NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(1);

// weak function
NLIB_VIS_PUBLIC_ALT void nlib_free(void* ptr);

// weak function
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC_ALT
void* nlib_calloc(size_t nmemb, size_t size)
    NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE2(1, 2);

// weak function
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC_ALT
void* nlib_realloc(void* ptr, size_t size) NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(2);

// weak function, not defined if CAFE or CTR
NLIB_VIS_PUBLIC size_t nlib_malloc_size(const void* ptr) NLIB_NONNULL;

// weak function(calls nlib_free(ptr) by default)
NLIB_VIS_PUBLIC_ALT void nlib_free_size(void* ptr, size_t size);

// weak function, not defined if WIN32 or CTR
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC_ALT
void* nlib_memalign(size_t alignment, size_t size)
    NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(2) NLIB_ATTRIBUTE_ALLOC_ALIGN(1);

#ifndef NLIB_MEMCPY
# define NLIB_MEMCPY(a, b, c) memcpy((a), (b), (c))
#endif

#ifndef NLIB_MEMMOVE
# define NLIB_MEMMOVE(a, b, c) memmove((a), (b), (c))
#endif

#ifndef NLIB_MEMSET
# define NLIB_MEMSET(a, b, c) memset((a), (b), (c))
#endif

// ctype.h without locale
static int nlib_isalnum(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isalpha(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isblank(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_iscntrl(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isdigit(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isgraph(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_islower(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isprint(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_ispunct(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isspace(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isupper(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_isxdigit(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_tolower(int ch) NLIB_ATTRIBUTE_CONST;
static int nlib_toupper(int ch) NLIB_ATTRIBUTE_CONST;

static NLIB_C_INLINE int nlib_isalnum(int ch) {
    return ('0' <= ch && ch <= '9') || ('A' <= ch && ch <= 'Z') || ('a' <= ch && ch <= 'z');
}
static NLIB_C_INLINE int nlib_isalpha(int ch) {
    return ('A' <= ch && ch <= 'Z') || ('a' <= ch && ch <= 'z');
}
static NLIB_C_INLINE int nlib_isblank(int ch) { return ch == ' ' || ch == '\t'; }
static NLIB_C_INLINE int nlib_iscntrl(int ch) { return (ch >= 0 && ch <= 0x1F) || ch == 0x7F; }
static NLIB_C_INLINE int nlib_isdigit(int ch) { return ('0' <= ch && ch <= '9'); }
static NLIB_C_INLINE int nlib_isgraph(int ch) { return ch >= 0x21 && ch <= 0x7E; }
static NLIB_C_INLINE int nlib_islower(int ch) { return (ch >= 'a' && ch <= 'z'); }
static NLIB_C_INLINE int nlib_isprint(int ch) { return ch >= 0x20 && ch <= 0x7E; }
static NLIB_C_INLINE int nlib_ispunct(int ch) { return (ch >= 0x00 && ch <= 0x20) || ch == 0x7F; }
static NLIB_C_INLINE int nlib_isspace(int ch) {
    return ((ch) == ' ' || (ch) == '\t' || (ch) == '\n');
}
static NLIB_C_INLINE int nlib_isupper(int ch) { return (ch >= 'A' && ch <= 'Z'); }
static NLIB_C_INLINE int nlib_isxdigit(int ch) {
    return (unsigned int)(ch - '0') < 10u ||
        (unsigned int)((ch | 0x20) - 'a') < 6u;
}
static NLIB_C_INLINE int nlib_tolower(int ch) {
    return (ch >= 'A' && ch <= 'Z') ? ch + ('a' - 'A') : ch;
}
static NLIB_C_INLINE int nlib_toupper(int ch) {
    return (ch >= 'a' && ch <= 'z') ? ch - ('a' - 'A') : ch;
}
// no isascii, toascii

// memcpy_s
static NLIB_C_INLINE
errno_t nlib_memcpy(void* __restrict s1, size_t s1max, const void* __restrict s2, size_t n) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return memcpy_s(s1, s1max, s2, n);
#else
#ifndef NLIB_NONNULL_ENABLED
    if (!s1 || !s2) return ERANGE;
#endif
    if (s1max < n) {
        NLIB_MEMSET(s1, 0, s1max);
        return ERANGE;
    }
    NLIB_MEMCPY(s1, s2, n);
    return 0;
#endif
}

// memmove_s
static NLIB_C_INLINE
errno_t nlib_memmove(void* s1, size_t s1max, const void* s2, size_t n) {
#if defined(_MSC_VER) || defined(__STDC_LIB_EXT1__)
    return memmove_s(s1, s1max, s2, n);
#else
#ifndef NLIB_NONNULL_ENABLED
    if (!s1 || !s2) return ERANGE;
#endif
    if (s1max < n) return ERANGE;
    NLIB_MEMMOVE(s1, s2, n);
    return 0;
#endif
}

static NLIB_C_INLINE
errno_t nlib_memset(void* buf, int ch, size_t n) {
    NLIB_EINVAL_IFNULL(buf);
    NLIB_MEMSET(buf, ch, n);
    return 0;
}

static int nlib_popcnt16(uint16_t x) NLIB_ATTRIBUTE_CONST;
static int nlib_popcnt32(uint32_t x) NLIB_ATTRIBUTE_CONST;
static int nlib_popcnt64(uint64_t x) NLIB_ATTRIBUTE_CONST;
#if defined(NLIB_SSE42)
static NLIB_ALWAYS_INLINE int nlib_popcnt16(uint16_t x) {
    return _mm_popcnt_u32(x);
}
static NLIB_ALWAYS_INLINE int nlib_popcnt32(uint32_t x) {
    return _mm_popcnt_u32(x);
}
static NLIB_ALWAYS_INLINE int nlib_popcnt64(uint64_t x) {
#ifdef NLIB_64BIT
    return (int)_mm_popcnt_u64(x);
#else
    uint32_t lo = (uint32_t)(x & 0xFFFFFFFFU);
    uint32_t hi = (uint32_t)((x >> 32) & 0xFFFFFFFFU);
    return _mm_popcnt_u32(lo) + _mm_popcnt_u32(hi);
#endif
}
#elif defined(NLIB_NEON)
static NLIB_ALWAYS_INLINE int nlib_popcnt16(uint16_t x) {
    uint8x8_t x0 = vcnt_u8(vreinterpret_u8_u64(vcreate_u64(x)));
#ifdef __aarch64__
    return vaddv_u8(x0);
#else
    uint8x8_t x1 = vpadd_u8(x0, x0);
    return vget_lane_u8(x1, 0);
#endif
}
static NLIB_ALWAYS_INLINE int nlib_popcnt32(uint32_t x) {
    uint8x8_t x0 = vcnt_u8(vreinterpret_u8_u64(vcreate_u64(x)));
#ifdef __aarch64__
    return vaddv_u8(x0);
#else
    uint8x8_t x1 = vpadd_u8(x0, x0);
    uint8x8_t x2 = vpadd_u8(x1, x1);
    return vget_lane_u8(x2, 0);
#endif
}
static NLIB_ALWAYS_INLINE int nlib_popcnt64(uint64_t x) {
    uint8x8_t x0 = vcnt_u8(vreinterpret_u8_u64(vcreate_u64(x)));
#ifdef __aarch64__
    return vaddv_u8(x0);
#else
    uint8x8_t x1 = vpadd_u8(x0, x0);
    uint8x8_t x2 = vpadd_u8(x1, x1);
    uint8x8_t x3 = vpadd_u8(x2, x2);
    return vget_lane_u8(x3, 0);
#endif
}
#else
extern NLIB_VIS_PUBLIC const unsigned char _nlib_popcnt_array[];
static NLIB_ALWAYS_INLINE int nlib_popcnt32(uint32_t x) {
    return _nlib_popcnt_array[(x >> 24) & 0xFF] + _nlib_popcnt_array[(x >> 16) & 0xFF] +
           _nlib_popcnt_array[(x >> 8) & 0xFF] + _nlib_popcnt_array[(x)& 0xFF];
}
static NLIB_ALWAYS_INLINE int nlib_popcnt64(uint64_t x) {
    return _nlib_popcnt_array[(x >> 56) & 0xFF] + _nlib_popcnt_array[(x >> 48) & 0xFF] +
           _nlib_popcnt_array[(x >> 40) & 0xFF] + _nlib_popcnt_array[(x >> 32) & 0xFF] +
           _nlib_popcnt_array[(x >> 24) & 0xFF] + _nlib_popcnt_array[(x >> 16) & 0xFF] +
           _nlib_popcnt_array[(x >> 8) & 0xFF] + _nlib_popcnt_array[(x)& 0xFF];
}
static NLIB_ALWAYS_INLINE int nlib_popcnt16(uint16_t x) {
    return _nlib_popcnt_array[(x >> 8) & 0xFF] + _nlib_popcnt_array[(x)& 0xFF];
}
#endif

// nlib_clz32(0x80000000) -> 0, nlib_clz32(1) -> 31
static int nlib_clz32(uint32_t x) NLIB_ATTRIBUTE_CONST;
// nlib_ctz32(0x80000000) -> 31, nlib_ctz32(1) -> 0
static int nlib_ctz32(uint32_t x) NLIB_ATTRIBUTE_CONST;
// nlib_clz64(INT64_MIN) -> 0, nlib_clz64(1) -> 63
static int nlib_clz64(uint64_t x) NLIB_ATTRIBUTE_CONST;
// nlib_ctz64(INT64_MIN) -> 63, nlib_ctz64(1) -> 0
static int nlib_ctz64(uint64_t x) NLIB_ATTRIBUTE_CONST;
#if defined(_MSC_VER)
static NLIB_ALWAYS_INLINE int nlib_clz32(uint32_t x) {
    DWORD cnt;
    return _BitScanReverse(&cnt, x) ? (int)(31 - cnt) : 32;
    // return (int)(__lzcnt(x));  // needs haswell+
}
static NLIB_ALWAYS_INLINE int nlib_ctz32(uint32_t x) {
    DWORD cnt;
    return _BitScanForward(&cnt, x) ? cnt : 32;
}
static NLIB_ALWAYS_INLINE int nlib_clz64(uint64_t x) {
#ifdef NLIB_64BIT
    DWORD cnt;
    return _BitScanReverse64(&cnt, x) ? (int)(63 - cnt) : 64;
    // return (int)(__lzcnt64(x));  // needs haswell+
#else
    DWORD cnt;
    DWORD dw = (DWORD)(x >> 32);
    if (_BitScanReverse(&cnt, dw)) {
        return (int)(31 - cnt);
    } else {
        dw = (DWORD)(x);
        return _BitScanReverse(&cnt, dw) ?
            (int)(63 - cnt) : 64;
    }
#endif
}
static NLIB_ALWAYS_INLINE int nlib_ctz64(uint64_t x) {
#ifdef NLIB_64BIT
    DWORD cnt;
    return _BitScanForward64(&cnt, x) ? cnt : 64;
#else
    DWORD cnt;
    DWORD dw = (DWORD)(x);
    if (_BitScanForward(&cnt, dw)) {
        return (int)(cnt);
    } else {
        dw = (DWORD)(x >> 32);
        return _BitScanForward(&cnt, dw) ?
            (int)(32 + cnt) : 64;
    }
#endif
}
#elif defined(CAFE)
static NLIB_ALWAYS_INLINE int nlib_clz32(uint32_t x) { return __CLZ32(x); }
static NLIB_ALWAYS_INLINE int nlib_ctz32(uint32_t x) { return 32 - nlib_clz32(~x & (x - 1)); }
static NLIB_ALWAYS_INLINE int nlib_clz64(uint64_t x) {
    int cnt;
    unsigned int dw = (unsigned int)(x >> 32);
    cnt = __CLZ32(dw);
    if (cnt < 32) {
        return cnt;
    } else {
        dw = (unsigned int)(x);
        cnt = __CLZ32(dw);
        return 32 + cnt;
    }
}
static NLIB_ALWAYS_INLINE int nlib_ctz64(uint64_t x) { return 64 - nlib_clz64(~x & (x - 1)); }
#elif defined(NN_PLATFORM_CTR)
static NLIB_ALWAYS_INLINE int nlib_clz32(uint32_t x) { return x != 0 ? __builtin_clz(x) : 32; }
static NLIB_ALWAYS_INLINE int nlib_ctz32(uint32_t x) { return 32 - nlib_clz32(~x & (x - 1)); }
static NLIB_ALWAYS_INLINE int nlib_clz64(uint64_t x) { return x != 0 ? __builtin_clzll(x) : 64; }
static NLIB_ALWAYS_INLINE int nlib_ctz64(uint64_t x) { return 64 - nlib_clz64(~x & (x - 1)); }
#else
static NLIB_ALWAYS_INLINE int nlib_clz32(uint32_t x) { return x != 0 ? __builtin_clz(x) : 32; }
static NLIB_ALWAYS_INLINE int nlib_ctz32(uint32_t x) { return x != 0 ? __builtin_ctz(x) : 32; }
static NLIB_ALWAYS_INLINE int nlib_clz64(uint64_t x) { return x != 0 ? __builtin_clzll(x) : 64; }
static NLIB_ALWAYS_INLINE int nlib_ctz64(uint64_t x) { return x != 0 ? __builtin_ctzll(x) : 64; }
#endif

static size_t nlib_strlcpy(char* __restrict s1, const char* __restrict s2, size_t s1max)
    NLIB_NONNULL;
static NLIB_C_INLINE
size_t nlib_strlcpy(char* __restrict s1, const char* __restrict s2, size_t s1max) {
#if defined(__FreeBSD__)
    return strlcpy(s1, s2, s1max);
#else
    size_t len = nlib_strlen(s2);
    if (NLIB_LIKELY(len < s1max)) {
        NLIB_MEMCPY(s1, s2, len + 1);
    } else if (NLIB_LIKELY(s1max > 0)) {
        NLIB_MEMCPY(s1, s2, s1max - 1);
        s1[s1max - 1] = '\0';
    }
    return len;
#endif
}

static uint32_t nlib_bitreverse32(uint32_t x) NLIB_ATTRIBUTE_CONST;
static uint64_t nlib_bitreverse64(uint64_t x) NLIB_ATTRIBUTE_CONST;

static NLIB_ALWAYS_INLINE uint32_t nlib_bitreverse32(uint32_t x) {
#if __has_builtin(__builtin_bitreverse32)
    return __builtin_bitreverse32(x);
#elif __has_builtin(__builtin_arm_rbit)
    return __builtin_arm_rbit(x);
#elif defined(__arm__) && !defined(NN_PLATFORM_CTR)
    return __rbit(x);
#else
    x = ((x & 0x55555555U) << 1) | ((x >> 1) & 0x55555555U);
    x = ((x & 0x33333333U) << 2) | ((x >> 2) & 0x33333333U);
    x = ((x & 0x0F0F0F0FU) << 4) | ((x >> 4) & 0x0F0F0F0FU);
#ifdef _MSC_VER
    x = _byteswap_ulong(x);
#elif defined(CAFE) || defined(NN_PLATFORM_CTR)
    x = (x << 24) | ((x & 0xFF00) << 8) |
        ((x >> 8) & 0xFF00) | (x >> 24);
#else
    x = __builtin_bswap32(x);
#endif
    return x;
#endif
}

static NLIB_ALWAYS_INLINE uint64_t nlib_bitreverse64(uint64_t x) {
#if __has_builtin(__builtin_bitreverse64)
    return __builtin_bitreverse64(x);
#elif __has_builtin(__builtin_arm_rbit64)
    return __builtin_arm_rbit64(x);
#elif __has_builtin(__builtin_arm_rbit)
    return __builtin_arm_rbit(x >> 32) |
           (((uint64_t)__builtin_arm_rbit(x)) << 32);
#elif defined(__arm__) && !defined(NN_PLATFORM_CTR)
    return __rbit(x >> 32) |
        (((uint64_t)__rbit(x)) << 32);
#else
    x = ((x & 0x5555555555555555ULL) << 1) | ((x >> 1) & 0x5555555555555555ULL);
    x = ((x & 0x3333333333333333ULL) << 2) | ((x >> 2) & 0x3333333333333333ULL);
    x = ((x & 0x0F0F0F0F0F0F0F0FULL) << 4) | ((x >> 4) & 0x0F0F0F0F0F0F0F0FULL);
#ifdef _MSC_VER
    x = _byteswap_uint64(x);
#elif defined(CAFE) || defined(NN_PLATFORM_CTR)
    x =
        (x << 56) |
        ((x & 0xFF00U) << 40) |
        ((x & 0xFF0000U) << 24) |
        ((x & 0xFF000000U) << 8) |
        ((x >> 8) & 0xFF000000U) |
        ((x >> 24) & 0xFF0000U) |
        ((x >> 40) & 0xFF00U) |
        (x >> 56);
#else
    x = __builtin_bswap64(x);
#endif
    return x;
#endif
}

#undef NLIB_MEMCPY
#undef NLIB_MEMMOVE
#undef NLIB_MEMSET

#ifdef __cplusplus
}
#endif

#if defined(_MSC_VER)
#if defined(n_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#elif defined(nx_misc_EXPORTS)
# undef NLIB_VIS_PUBLIC
# define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif
#endif

#endif  // INCLUDE_NN_NLIB_PLATFORM_H_
