﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_CAFE_H_
#define INCLUDE_NN_NLIB_PLATFORM_CAFE_H_
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
#error do not include directly
#endif
#ifdef CAFE

#ifdef __cplusplus
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS
#endif
#ifndef __STDC_CONSTANT_MACROS
#define __STDC_CONSTANT_MACROS
#endif
#endif

#include <ppc_ps.h>
#include <cafe/os.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NLIB_CAFE_PPC
#define NLIB_CAFE_PPC
#endif
#define NLIB_ALWAYS_INLINE inline __attribute__((always_inline))
#define NLIB_NEVER_INLINE __attribute__((__noinline__))
#ifdef __cplusplus
#define NLIB_LIKELY(x) __builtin_expect(!!(x), 1)
#define NLIB_UNLIKELY(x) __builtin_expect(!!(x), 0)
#else
#define NLIB_LIKELY(x) (x)
#define NLIB_UNLIKELY(x) (x)
#endif
#define NLIB_EXPECT(var, exp_value) __builtin_expect((var), (exp_value))
// ghs warns even if (void)func(); written
#define NLIB_CHECK_RESULT
// #define NLIB_CHECK_RESULT __attribute__((warn_unused_result))
#define NLIB_NORETURN __attribute__((noreturn))
#define NLIB_FALLTHROUGH
#define NLIB_NONNULL __attribute__((nonnull))
#define NLIB_NONNULL_1 __attribute__((nonnull(1)))
#define NLIB_NONNULL_2 __attribute__((nonnull(2)))
#define NLIB_NONNULL_3 __attribute__((nonnull(3)))
#define NLIB_NONNULL_4 __attribute__((nonnull(4)))
#define NLIB_NONNULL_5 __attribute__((nonnull(5)))
#define NLIB_NONNULL_ENABLED
#define NLIB_ATTRIBUTE_MALLOC __attribute__((malloc))
#define NLIB_ATTRIBUTE_PURE __attribute__((pure))
#define NLIB_ATTRIBUTE_CONST __attribute__((const))
#define NLIB_ATTRIBUTE_ALLOC_SIZE1(n)
#define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1)
#define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
#define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
#ifndef NLIB_DEPRECATED
#define NLIB_DEPRECATED __attribute__((deprecated))
#endif
#ifndef NLIB_DEPRECATED_MSG
#define NLIB_DEPRECATED_MSG(msg) __attribute__((deprecated))
#endif
#define NLIB_VIS_HIDDEN
#define NLIB_VIS_PUBLIC
#define NLIB_WEAKSYMBOL __attribute__((weak))

#define NLIB_MEMORY_ORDER_RELEASE __LWSYNC()
#define NLIB_MEMORY_ORDER_ACQUIRE __ISYNC()
#define NLIB_MEMORY_ORDER_ACQ_REL \
    __LWSYNC();                   \
    __ISYNC()
#define NLIB_MEMORY_ORDER_SEQ_CST OSCoherencyBarrier()

// GHS does not support '%zu', and cafe is 32bit environment
#define __PRIS_PREFIX

typedef unsigned int nlib_tls;

typedef OSFastMutex nlib_mutex;
#define NLIB_MUTEX_INITIALIZER \
    { 0 }
#define NLIB_RECURSIVE_MUTEX_INITIALIZER \
    { 0 }
#define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER \
    { 0 }

typedef void* nlib_thread;

typedef OSSemaphore nlib_semaphore;

typedef OSFastCond nlib_cond;
#define NLIB_COND_INITIALIZER \
    { 0 }

void MyNoreturn_(void) __attribute__((noreturn));
#define NLIB_ASSUME(cond) \
    switch (0)            \
    case 0:               \
    default:              \
        if (cond)         \
            ;             \
        else              \
            MyNoreturn_()

#define NLIB_ATOMIC_RELAXED (0)
#define NLIB_ATOMIC_ACQUIRE (1)
#define NLIB_ATOMIC_RELEASE (2)
#define NLIB_ATOMIC_ACQ_REL (3)
#define NLIB_ATOMIC_SEQ_CST (7)

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    int32_t rval = *(volatile int32_t*)ptr;
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    *(volatile int32_t*)ptr = val;
    if (memorder == NLIB_ATOMIC_SEQ_CST) OSCoherencyBarrier();
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val, int memorder) {
    uint32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSSwapAtomic((volatile OSAtomicVar*)ptr, (uint32_t)val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return (int32_t)x;
}

static __inline void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder) {
    return (void*)nlib_atomic_exchange32((int32_t*)ptr, (int32_t)val, memorder);
}

static NLIB_ALWAYS_INLINE int
nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected, int32_t desired, int weak,
                               int success_memorder, int failure_memorder) {
    if (success_memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (success_memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    if (weak == 0) {
        BOOL result = OSCompareAndSwapAtomicEx((volatile OSAtomicVar*)ptr, (u32)*expected,
                                               (u32)desired, (u32*)expected);
        if (result) {
            if (success_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
        } else {
            if (failure_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
        }
        return result;
    } else {
        u32 orig_val;
        orig_val = (u32)__LWARX((u32*)ptr, 0);
        if (orig_val == *expected) {
            __DCBST(0, (u32)ptr);
            if (__STWCX((u32*)ptr, 0, (u32)desired)) {
                if (success_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
                return 1;
            }
            if (failure_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
            return 0;
        } else {
            *expected = (int32_t)orig_val;
            if (failure_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
            return 0;
        }
    }
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x + val;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic((volatile OSAtomicVar*)ptr, -val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x - val;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAndAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x & val;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSXorAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x ^ val;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSOrAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x | val;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic((volatile OSAtomicVar*)ptr, -val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAndAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSXorAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder) {
    int32_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSOrAtomic((volatile OSAtomicVar*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    int64_t rval = (int64_t)OSGetAtomic64((volatile OSAtomicVar64*)ptr);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    OSSetAtomic64((volatile OSAtomicVar64*)ptr, (u64)val);
    if (memorder == NLIB_ATOMIC_SEQ_CST) OSCoherencyBarrier();
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder) {
    uint64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSSwapAtomic64((volatile OSAtomicVar64*)ptr, (uint64_t)val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return (int64_t)x;
}

static NLIB_ALWAYS_INLINE int
nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected, int64_t desired, int weak,
                               int success_memorder, int failure_memorder) {
    BOOL result;
    (void)weak;
    if (success_memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (success_memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;

    result = OSCompareAndSwapAtomicEx64((volatile OSAtomicVar64*)ptr, (u64)*expected, (u64)desired,
                                        (u64*)expected);
    if (result) {
        if (success_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    } else {
        if (failure_memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    }
    return result;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x + val;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic64((volatile OSAtomicVar64*)ptr, -val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x - val;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAndAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x & val;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSXorAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x ^ val;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSOrAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x | val;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAddAtomic64((volatile OSAtomicVar64*)ptr, -val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSAndAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSXorAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder) {
    int64_t x;
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    x = OSOrAtomic64((volatile OSAtomicVar64*)ptr, val);
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return x;
}

static NLIB_ALWAYS_INLINE void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    void* rval = *(void* volatile*)ptr;
    if (memorder & NLIB_ATOMIC_ACQUIRE) NLIB_MEMORY_ORDER_ACQUIRE;
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        OSCoherencyBarrier();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    *(void* volatile*)ptr = val;
    if (memorder == NLIB_ATOMIC_SEQ_CST) OSCoherencyBarrier();
}

static NLIB_ALWAYS_INLINE int
nlib_atomic_compare_exchangeptr(void** ptr, void** expected, void* desired, int weak,
                                int success_memorder, int failure_memorder) {
    return nlib_atomic_compare_exchange32((int32_t*)ptr, (int32_t*)expected, (int32_t)desired, weak,
                                          success_memorder, failure_memorder);
}

static NLIB_ALWAYS_INLINE void nlib_atomic_thread_fence(int memorder) {
    switch (memorder) {
    case NLIB_ATOMIC_RELAXED:
        break;
    case NLIB_ATOMIC_ACQUIRE:
        NLIB_MEMORY_ORDER_ACQUIRE;
        break;
    case NLIB_ATOMIC_RELEASE:
        NLIB_MEMORY_ORDER_RELEASE;
        break;
    case NLIB_ATOMIC_ACQ_REL:
        NLIB_MEMORY_ORDER_ACQ_REL;
        break;
    default:
        NLIB_MEMORY_ORDER_SEQ_CST;
        break;
    }
}

#define NLIB_FD_O_RDONLY (0x0000)
#define NLIB_FD_O_WRONLY (0x0001)
#define NLIB_FD_O_RDWR (0x0002)
#define NLIB_FD_O_APPEND (0x0008)
#define NLIB_FD_O_CREAT (0x0100)
#define NLIB_FD_O_TRUNC (0x0200)
#define NLIB_FD_O_EXCL (0x0400)

#ifdef __cplusplus
}
#endif

#define NLIB_NOEXCEPT

// --restrict is not specified in SDK's configuration
#ifndef __cplusplus
#define __restrict
#endif

#define NLIB_MEMCPY(a, b, c) OSBlockMove((a), (b), (c), FALSE)
#define NLIB_MEMMOVE(a, b, c) OSBlockMove((a), (b), (c), FALSE)
#define NLIB_MEMSET(a, b, c) OSBlockSet((a), (b), (c))

#ifndef NLIB_HAS_ZLIB
#define NLIB_HAS_ZLIB
#endif

#ifndef NLIB_HAS_LIBCURL
#define NLIB_HAS_LIBCURL
#endif

#endif
#endif  // INCLUDE_NN_NLIB_PLATFORM_CAFE_H_
