﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_SOCKET_H_
#define INCLUDE_NN_NLIB_PLATFORM_SOCKET_H_

#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
#error Do not include this file directly
#endif

#if defined(_MSC_VER) || defined(NLIB_UNIX)
#define NLIB_SOCKET_ENABLED

// sizeof(SOCKET) is 8 on Win64, but it's safe because the upper 32bit of SOCKET is always 0.
// https://stackoverflow.com/questions/1953639/is-it-safe-to-cast-socket-to-int-under-win64
typedef int nlib_sock;
#define NLIB_SOCKET_INVALID (nlib_sock)(-1)

#if defined(_MSC_VER)
#define NLIB_SOCK_NONBLOCK 0x8000
#elif defined(SOCK_NONBLOCK)
#define NLIB_SOCK_NONBLOCK SOCK_NONBLOCK
#else
#define NLIB_SOCK_NONBLOCK O_NONBLOCK
#endif

#if !defined(TCP_FASTOPEN) && defined(_MSC_VER)
// (older) Windows SDK may not define TCP_FASTOPEN even if available
#define TCP_FASTOPEN 15
#endif

NLIB_VIS_PUBLIC errno_t nlib_socket(nlib_sock* sockfd, int af, int type, int protocol);

typedef struct sockaddr nlib_sockaddr;
typedef struct sockaddr_in nlib_sockaddr_in;
typedef struct sockaddr_in6 nlib_sockaddr_in6;
#ifdef _MSC_VER
struct nlib_msghdr_ {
    void* msg_name;
    uint32_t msg_namelen;
    nlib_fd_iovec* msg_iov;
    size_t msg_iovlen;
    void* msg_control;
    size_t msg_controllen;
    int msg_flags;
};
typedef struct nlib_msghdr_ nlib_msghdr;
#define NLIB_CMSG_FIRSTHDR(msgh)                                                      \
    ((msgh)->msg_controllen >= sizeof(WSACMSGHDR) ? (LPWSACMSGHDR)(msgh)->msg_control \
                                                  : (LPWSACMSGHDR)NULL)
#define NLIB_CMSG_NXTHDR(msg, cmsg)                                                        \
    (((cmsg) == NULL)                                                                      \
         ? NLIB_CMSG_FIRSTHDR(msg)                                                         \
         : ((((PUCHAR)(cmsg) + WSA_CMSGHDR_ALIGN((cmsg)->cmsg_len) + sizeof(WSACMSGHDR)) > \
             (PUCHAR)((msg)->msg_control) + (msg)->msg_controllen)                         \
                ? (LPWSACMSGHDR)NULL                                                       \
                : (LPWSACMSGHDR)((PUCHAR)(cmsg) + WSA_CMSGHDR_ALIGN((cmsg)->cmsg_len))))
#define NLIB_CMSG_SPACE(length) WSA_CMSG_SPACE(length)
#define NLIB_CMSG_LEN(length) WSA_CMSG_LEN(length)
#define NLIB_CMSG_DATA(cmsg) WSA_CMSG_DATA(cmsg)
#else
typedef struct msghdr nlib_msghdr;
#define NLIB_CMSG_FIRSTHDR(msgh) CMSG_FIRSTHDR(msgh)
#define NLIB_CMSG_NXTHDR(msgh, cmsg) CMSG_NXTHDR(msgh, cmsg)
// #define NLIB_CMSG_ALIGN(length) CMSG_ALIGN(length)
#define NLIB_CMSG_SPACE(length) CMSG_SPACE(length)
#define NLIB_CMSG_LEN(length) CMSG_LEN(length)
#define NLIB_CMSG_DATA(cmsg) CMSG_DATA(cmsg)
#endif
typedef struct cmsghdr nlib_cmsghdr;
NLIB_VIS_PUBLIC errno_t nlib_bind(nlib_sock sockfd, const nlib_sockaddr* addr, uint32_t namelen);
NLIB_VIS_PUBLIC errno_t nlib_listen(nlib_sock sockfd, int backlog);
NLIB_VIS_PUBLIC
errno_t nlib_accept(nlib_sock* __restrict s, nlib_sock sockfd, nlib_sockaddr* __restrict addr,
                    uint32_t* __restrict addrlen, int flags);
NLIB_VIS_PUBLIC
errno_t nlib_accept_for(nlib_sock* __restrict s, nlib_sock sockfd, nlib_sockaddr* __restrict addr,
                        uint32_t* __restrict addrlen, int flags, nlib_duration timeout);
NLIB_VIS_PUBLIC
errno_t nlib_connect(nlib_sock sockfd, const nlib_sockaddr* addr, uint32_t addrlen);
NLIB_VIS_PUBLIC
errno_t nlib_connect_for(nlib_sock sockfd, const nlib_sockaddr* addr, uint32_t addrlen, int flags,
                         nlib_duration timeout);

NLIB_VIS_PUBLIC
errno_t
nlib_sendto(size_t* __restrict size, nlib_sock sockfd, const void* __restrict buf, size_t len,
            int flags, const nlib_sockaddr* __restrict dest_addr, uint32_t addrlen);
NLIB_VIS_PUBLIC
errno_t nlib_sendmsg(size_t* __restrict size, nlib_sock sockfd, const nlib_msghdr* msg, int flags);
static NLIB_C_INLINE errno_t nlib_send(size_t* __restrict size, nlib_sock sockfd,
                                       const void* __restrict buf, size_t len, int flags) {
    return nlib_sendto(size, sockfd, buf, len, flags, NULL, 0);
}

NLIB_VIS_PUBLIC
errno_t nlib_recvfrom(size_t* __restrict size, nlib_sock sockfd, void* __restrict buf, size_t len,
                      int flags, nlib_sockaddr* __restrict dest_addr, uint32_t* __restrict addrlen);
NLIB_VIS_PUBLIC
errno_t nlib_recvmsg(size_t* __restrict size, nlib_sock sockfd, nlib_msghdr* msg, int flags);
static NLIB_C_INLINE errno_t nlib_recv(size_t* __restrict size, nlib_sock sockfd,
                                       void* __restrict buf, size_t len, int flags) {
    return nlib_recvfrom(size, sockfd, buf, len, flags, NULL, NULL);
}

NLIB_VIS_PUBLIC errno_t nlib_closesocket(nlib_sock sockfd);

#ifdef _MSC_VER
#define SHUT_RD SD_RECEIVE
#define SHUT_WR SD_SEND
#define SHUT_RDWR SD_BOTH
#endif

NLIB_VIS_PUBLIC errno_t nlib_shutdownsocket(nlib_sock sockfd, int how);

#ifndef NLIB_LITTLE_ENDIAN
static NLIB_C_INLINE uint32_t nlib_htonl(uint32_t hostlong) {
    return hostlong;
}
static NLIB_C_INLINE uint16_t nlib_htons(uint16_t hostshort) {
    return hostshort;
}
static NLIB_C_INLINE uint32_t nlib_ntohl(uint32_t netlong) {
    return netlong;
}
static NLIB_C_INLINE uint16_t nlib_ntohs(uint16_t netshort) {
    return netshort;
}
#else
static NLIB_C_INLINE uint32_t nlib_htonl(uint32_t hostlong) {
#ifdef _MSC_VER
    return _byteswap_ulong(hostlong);
#else
    return __builtin_bswap32(hostlong);
#endif
}
static NLIB_C_INLINE uint16_t nlib_htons(uint16_t hostshort) {
#ifdef _MSC_VER
    return _byteswap_ushort(hostshort);
#else
    return ((hostshort & 0xFF) << 8) | ((hostshort >> 8) & 0xFF);
#endif
}
static NLIB_C_INLINE uint32_t nlib_ntohl(uint32_t netlong) {
#ifdef _MSC_VER
    return _byteswap_ulong(netlong);
#else
    return __builtin_bswap32(netlong);
#endif
}
static NLIB_C_INLINE uint16_t nlib_ntohs(uint16_t netshort) {
#ifdef _MSC_VER
    return _byteswap_ushort(netshort);
#else
    return ((netshort & 0xFF) << 8) | ((netshort >> 8) & 0xFF);
#endif
}
#endif

typedef struct in_addr nlib_in_addr;
typedef struct in6_addr nlib_in6_addr;
NLIB_VIS_PUBLIC errno_t nlib_inet_pton(int af, const char* __restrict src, void* __restrict dst);
NLIB_VIS_PUBLIC errno_t nlib_inet_ntop(int af, const void* __restrict src, char* __restrict dst,
                                       uint32_t dst_size);

/*
# define EAI_ADDRFAMILY 1
# define EAI_AGAIN 2
# define EAI_BADFLAGS 3
# define EAI_FAIL 4
# define EAI_FAMILY 5
# define EAI_MEMORY 6
# define EAI_NODATA 7
# define EAI_NONAME 8
# define EAI_SERVICE 9
# define EAI_SOCKTYPE 10
# define EAI_SYSTEM 11
# define EAI_BADHINTS 12
# define EAI_PROTOCOL 13
# define EAI_OVERFLOW 14
*/

// for EAI_ADDRFAMILY, EAI_AGAIN, ... etc.
typedef int eai_error_t;
typedef struct addrinfo nlib_addrinfo;
NLIB_VIS_PUBLIC
eai_error_t nlib_getaddrinfo(const char* __restrict node, const char* __restrict service,
                             const nlib_addrinfo* __restrict hints, nlib_addrinfo** __restrict res);
NLIB_VIS_PUBLIC
eai_error_t
nlib_getnameinfo(const nlib_sockaddr* __restrict sa, uint32_t salen, char* __restrict host,
                 uint32_t hostlen, char* __restrict serv, uint32_t servlen, int flags);
NLIB_VIS_PUBLIC void nlib_freeaddrinfo(nlib_addrinfo* res);

NLIB_VIS_PUBLIC
errno_t nlib_getsockopt(nlib_sock sockfd, int level, int optname, void* __restrict optval,
                        uint32_t* __restrict optlen);
NLIB_VIS_PUBLIC
errno_t
nlib_setsockopt(nlib_sock sockfd, int level, int optname, const void* optval, uint32_t optlen);

typedef fd_set nlib_fd_set;
NLIB_VIS_PUBLIC
errno_t nlib_select(size_t* __restrict n, int nfds, nlib_fd_set* __restrict readfds,
                    nlib_fd_set* __restrict writefds, nlib_fd_set* __restrict exceptfds,
                    nlib_duration timeout);

#define NLIB_FD_CLR FD_CLR
#define NLIB_FD_ISSET FD_ISSET
#ifdef _MSC_VER
#define NLIB_FD_SET(fd, set) FD_SET((SOCKET)fd, set)
#else
#define NLIB_FD_SET FD_SET
#endif
#define NLIB_FD_ZERO FD_ZERO

#if defined(_MSC_VER)
struct nlib_pollfd_ {
    nlib_sock fd;
    int16_t events;
    int16_t revents;
};
typedef struct nlib_pollfd_ nlib_pollfd;
#else
typedef struct pollfd nlib_pollfd;
#endif
NLIB_VIS_PUBLIC
errno_t
nlib_poll(size_t* __restrict n, nlib_pollfd* __restrict fds, uint32_t nfds, nlib_duration timeout);

#ifdef _MSC_VER
#define NLIB_MSG_DONTWAIT (0x4000)
#else
#define NLIB_MSG_DONTWAIT MSG_DONTWAIT
#endif

#ifdef MSG_FASTOPEN
#define NLIB_MSG_FASTOPEN MSG_FASTOPEN
#else
#define NLIB_MSG_FASTOPEN 0x20000000
#endif

// NLIB_VIS_PUBLIC errno_t nlib_sockatmark(int* is_marked, nlib_sock sockfd);
NLIB_VIS_PUBLIC
errno_t
nlib_getsockname(nlib_sock sockfd, nlib_sockaddr* __restrict addr, uint32_t* __restrict addrlen);
NLIB_VIS_PUBLIC
errno_t
nlib_getpeername(nlib_sock sockfd, nlib_sockaddr* __restrict addr, uint32_t* __restrict addrlen);
NLIB_VIS_PUBLIC errno_t nlib_setnonblocking(nlib_sock sockfd, int nonblock);

#endif

#endif  // INCLUDE_NN_NLIB_PLATFORM_SOCKET_H_
