﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMPLESINGLETON_H_
#define INCLUDE_NN_NLIB_SIMPLESINGLETON_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN

template<class T>
class SimpleSingleton NLIB_FINAL {
 private:
    // This ensures Instance() called before main()
    struct Creator {
        Creator() NLIB_NOEXCEPT { SimpleSingleton<T>::Instance(); }
        inline void DummyFunc() const NLIB_NOEXCEPT {}
    };
    static Creator creator;

 public:
    static T& Instance() NLIB_NOEXCEPT {
        static T obj;

        // This ensures 'obj' is already constructed
        // when called from a static object's constructor.
        creator.DummyFunc();
        return obj;
    }
    static T& GetInstance() NLIB_NOEXCEPT { return Instance(); }

 private:
    SimpleSingleton();
    NLIB_DISALLOW_COPY_AND_ASSIGN(SimpleSingleton);
};

template<class T>
typename SimpleSingleton<T>::Creator SimpleSingleton<T>::creator;

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SIMPLESINGLETON_H_
