﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_SOCKET_ENABLED

#include "nn/nlib/Config.h"
#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

struct TcpOutputStreamSettings {
    void* buf;
    size_t buffer_size;
    TcpOutputStreamSettings() : buf(nullptr), buffer_size(8192) {}
};

class NLIB_VIS_PUBLIC TcpOutputStream NLIB_FINAL : public OutputStream {
 public:
    TcpOutputStream() NLIB_NOEXCEPT : socket_(NLIB_SOCKET_INVALID),
                                      buf_(nullptr),
                                      buf_size_(0),
                                      buffer_holder_(nullptr) {}
    virtual ~TcpOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(nlib_sock sockfd) NLIB_NOEXCEPT {
        TcpOutputStreamSettings settings;
        return this->Init(sockfd, settings);
    }
    errno_t Init(nlib_sock sockfd, const TcpOutputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    virtual bool
    PushBuffer_(const void* p, size_t nbytes, bool do_flush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_sock socket_;
    uint8_t* buf_;
    size_t buf_size_;
    uint8_t* buffer_holder_;
};

NLIB_NAMESPACE_END

#endif
#endif  // INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_
