﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TYPETRAITS_H_
#define INCLUDE_NN_NLIB_TYPETRAITS_H_

#include "nn/nlib/Config.h"

#ifndef NLIB_HAS_NATIVE_TYPETRAITS

#ifdef __GLIBCXX__
#include <type_traits>
#endif

NLIB_NAMESPACE_BEGIN

// for std::integral_constant
template<class T, T v>
struct IntegralConstant {
    static const T value = v;
    typedef T value_type;
    typedef IntegralConstant type;
    operator value_type() const { return value; }
};

// for std::true_type, std::false_type
typedef IntegralConstant<bool, true> TrueType;
typedef IntegralConstant<bool, false> FalseType;

#define NLIB_DEF_TRUETYPE(name, tp) \
    template<>                      \
    struct name<tp> {               \
        typedef TrueType type;      \
    }

namespace tt_detail {

template<class T>
struct my_or1 {
    typedef T type;
};

template<class T1, class T2>
struct my_or2;
template<class T>
struct my_or2<TrueType, T> {
    typedef TrueType type;
};
template<class T>
struct my_or2<FalseType, T> {
    typedef typename my_or1<T>::type type;
};

template<class T1, class T2, class T3>
struct my_or3;
template<class T1, class T2>
struct my_or3<TrueType, T1, T2> {
    typedef TrueType type;
};
template<class T1, class T2>
struct my_or3<FalseType, T1, T2> {
    typedef typename my_or2<T1, T2>::type type;
};

template<class T1, class T2, class T3, class T4>
struct my_or4;
template<class T1, class T2, class T3>
struct my_or4<TrueType, T1, T2, T3> {
    typedef TrueType type;
};
template<class T1, class T2, class T3>
struct my_or4<FalseType, T1, T2, T3> {
    typedef typename my_or3<T1, T2, T3>::type type;
};

template<class T1 = FalseType, class T2 = FalseType, class T3 = FalseType, class T4 = FalseType>
struct my_or {
    typedef typename my_or4<T1, T2, T3, T4>::type type;
};

}  // namespace tt_detail

// for std::enable_if
template<bool B, class T = void>
struct EnableIf {};
template<class T>
struct EnableIf<true, T> {
    typedef T type;
};

// for std::conditional
template<bool B, class T, class F>
struct Conditional {
    typedef T type;
};
template<class T, class F>
struct Conditional<false, T, F> {
    typedef F type;
};

// for std::remove_cv
template<class T>
struct RemoveConst {
    typedef T type;
};
template<class T>
struct RemoveConst<const T> {
    typedef T type;
};

template<class T>
struct RemoveVolatile {
    typedef T type;
};
template<class T>
struct RemoveVolatile<volatile T> {
    typedef T type;
};

template<class T>
struct RemoveCv {
    typedef typename RemoveVolatile<typename RemoveConst<T>::type>::type type;
};

// for std::remove_pointer
template<class T>
struct RemovePtr {
    typedef T type;
};
template<class T>
struct RemovePtr<T*> {
    typedef T type;
};
template<class T>
struct RemovePtr<const T*> {
    typedef T type;
};
template<class T>
struct RemovePtr<volatile T*> {
    typedef T type;
};
template<class T>
struct RemovePtr<const volatile T*> {
    typedef T type;
};

// for std::remove_reference
template<class T>
struct RemoveRef {
    typedef T type;
};
template<class T>
struct RemoveRef<T&> {
    typedef T type;
};

template<class T>
struct AddRef {
    typedef T& type;
};
template<>
struct AddRef<void> {
    typedef void type;
};

// for std::is_same
template<class T1, class T2>
struct IsSame : public FalseType {};
template<class T>
struct IsSame<T, T> : public TrueType {};

// for std::is_void
namespace tt_detail {
template<class T>
struct is_void {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_void, void);
}  // namespace tt_detail

template<class T>
struct IsVoid : public tt_detail::is_void<typename RemoveCv<T>::type>::type {};

// for std::is_integral
namespace tt_detail {

template<class T>
struct is_integral_ex {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_integral_ex, signed char);
NLIB_DEF_TRUETYPE(is_integral_ex, signed short);
NLIB_DEF_TRUETYPE(is_integral_ex, signed int);
NLIB_DEF_TRUETYPE(is_integral_ex, signed long);
NLIB_DEF_TRUETYPE(is_integral_ex, signed long long);

template<class T>
struct is_integral {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_integral, bool);
NLIB_DEF_TRUETYPE(is_integral, char);
NLIB_DEF_TRUETYPE(is_integral, wchar_t);
NLIB_DEF_TRUETYPE(is_integral, short);
NLIB_DEF_TRUETYPE(is_integral, int);
NLIB_DEF_TRUETYPE(is_integral, long);
NLIB_DEF_TRUETYPE(is_integral, long long);

NLIB_DEF_TRUETYPE(is_integral, unsigned char);
NLIB_DEF_TRUETYPE(is_integral, unsigned short);
NLIB_DEF_TRUETYPE(is_integral, unsigned int);
NLIB_DEF_TRUETYPE(is_integral, unsigned long long);
NLIB_DEF_TRUETYPE(is_integral, unsigned long);
}  // namespace tt_detail

template<class T>
struct IsIntegral
    : public tt_detail::my_or<
          typename tt_detail::is_integral<typename RemoveCv<T>::type>::type,
          typename tt_detail::is_integral_ex<typename RemoveCv<T>::type>::type>::type {};

// for std::is_signed
namespace tt_detail {

template<class T>
struct is_signed_ex {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_signed_ex, signed char);
NLIB_DEF_TRUETYPE(is_signed_ex, signed short);
NLIB_DEF_TRUETYPE(is_signed_ex, signed int);
NLIB_DEF_TRUETYPE(is_signed_ex, signed long);
NLIB_DEF_TRUETYPE(is_signed_ex, signed long long);

template<class T>
struct is_signed {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_signed, char);
NLIB_DEF_TRUETYPE(is_signed, short);
NLIB_DEF_TRUETYPE(is_signed, int);
NLIB_DEF_TRUETYPE(is_signed, long);
NLIB_DEF_TRUETYPE(is_signed, long long);
NLIB_DEF_TRUETYPE(is_signed, float);
NLIB_DEF_TRUETYPE(is_signed, double);
}  // namespace tt_detail

template<class T>
struct IsSigned : public tt_detail::my_or<
                      typename tt_detail::is_signed<typename RemoveCv<T>::type>::type,
                      typename tt_detail::is_signed_ex<typename RemoveCv<T>::type>::type>::type {};

template<class T>
struct IsUnsigned {
    typedef FalseType type;
};
template<>
struct IsUnsigned<bool> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<wchar_t> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<unsigned char> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<unsigned short> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<unsigned int> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<unsigned long> {
    typedef TrueType type;
};
template<>
struct IsUnsigned<unsigned long long> {
    typedef TrueType type;
};

// for std::is_floating_point
namespace tt_detail {

template<class T>
struct is_floating_point {
    typedef FalseType type;
};
NLIB_DEF_TRUETYPE(is_floating_point, float);
NLIB_DEF_TRUETYPE(is_floating_point, double);
NLIB_DEF_TRUETYPE(is_floating_point, long double);

}  // namespace tt_detail

template<class T>
struct IsFloatingPoint : public tt_detail::is_floating_point<typename RemoveCv<T>::type>::type {};

// for std::is_arithmetic
template<class T>
struct IsArithmetic
    : public IntegralConstant<bool, IsIntegral<T>::value || IsFloatingPoint<T>::value> {};

// for std::is_pointer
namespace tt_detail {
template<class T>
struct IsPointerHelper : public FalseType {};
template<class T>
struct IsPointerHelper<T*> : public TrueType {};
}  // namespace tt_detail
template<class T>
struct IsPointer : public tt_detail::IsPointerHelper<typename RemoveCv<T>::type> {};

// for std::is_reference
template<class T>
struct IsReference : public FalseType {};
template<class T>
struct IsReference<T&> : public TrueType {};

// for std::is_array
template<class T>
struct IsArray : public FalseType {};
template<class T>
struct IsArray<T[]> : public TrueType {};
template<class T, size_t N>
struct IsArray<T[N]> : public TrueType {};

// for std::is_member_pointer
namespace tt_detail {
template<class T>
struct is_member_pointer : public FalseType {};
template<class T, class U>
struct is_member_pointer<T U::*> : public TrueType {};
}  // namespace tt_detail
template<class T>
struct IsMemberPointer : public tt_detail::is_member_pointer<typename RemoveCv<T>::type> {};

// for std::is_function
namespace tt_detail {
template<bool B>
struct is_function_ {
    typedef FalseType type;
};
template<>
struct is_function_<true> {
    typedef TrueType type;
};
template<class T>
struct is_function {
 private:
    typedef char One;
    typedef struct {
        char a[2];
    } Two;
    template<class U>
    static One test(...);
    template<class U>
    static Two test(U (*)[1]);

 public:
#ifdef CAFE
#pragma diag_suppress 1931
#endif
    typedef typename is_function_<sizeof(test<T>(0)) == 1>::type type;
#ifdef CAFE
#pragma diag_warning 1931
#endif
};
template<class T>
struct is_function<T&> {
    typedef FalseType type;
};
template<>
struct is_function<void> {
    typedef FalseType type;
};
template<>
struct is_function<const void> {
    typedef FalseType type;
};
template<>
struct is_function<volatile void> {
    typedef FalseType type;
};
template<>
struct is_function<const volatile void> {
    typedef FalseType type;
};
}  // namespace tt_detail
template<class T>
struct IsFunction : public tt_detail::is_function<T>::type {};

#if !defined(NN_PLATFORM_CTR)
template<class T>
struct IsEnum : public IntegralConstant<bool, __is_enum(T)> {};
#else
// for std::is_enum
namespace tt_detail {
typedef char (&SizeOverOne)[2];
template<class T, bool convert_possible = !IsArithmetic<T>::value && !IsReference<T>::value &&
                                          !IsPointer<T>::value && !IsMemberPointer<T>::value &&
                                          !IsFunction<T>::value && !IsVoid<T>::value &&
                                          !IsArray<T>::value>
struct ConsumeUDC {
    operator T() const;
};
template<class T>
struct ConsumeUDC<T, false> {
    operator SizeOverOne() const;
};
char enum_check(bool x);
char enum_check(char x);
char enum_check(signed char x);
char enum_check(unsigned char x);
char enum_check(wchar_t x);
char enum_check(signed short x);
char enum_check(unsigned short x);
char enum_check(signed int x);
char enum_check(unsigned int x);
char enum_check(signed long x);
char enum_check(unsigned long x);
char enum_check(signed long long x);
char enum_check(unsigned long long x);
char enum_check(float x);
char enum_check(double x);
char enum_check(long double x);
SizeOverOne enum_check(SizeOverOne x);
SizeOverOne enum_check(...);
template<class T>
struct is_enum {
    enum { value = sizeof(enum_check(ConsumeUDC<T>())) == 1 };
};
}  // namespace tt_detail

template<class T>
struct IsEnum : public IntegralConstant<bool, tt_detail::is_enum<T>::value> {};
#endif

#if !defined(NN_PLATFORM_CTR) && !defined(CAFE)
template<class T>
struct IsPod
    : public IntegralConstant<bool, __is_pod(T) || IsEnum<T>::value || IsArithmetic<T>::value ||
                                        IsPointer<T>::value || IsMemberPointer<T>::value> {};

template<class T>
struct IsTriviallyDestructible : public IntegralConstant<bool, __has_trivial_destructor(T)> {};
#ifdef _MSC_VER
template<class T>
struct IsTriviallyDefaultConstructible
    : public IntegralConstant<bool, __has_trivial_default_constructor(T)> {};
#else
template<class T>
struct IsTriviallyDefaultConstructible
    : public IntegralConstant<bool, std::has_trivial_default_constructor<T>::value> {};
#endif

#else
// for std::is_pod
namespace tt_detail {

template<class T>
struct is_pod
    : public tt_detail::my_or<typename IsArithmetic<T>::type, typename IsPointer<T>::type,
                              typename IsMemberPointer<T>::type, typename IsEnum<T>::type>::type {};

template<class T>
struct is_pod<T[]> {
    typedef typename is_pod<typename RemoveCv<T>::type>::type type;
};
template<class T, size_t N>
struct is_pod<T[N]> {
    typedef typename is_pod<typename RemoveCv<T>::type>::type type;
};

}  // namespace tt_detail

// NOTE: internal use only
//   treats enum as non-POD.
//   treats struct as non-POD.
//
//   You have to define specialized IsPod when you have to use IsPos on struct.
//     example:
//     template<> struct IsPod<MyType> : public TrueType {};
template<class T>
struct IsTriviallyDestructible : public tt_detail::is_pod<typename RemoveCv<T>::type>::type {};
template<class T>
struct IsTriviallyDefaultConstructible
    : public tt_detail::is_pod<typename RemoveCv<T>::type>::type {};

template<class T>
struct IsPod : public IsTriviallyDestructible<T> {};
#endif

#if !defined(NN_PLATFORM_CTR)
template<class T>
struct IsEmpty : public IntegralConstant<bool, __is_empty(T)> {};
#else
// for std::is_empty
namespace tt_detail {
template<class T>
struct IsEmptyHelper1 : public T {
    int data[256];

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(IsEmptyHelper1);
};

struct IsEmptyHelper2 {
    int data[256];
};
}  // namespace tt_detail

template<class T>
struct IsEmpty : public IntegralConstant<bool, sizeof(tt_detail::IsEmptyHelper1<T>) ==
                                                   sizeof(tt_detail::IsEmptyHelper2)> {};
#endif

#if !defined(NN_PLATFORM_CTR)
template<class T>
struct IsClass : public IntegralConstant<bool, __is_class(T)> {};
#else
// note that IsClass<union type>::value becomes true
template<class T>
class IsClass {
    typedef char yes;
    typedef struct {
        char a[2];
    } no;

    template<class X>
    static yes check(void (X::*)(void));
    template<class X>
    static no check(...);

 public:
    static const bool value = sizeof(check<T>(NULL)) == sizeof(yes);
};
#endif

// template<class T> struct AddLvalueReference { typedef T& type; };
// template<class T> struct AddLvalueReference<T&> { typedef T& type; };

NLIB_NAMESPACE_END

#undef NLIB_DEF_TRUETYPE
#else

#include <type_traits>
#include <utility>
NLIB_NAMESPACE_BEGIN

#ifdef NLIB_HAS_TR1_TYPETRAITS
#define NLIB_DEF_REDIRECT(nlibstruct, stdstruct) \
                                                 \
    template<class T>                            \
    struct nlibstruct : public std::tr1::stdstruct<T> {}
template<class T, T v>
struct IntegralConstant : public std::tr1::integral_constant<T, v> {};
typedef std::tr1::true_type TrueType;
typedef std::tr1::false_type FalseType;
template<bool B, class T = void>
struct EnableIf {};
template<class T>
struct EnableIf<true, T> {
    typedef T type;
};
template<bool B, class T, class F>
struct Conditional {
    typedef T type;
};
template<class T, class F>
struct Conditional<false, T, F> {
    typedef F type;
};
template<class T1, class T2>
struct IsSame : public std::tr1::is_same<T1, T2> {};

NLIB_DEF_REDIRECT(IsTriviallyDestructible, has_trivial_destructor);
NLIB_DEF_REDIRECT(IsTriviallyDefaultConstructible, has_trivial_default_constructor);

#else
#define NLIB_DEF_REDIRECT(nlibstruct, stdstruct) \
                                                 \
    template<class T>                            \
    struct nlibstruct : public std::stdstruct<T> {}
template<class T, T v>
struct IntegralConstant : public std::integral_constant<T, v> {};
typedef std::true_type TrueType;
typedef std::false_type FalseType;
template<bool B, class T>
struct EnableIf : public std::enable_if<B, T> {};
template<bool B, class T, class F>
struct Conditional : public std::conditional<B, T, F> {};
template<class T1, class T2>
struct IsSame : public std::is_same<T1, T2> {};

// still unimplemented
// #if defined(__GLIBCXX__) && __GLIBCXX__ <= 20141011
#if defined(__GLIBCXX__)
template<class T>
struct IsTriviallyDestructible : public IntegralConstant<bool, __has_trivial_destructor(T)> {};
template<class T>
struct IsTriviallyDefaultConstructible
    : public IntegralConstant<bool, __has_trivial_constructor(T)> {};
#else
NLIB_DEF_REDIRECT(IsTriviallyDestructible, is_trivially_destructible);
NLIB_DEF_REDIRECT(IsTriviallyDefaultConstructible, is_trivially_constructible);
#endif

#endif

NLIB_DEF_REDIRECT(RemoveConst, remove_const);
NLIB_DEF_REDIRECT(RemoveVolatile, remove_volatile);
NLIB_DEF_REDIRECT(RemovePtr, remove_pointer);
NLIB_DEF_REDIRECT(RemoveRef, remove_reference);
NLIB_DEF_REDIRECT(AddRef, add_lvalue_reference);
NLIB_DEF_REDIRECT(RemoveCv, remove_cv);
NLIB_DEF_REDIRECT(IsVoid, is_void);
NLIB_DEF_REDIRECT(IsIntegral, is_integral);
NLIB_DEF_REDIRECT(IsSigned, is_signed);
NLIB_DEF_REDIRECT(IsUnsigned, is_unsigned);
NLIB_DEF_REDIRECT(IsFloatingPoint, is_floating_point);
NLIB_DEF_REDIRECT(IsArithmetic, is_arithmetic);
NLIB_DEF_REDIRECT(IsPointer, is_pointer);
NLIB_DEF_REDIRECT(IsReference, is_reference);
NLIB_DEF_REDIRECT(IsArray, is_array);
NLIB_DEF_REDIRECT(IsMemberPointer, is_member_pointer);
NLIB_DEF_REDIRECT(IsFunction, is_function);
NLIB_DEF_REDIRECT(IsEnum, is_enum);

NLIB_DEF_REDIRECT(IsPod, is_pod);
NLIB_DEF_REDIRECT(IsEmpty, is_empty);
NLIB_DEF_REDIRECT(IsClass, is_class);

NLIB_NAMESPACE_END

#undef NLIB_DEF_REDIRECT
#endif

NLIB_NAMESPACE_BEGIN

#if (defined(_MSVC_LANG) && _MSVC_LANG > 201402L) || __cplusplus >= 201703L
template<typename T>
using IsSwappable = std::is_swappable<T>;
#elif defined(NLIB_HAS_NATIVE_TYPETRAITS) && defined(_MSC_VER)
// https://stackoverflow.com/questions/26744589/what-is-a-proper-way-to-implement-is-swappable-to-test-for-the-swappable-concept
namespace detail {
using std::swap;
template<typename T>
struct can_call_swap_impl {
    template<typename U>
    static auto check(int)
        -> decltype(swap(std::declval<T&>(), std::declval<T&>()), std::true_type());
    template<typename>
    static std::false_type check(...);
    using type = decltype(check<T>(0));
};
template<typename T>
struct can_call_swap : can_call_swap_impl<T>::type {};
}  // namespace detail
namespace detail2 {
struct tag {};
template<class T>
tag swap(T&, T&);
template<typename T>
struct would_call_std_swap_impl {
    template<typename U>
    static auto check(int) -> std::integral_constant<
        bool, std::is_same<decltype(swap(std::declval<U&>(), std::declval<U&>())), tag>::value>;
    template<typename>
    static std::false_type check(...);
    using type = decltype(check<T>(0));
};
template<typename T>
struct would_call_std_swap : would_call_std_swap_impl<T>::type {};
}  // namespace detail2
template<typename T>
struct IsSwappable : std::integral_constant<bool, detail::can_call_swap<T>::value &&
                                                      (!detail2::would_call_std_swap<T>::value ||
                                                       (std::is_move_assignable<T>::value &&
                                                        std::is_move_constructible<T>::value))> {};
template<typename T, size_t N>
struct IsSwappable<T[N]> : IsSwappable<T> {};
#else
namespace detail {

template<class T>
class HasSwapMemFn {
    typedef char yes;
    typedef struct {
        char a[2];
    } no;

    template<class X, T& (X::*Func)(T&, move_tag)>
    struct helper {};
    template<class X>
    static yes check(helper<X, &X::assign>* p);
    template<class X>
    static no check(...);

 public:
#ifdef CAFE
#pragma diag_suppress 1931
#endif
    static const bool value = sizeof(check<T>(NULL)) == sizeof(yes);
#ifdef CAFE
#pragma diag_warning 1931
#endif
};

template<class T, bool isClass>
struct IsSwappable_ : public IntegralConstant<bool, detail::HasSwapMemFn<T>::value> {};

template<class T>
struct IsSwappable_<T, false>
    : public IntegralConstant<bool, IsArithmetic<T>::value || IsPointer<T>::value ||
                                        IsFunction<T>::value || IsMemberPointer<T>::value ||
                                        IsEnum<T>::value> {};

}  // namespace detail

// true if T has swap(T&) member function
// or T is a simple value.
// T may be swappable if IsSwappable<T>::value is false.
template<class T>
struct IsSwappable : public detail::IsSwappable_<T, IsClass<T>::value> {};

#endif
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_TYPETRAITS_H_
