﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_URITEMPLATE_H_
#define INCLUDE_NN_NLIB_URITEMPLATE_H_

#include <utility>

#include "nn/nlib/Uri.h"
#include "nn/nlib/Swap.h"

NLIB_NAMESPACE_BEGIN

class OutputStream;

// See RFC6570: https://tools.ietf.org/html/rfc6570
class NLIB_VIS_PUBLIC UriTemplate NLIB_FINAL {
 public:
    static bool IsUriTemplate(const char* first, const char* last) NLIB_NOEXCEPT;
    static bool IsUriTemplate(const char* str) NLIB_NOEXCEPT {
        return IsUriTemplate(str, str + nlib_strlen(str));
    }
    NLIB_CEXPR UriTemplate() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~UriTemplate() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(UriTemplate);
    void Reset() NLIB_NOEXCEPT;
    errno_t SetTemplate(const char* first, const char* last) NLIB_NOEXCEPT;
    errno_t SetTemplate(const char* str) NLIB_NOEXCEPT {
        return SetTemplate(str, str + nlib_strlen(str));
    }
    errno_t SetParameter(const char* param, const char* value) NLIB_NOEXCEPT;
    void ResetParameter() NLIB_NOEXCEPT;
    errno_t Resolve(size_t* written, char* str, size_t n) const NLIB_NOEXCEPT;
    template<size_t N>
    errno_t Resolve(size_t* written, char (&str)[N]) const NLIB_NOEXCEPT {
        return Resolve(written, str, N);
    }
    std::pair<errno_t, size_t> Resolve(char* str, size_t n) const NLIB_NOEXCEPT {
        size_t written;
        errno_t e = Resolve(&written, str, n);
        return std::make_pair(e, written);
    }
    template<size_t N>
    std::pair<errno_t, size_t> Resolve(char (&str)[N]) const NLIB_NOEXCEPT {
        size_t written;
        errno_t e = Resolve(&written, str, N);
        return std::make_pair(e, written);
    }

 private:
    struct UriTemplatePrivate;
    mutable UriTemplatePrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(UriTemplate);
};

NLIB_NAMESPACE_END
#ifndef __cpp_rvalue_references
NLIB_DEFINE_STD_SWAP(::nlib_ns::UriTemplate)
#endif
#endif  // INCLUDE_NN_NLIB_URITEMPLATE_H_
