﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_HAS_ZLIB

#include "nn/nlib/InputStream.h"
#include "nn/nlib/InputConverterStream.h"

NLIB_NAMESPACE_BEGIN

struct ZlibInputStreamSettings {
    typedef void* (*ReallocFunc)(void* p, size_t n)NLIB_NOEXCEPT_FUNCPTR;
    void* buf_in;
    void* buf_out;
    size_t buffer_size;
    ReallocFunc realloc_func;

 public:
    NLIB_CEXPR ZlibInputStreamSettings() NLIB_NOEXCEPT : buf_in(nullptr),
                                                         buf_out(nullptr),
                                                         buffer_size(64 * 1024),
                                                         realloc_func(::nlib_realloc) {}
    NLIB_CEXPR
    ZlibInputStreamSettings(void* buf_in_, void* buf_out_, size_t buffer_size_) NLIB_NOEXCEPT
        : buf_in(buf_in_),
          buf_out(buf_out_),
          buffer_size(buffer_size_),
          realloc_func(::nlib_realloc) {}
    NLIB_CEXPR ZlibInputStreamSettings(void* buf_in_, void* buf_out_, size_t buffer_size_,
                                       ReallocFunc realloc_func_) NLIB_NOEXCEPT
        : buf_in(buf_in_),
          buf_out(buf_out_),
          buffer_size(buffer_size_),
          realloc_func(realloc_func_) {}
};

class NLIB_VIS_PUBLIC ZlibInputStream NLIB_FINAL : public InputConverterStream {
 public:
    NLIB_CEXPR ZlibInputStream() NLIB_NOEXCEPT : prv_(nullptr) {}
    virtual ~ZlibInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR ZlibInputStreamSettings settings;
        return this->Init(settings);
    }
    errno_t Init(const ZlibInputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    struct ZlibInputStreamPrivate;
    ZlibInputStreamPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ZlibInputStream);
};

NLIB_NAMESPACE_END

#endif
#endif  // INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_
