﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_
#define INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_

#include "nn/nlib/exi/ExiAllocator.h"
#include "nn/nlib/exi/Types.h"

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace exi {

class NLIB_VIS_PUBLIC ExiErrorStatus NLIB_FINAL {
 public:
    enum ErrorValue {
        kOk = 0,
        kOutOfMemory,
        kArgumentError,
        kIoError,
        kUnexpectedEos,
        kStreamDetached,
        kIllegalCharacter,
        kInternalError,
        kNotSupported,
        kDuplicateAttribute,
        kReservedPrefix,
        kUndeclaringPrefix,
        kAttributeDefaultNs,
        kIllegalComment,
        kIllegalPi,
        kIllegalBinary,
        kIllegalBinaryIdx,
        kIllegalBinaryEvt,
        kSyntaxError,
        kIllegalBinaryHdr,
        kFuncNotAvailable,
        OK                   = kOk,
        OUT_OF_MEMORY        = kOutOfMemory,
        ARGUMENT_ERROR       = kArgumentError,
        IO_ERROR             = kIoError,
        UNEXPECTED_EOS       = kUnexpectedEos,
        STREAM_DETACHED      = kStreamDetached,
        ILLEGAL_CHARACTER    = kIllegalCharacter,
        INTERNAL_ERROR       = kInternalError,
        NOT_SUPPORTED        = kNotSupported,
        DUPLICATE_ATTRIBUTE  = kDuplicateAttribute,
        RESERVED_PREFIX      = kReservedPrefix,
        UNDECLARING_PREFIX   = kUndeclaringPrefix,
        ATTRIBUTE_DEFAULT_NS = kAttributeDefaultNs,
        ILLEGAL_COMMENT      = kIllegalComment,
        ILLEGAL_PI           = kIllegalPi,
        ILLEGAL_BINARY       = kIllegalBinary,
        ILLEGAL_BINARY_IDX   = kIllegalBinaryIdx,
        ILLEGAL_BINARY_EVT   = kIllegalBinaryEvt,
        SYNTAX_ERROR         = kSyntaxError,
        ILLEGAL_BINARY_HDR   = kIllegalBinaryHdr,
        FUNC_NOT_AVAILABLE   = kFuncNotAvailable
    };

 public:
    ExiErrorStatus() NLIB_NOEXCEPT : error_(OK), line_(0), filename_(nullptr) {}
    ErrorValue GetError() const NLIB_NOEXCEPT { return static_cast<ErrorValue>(error_); }
    int GetLineNo() const NLIB_NOEXCEPT { return line_; }
    const char* GetFileName() const NLIB_NOEXCEPT { return filename_; }
    bool SetError(ErrorValue e, int line, const char* filename) NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT;
    NLIB_SAFE_BOOL(ExiErrorStatus, error_ == OK)

 private:
    int error_;             // ErrorValue
    int line_;              // __LINE__
    const char* filename_;  // __FILE__
    NEXI_DISALLOW_NEW_DELETE(ExiErrorStatus);
    NLIB_DISALLOW_COPY_AND_ASSIGN(ExiErrorStatus);
};

// for unit-tests, make NEXI_SET_ERROR() work if ErrorStatus is not set.
#define NEXI_SET_ERROR(e)                                                                     \
    (this->GetExiErrorStatus() ? this->GetExiErrorStatus()->SetError((e), __LINE__, __FILE__) \
                               : false)
#define NEXI_IS_ERROR (nlib_is_error(*this->GetExiErrorStatus()))
#define NEXI_RESET_ERROR this->GetExiErrorStatus()->Reset()
#define NEXI_CHECK_OUT_OF_MEMORY \
    if (ExiAllocator::IsOutOfMemory()) NEXI_SET_ERROR(ExiErrorStatus::OUT_OF_MEMORY)

}  // namespace exi
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_
