﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_
#define INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_

#include "nn/nlib/Config.h"

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace heap {

class NLIB_VIS_PUBLIC UnitHeap NLIB_FINAL {
 public:
    typedef void (*DestructorHandler)(void* start, void* end) NLIB_NOEXCEPT_FUNCPTR;
    UnitHeap() NLIB_NOEXCEPT;
    ~UnitHeap() NLIB_NOEXCEPT {
        // Do not assert by GetAllocatedCount().
        // Allow deallote without destroy.
        if (handler_) {
            (*handler_)(start_, end_);
        }
    }
    errno_t Init(void* start, size_t size, DestructorHandler handler, size_t unit, size_t algn,
                 uint32_t heap_option) NLIB_NOEXCEPT;
    void* Alloc() NLIB_NOEXCEPT;
    errno_t Free(void* p) NLIB_NOEXCEPT;
    size_t GetUnitSize() const NLIB_NOEXCEPT { return unit_size_; }
    size_t GetUnitCount() const NLIB_NOEXCEPT { return unit_count_; }
    size_t GetAllocatedCount() const NLIB_NOEXCEPT { return allocated_count_; }
    void Dump() NLIB_NOEXCEPT;

 private:
    struct Node {
        Node* next;
    };

    size_t unit_size_;
    size_t baseunit_count_;
    Node* free_list_;
    size_t unit_count_;
    size_t allocated_count_;

    uint32_t heap_option_;
    uint8_t* start_;
    uint8_t* end_;
    DestructorHandler handler_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(UnitHeap);
};

}  // namespace heap
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_
