﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_
#define INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_

#include <utility>

#include "nn/nlib/msgpack/MpObject.h"
#include "nn/nlib/Swap.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
class InputStream;
namespace msgpack {

class NLIB_VIS_PUBLIC CsvReader NLIB_FINAL {
 public:
    // In Conformance with RFC4180(https://www.ietf.org/rfc/rfc4180.txt).
    // except that
    //  pass UTF-8, the UTF-8 string is checked validity.
    //  pass CR, LF, CRLF as a new line.
    NLIB_CEXPR CsvReader() NLIB_NOEXCEPT : prv_(nullptr), error_(0) {}
    ~CsvReader() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(CsvReader);
    void Reset() NLIB_NOEXCEPT;
    errno_t Init() NLIB_NOEXCEPT;
    errno_t Open(InputStream* stream) NLIB_NOEXCEPT;
    bool Close() NLIB_NOEXCEPT;
    errno_t GetErrorValue() const NLIB_NOEXCEPT { return error_; }

 public:
    // to be derecated
    static bool Read(MpObject* obj, const nlib_utf8_t* csvtext) NLIB_NOEXCEPT NLIB_NONNULL;
#ifdef __cpp_rvalue_references
    static std::pair<errno_t, std::unique_ptr<MpObject> >
    Read(const nlib_utf8_t* csvtext) NLIB_NOEXCEPT;
#endif
    bool Read(MpObject* obj) NLIB_NOEXCEPT NLIB_NONNULL;
    InputStream* GetStream() NLIB_NOEXCEPT;
    NLIB_SAFE_BOOL(CsvReader, error_ == 0)

 private:
    void SetError(errno_t e) NLIB_NOEXCEPT {
        if (error_ == 0) error_ = e;
    }

 private:
    struct CsvReaderPrivate;
    CsvReaderPrivate* prv_;
    errno_t error_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(CsvReader);
};

}  // namespace msgpack
NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::CsvReader)
#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_
