﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/
#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONSCHEMA_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONSCHEMA_H_

#include <utility>
#include "nn/nlib/NativePathMapper.h"
#include "nn/nlib/msgpack/MpObject.h"
#include "nn/nlib/msgpack/JsonStreamParser.h"
#include "nn/nlib/Swap.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif
NLIB_NAMESPACE_BEGIN
namespace msgpack {

class JsonSchemaValidatorPrivate;
class NLIB_VIS_PUBLIC JsonSchemaValidator NLIB_FINAL {
 public:
    enum Result {
        kOk = 0,
        kInvalidParam,
        kAlreadyInitialized,
        kOutOfMemory,
        kFail,
        kInvalidSchema,
        kVersionNotSupported
    };
    struct Detail {
        nlib_utf8_t path[1024];
    };
    NLIB_CEXPR JsonSchemaValidator() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~JsonSchemaValidator() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(JsonSchemaValidator);
    void Reset() NLIB_NOEXCEPT;
    Result Init(const nlib_byte_t* schema_bytecode, size_t n) NLIB_NOEXCEPT;
    Result Validate(const MpObject& obj, Detail* detail) NLIB_NOEXCEPT;

 private:
    JsonSchemaValidatorPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonSchemaValidator);
};

struct JsonSchemaConverterSettings NLIB_FINAL {
    NLIB_CEXPR JsonSchemaConverterSettings() NLIB_NOEXCEPT : encode_title(false),
                                                             encode_description(false),
                                                             uri_mapper(nullptr),
                                                             parser_settings() {}
    bool encode_title;
    bool encode_description;
    const NativePathMapper* uri_mapper;
    JsonStreamParserSettings parser_settings;
};

class JsonSchemaConverterPrivate;
class NLIB_VIS_PUBLIC JsonSchemaConverter NLIB_FINAL {
 public:
    enum Result {
        kOk = 0,
        kInvalidParam,
        kAlreadyInitialized,
        kOutOfMemory,
        kInvalidSchema,
        kFileNotFound,
        kIoError,
        kUriError,
        kBufferSizeNotEnough
    };
    struct Detail {
        nlib_utf8_t path[1024];
    };
    NLIB_CEXPR JsonSchemaConverter() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~JsonSchemaConverter() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(JsonSchemaConverter);
    Result Init(const JsonSchemaConverterSettings& settings) NLIB_NOEXCEPT;
    Result Init() NLIB_NOEXCEPT {
        JsonSchemaConverterSettings settings;
        return Init(settings);
    }
    void Reset() NLIB_NOEXCEPT;
    Result Convert(const char* uri, Detail* detail) NLIB_NOEXCEPT;
    Result Convert(MpObject* schema, Detail* detail) NLIB_NOEXCEPT;
    Result Export(size_t* written, nlib_byte_t* buf, size_t bufsize) const NLIB_NOEXCEPT;
    template<size_t N>
    Result Export(size_t* written, nlib_byte_t (&buf)[N]) const NLIB_NOEXCEPT {
        return Export(written, &buf[0], N);
    }
#ifdef __cpp_rvalue_references
    std::tuple<Result, std::unique_ptr<nlib_byte_t[]>, size_t> Export() const NLIB_NOEXCEPT;
#endif

 private:
    JsonSchemaConverterPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonSchemaConverter);
};

}  // namespace msgpack
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::JsonSchemaConverter)
NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::JsonSchemaValidator)

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif
#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONSCHEMA_H_
