﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
#define INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_

#include <utility>
#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"

#ifdef __cpp_rvalue_references
#include <tuple>
#endif

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {

class MpObject;
class NLIB_VIS_PUBLIC MpWalker NLIB_FINAL {
 public:
    NLIB_CEXPR MpWalker() NLIB_NOEXCEPT : cur_(nullptr), size_(0) {}
    NLIB_CEXPR MpWalker(const void* p, size_t n) NLIB_NOEXCEPT
        : cur_(static_cast<const uint8_t*>(p)),
          size_(n) {}
    bool Init(const void* p, size_t n) NLIB_NOEXCEPT {
        if (n > RSIZE_MAX) return false;
        cur_  = static_cast<const uint8_t*>(p);
        size_ = n;
        return true;
    }

    MpWalker operator[](size_t array_idx) const NLIB_NOEXCEPT;
    MpWalker operator[](const nlib_utf8_t* key) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker operator[](int array_idx) const NLIB_NOEXCEPT {
        if (array_idx < 0)
            return MpWalker();
        else
            return operator[](static_cast<size_t>(array_idx));
    }
    const nlib_byte_t* GetPtr() const NLIB_NOEXCEPT {
        return reinterpret_cast<const nlib_byte_t*>(cur_);
    }
    size_t GetSize() const NLIB_NOEXCEPT { return size_; }

    MpWalker At(size_t idx) const NLIB_NOEXCEPT;
    MpWalker At(size_t idx, const nlib_utf8_t** key, size_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker Find(const nlib_utf8_t* key) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker Skip() const NLIB_NOEXCEPT;
    MpWalker GetString(const nlib_utf8_t** str, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetBinary(const void** bin, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetExt(int8_t* tp, const void** bin, uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetMapCount(uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetArrayCount(uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetNil() const NLIB_NOEXCEPT;
    MpWalker GetBoolean(bool* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetInt(int32_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetInt(int64_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetUint(uint32_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetUint(uint64_t* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetFloat(float* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetDouble(double* val) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker GetTimestamp(nlib_time* val) const NLIB_NOEXCEPT NLIB_NONNULL;

#ifdef __cpp_rvalue_references
    std::tuple<MpWalker, const nlib_utf8_t*, uint32_t> GetString() const NLIB_NOEXCEPT {
        std::tuple<MpWalker, const nlib_utf8_t*, uint32_t> rval;
        std::get<0>(rval) = GetString(&std::get<1>(rval), &std::get<2>(rval));
        return rval;
    }
    std::tuple<MpWalker, const nlib_byte_t*, uint32_t> GetBinary() const NLIB_NOEXCEPT {
        std::tuple<MpWalker, const nlib_byte_t*, uint32_t> rval;
        std::get<0>(rval) =
            GetBinary(reinterpret_cast<const void**>(&std::get<1>(rval)), &std::get<2>(rval));
        return rval;
    }
    std::tuple<MpWalker, int8_t, const nlib_byte_t*, uint32_t> GetExt() const NLIB_NOEXCEPT {
        std::tuple<MpWalker, int8_t, const nlib_byte_t*, uint32_t> rval;
        std::get<0>(rval) = GetExt(&std::get<1>(rval),
                                   reinterpret_cast<const void**>(&std::get<2>(rval)),
                                   &std::get<3>(rval));
        return rval;
    }
#endif
    std::pair<MpWalker, uint32_t> GetMapCount() const NLIB_NOEXCEPT {
        std::pair<MpWalker, uint32_t> rval;
        rval.first = GetMapCount(&rval.second);
        return rval;
    }
    std::pair<MpWalker, uint32_t> GetArrayCount() const NLIB_NOEXCEPT {
        std::pair<MpWalker, uint32_t> rval;
        rval.first = GetArrayCount(&rval.second);
        return rval;
    }
    std::pair<MpWalker, bool> GetBoolean() const NLIB_NOEXCEPT {
        std::pair<MpWalker, bool> rval;
        rval.first = GetBoolean(&rval.second);
        return rval;
    }
    std::pair<MpWalker, int32_t> GetInt32() const NLIB_NOEXCEPT {
        std::pair<MpWalker, int32_t> rval;
        rval.first = GetInt(&rval.second);
        return rval;
    }
    std::pair<MpWalker, int64_t> GetInt64() const NLIB_NOEXCEPT {
        std::pair<MpWalker, int64_t> rval;
        rval.first = GetInt(&rval.second);
        return rval;
    }
    std::pair<MpWalker, uint32_t> GetUint32() const NLIB_NOEXCEPT {
        std::pair<MpWalker, uint32_t> rval;
        rval.first = GetUint(&rval.second);
        return rval;
    }
    std::pair<MpWalker, uint64_t> GetUint64() const NLIB_NOEXCEPT {
        std::pair<MpWalker, uint64_t> rval;
        rval.first = GetUint(&rval.second);
        return rval;
    }
    std::pair<MpWalker, float> GetFloat() const NLIB_NOEXCEPT {
        std::pair<MpWalker, float> rval;
        rval.first = GetFloat(&rval.second);
        return rval;
    }
    std::pair<MpWalker, double> GetDouble() const NLIB_NOEXCEPT {
        std::pair<MpWalker, double> rval;
        rval.first = GetDouble(&rval.second);
        return rval;
    }
    std::pair<MpWalker, nlib_time> GetTimestamp() const NLIB_NOEXCEPT {
        std::pair<MpWalker, nlib_time> rval;
        rval.first = GetTimestamp(&rval.second);
        return rval;
    }
    NLIB_SAFE_BOOL(MpWalker, IsOk())

 private:
    bool IsOk() const NLIB_NOEXCEPT { return cur_ != nullptr; }

    const uint8_t* cur_;
    size_t size_;
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
