﻿
#pragma once
#ifndef INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_
#define INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_

#include "lz4frame.h"
#include "nn/nlib/Config.h"
#include "nn/nlib/OutputStream.h"
#include "nn/nlib/OutputConverterStream.h"

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace oss {

class Lz4OutputTransform;
class Lz4OutputStreamSettings {
 public:
    Lz4OutputStreamSettings() NLIB_NOEXCEPT {
        nlib_memset(&pref_, 0, sizeof(pref_));
        pref_.autoFlush           = 1;
        pref_.frameInfo.blockMode = LZ4F_blockLinked;
        pref_.compressionLevel    = 9;  // hc
    }
    void SetContentSize(uint64_t size) NLIB_NOEXCEPT { pref_.frameInfo.contentSize = size; }
    uint64_t GetContentSize() const NLIB_NOEXCEPT { return pref_.frameInfo.contentSize; }
    void SetCompressionLevel(int level) NLIB_NOEXCEPT {
        if (level < 0)
            level = 0;
        else if (level > 16)
            level = 16;
        pref_.compressionLevel = level;
    }
    int GetCompressionLevel() NLIB_NOEXCEPT { return pref_.compressionLevel; }
    void SetMaxBlockSize(LZ4F_blockSizeID_t max_block_size) NLIB_NOEXCEPT {
        pref_.frameInfo.blockSizeID = max_block_size;
    }
    LZ4F_blockSizeID_t GetMaxBlockSize() const NLIB_NOEXCEPT { return pref_.frameInfo.blockSizeID; }
    void SetBlockIndependence(bool independence) NLIB_NOEXCEPT {
        pref_.frameInfo.blockMode = independence ? LZ4F_blockIndependent : LZ4F_blockLinked;
    }
    bool GetBlockIndependence() const NLIB_NOEXCEPT {
        return (pref_.frameInfo.blockMode != LZ4F_blockLinked);
    }

 private:
    LZ4F_preferences_t pref_;
    friend class Lz4OutputTransform;
};

class NLIB_VIS_PUBLIC Lz4OutputStream NLIB_FINAL : public OutputConverterStream {
 public:
    NLIB_CEXPR Lz4OutputStream() NLIB_NOEXCEPT : prv_(nullptr) {}
    virtual ~Lz4OutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        Lz4OutputStreamSettings settings;
        return this->Init(settings);
    }
    errno_t Init(const Lz4OutputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    struct Lz4OutputStreamPrivate;
    Lz4OutputStreamPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(Lz4OutputStream);
};

}  // namespace oss
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_
