﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMD_SIMDVECTOR4_H_
#define INCLUDE_NN_NLIB_SIMD_SIMDVECTOR4_H_

#include "nn/nlib/simd/SimdFloat.h"

NLIB_NAMESPACE_BEGIN
namespace simd {

class NLIB_VIS_HIDDEN Vector4 {
 public:
    template<typename MyVector4>
    static SimdVector __vectorcall LoadFloat4(const MyVector4* p) NLIB_NOEXCEPT;
    template<typename MyVector4>
    static void __vectorcall StoreFloat4(MyVector4* p, SimdVectorArg vec) NLIB_NOEXCEPT;
    static bool __vectorcall CmpEq(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpEq(vec1, vec2));
    }
    static bool __vectorcall CmpLt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpLt(vec1, vec2));
    }
    static bool __vectorcall CmpLe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpLe(vec1, vec2));
    }
    static bool __vectorcall CmpGt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpGt(vec1, vec2));
    }
    static bool __vectorcall CmpGe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpGe(vec1, vec2));
    }
    static bool __vectorcall CmpNe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpNe(vec1, vec2));
    }
    static bool __vectorcall CmpNearEq(SimdVectorArg vec1, SimdVectorArg vec2,
                                       SimdVectorArg eps) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::CmpNearEq(vec1, vec2, eps));
    }
    static bool __vectorcall IsNaN(SimdVectorArg vec) NLIB_NOEXCEPT {
        return !F128::IsAllMaskFalse(F128::IsNaN(vec));
    }
    static bool __vectorcall IsInfinite(SimdVectorArg vec) NLIB_NOEXCEPT {
        return !F128::IsAllMaskFalse(F128::IsInfinite(vec));
    }
    static bool __vectorcall InBound(SimdVectorArg vec, SimdVectorArg bounds) NLIB_NOEXCEPT {
        return F128::IsAllMaskTrue(F128::InBound(vec, bounds));
    }

    static f128 __vectorcall Dot(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    template<bool SetLane0, bool SetLane1, bool SetLane2, bool SetLane3>
    static f128 __vectorcall DotEx(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static f128 __vectorcall Dot2(SimdVectorArg a, SimdVectorArg b0,
                                  SimdVectorArg b1) NLIB_NOEXCEPT;
    static f128 __vectorcall Dot3(SimdVectorArg a, SimdVectorArg b0, SimdVectorArg b1,
                                  SimdVectorArg b2) NLIB_NOEXCEPT;
    static f128 __vectorcall Dot4(SimdVectorArg a, SimdVectorArg b0, SimdVectorArg b1,
                                  SimdVectorArg b2, SimdVectorArg b3) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Normalize(SimdVectorArg vec) NLIB_NOEXCEPT;
    static SimdVector __vectorcall NormalizeEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall LengthSq(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall Length(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall LengthEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpLength(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpLengthEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall GetAngle(SimdVectorArg vec1_normalized,
                                      SimdVectorArg vec2_normalized) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Reflect(SimdVectorArg vec, SimdVectorArg normal) NLIB_NOEXCEPT;
    // static SimdVector __vectorcall Transform(SimdMatrixArg m, SimdVectorArg vec) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Transform(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT;

    // TransformSequence(....)
    // Refract, Ortho

 private:
    Vector4();  // forbidden
};

#ifndef NLIB_DOXYGEN

#define NLIB_M(tp) inline tp __vectorcall
#define NLIB_MH(tp) inline tp __vectorcall

template<typename MyVector4>
// MyVector is a structure like 'struct MyVector4 { float x, y, z, w; };'
// nlib_ns::simd::Float4, DirectX::XMFLOAT3, DirectX::XMFLOAT3A for example
NLIB_MH(SimdVector) Vector4::LoadFloat4(const MyVector4* p) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(sizeof(p->x) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->y) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->z) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->w) == 4);
    return F128::LoadA4(reinterpret_cast<const float*>(&p->x));
}

template<typename MyVector4>
// MyVector is a structure like 'struct MyVector4 { float x, y, z, w; };'
// nlib_ns::simd::Float4, DirectX::XMFLOAT3, DirectX::XMFLOAT3A for example
NLIB_MH(void) Vector4::StoreFloat4(MyVector4* p, SimdVectorArg vec) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(sizeof(p->x) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->y) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->z) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->w) == 4);
    F128::StoreA4(reinterpret_cast<float*>(&p->x), vec);
}

// r = { dot, dot, dot, dot }
NLIB_M(f128) Vector4::Dot(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = ret.vec.v[1] = ret.vec.v[2] = ret.vec.v[3] =
        vec1.vec.v[0] * vec2.vec.v[0] + vec1.vec.v[1] * vec2.vec.v[1] +
        vec1.vec.v[2] * vec2.vec.v[2] + vec1.vec.v[3] * vec2.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_dp_ps(vec1, vec2, 0xFF);
#elif defined(NLIB_NEON)
#ifdef __aarch64__
    float32x4_t tmp = vmulq_f32(vec1, vec2);
    tmp             = vpaddq_f32(tmp, tmp);
    tmp             = vpaddq_f32(tmp, tmp);
    return tmp;
#else
    float32x4_t tmp = vmulq_f32(vec1, vec2);
    float32x2_t v1  = vget_low_f32(tmp);
    float32x2_t v2  = vget_high_f32(tmp);
    v1              = vpadd_f32(v1, v1);
    v2              = vpadd_f32(v2, v2);
    v1              = vadd_f32(v1, v2);
    return vcombine_f32(v1, v1);
#endif
#elif defined(CAFE)
    f128 tmp  = F128::Mult(vec1, vec2);
    f32x2 val = __PS_ADD(tmp.vec.ps[0], tmp.vec.ps[1]);
    val       = __PS_SUM0(val, val, val);
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(val[0]);
    return ret;
#endif
}

template<bool SetLane0, bool SetLane1, bool SetLane2, bool SetLane3>
// r[i] = SetLane[i] ? dot : 0.f
NLIB_MH(f128) Vector4::DotEx(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if !defined(NLIB_F128_SIMD_NOUSE) && defined(NLIB_SSE41)
    return _mm_dp_ps(
        vec1, vec2,
        (0xF0 | (SetLane0 ? 1 : 0) | (SetLane1 ? 2 : 0) | (SetLane2 ? 4 : 0) | (SetLane3 ? 8 : 0)));
#else
    return F128::Splat<SetLane0, SetLane1, SetLane2, SetLane3>(F128::SetZero(), Dot(vec1, vec2));
#endif
}

// r = { dot(a, b0), dot(a, b1), 0.f, 0.f }
NLIB_M(f128) Vector4::Dot2(SimdVectorArg a, SimdVectorArg b0, SimdVectorArg b1) NLIB_NOEXCEPT {
#if !defined(NLIB_F128_SIMD_NOUSE) && defined(NLIB_NEON)
#ifdef __aarch64__
    float32x4_t zero = vdupq_n_f32(0);
    float32x4_t tmp0 = vmulq_f32(a, b0);
    float32x4_t tmp1 = vmulq_f32(a, b1);
    tmp0             = vpaddq_f32(tmp0, tmp1);
    tmp0             = vpaddq_f32(tmp0, zero);
    return tmp0;
#else
    float32x4x2_t trn  = vtrnq_f32(b0, b1);
    float32x2_t lo     = vget_low_f32(a);
    float32x2_t hi     = vget_high_f32(a);
    float32x4_t xxzz   = vcombine_f32(vdup_lane_f32(lo, 0), vdup_lane_f32(hi, 0));
    float32x4_t yyww   = vcombine_f32(vdup_lane_f32(lo, 1), vdup_lane_f32(hi, 1));
    float32x4_t tmp    = vmulq_f32(trn.val[0], xxzz);
    tmp                = vmlaq_f32(tmp, yyww, trn.val[1]);
    float32x2_t result = vadd_f32(vget_low_f32(tmp), vget_high_f32(tmp));
    return vcombine_f32(result, vdup_n_f32(0.f));
#endif
#else
    f128 t0 = Vector4::DotEx<true, false, false, false>(a, b0);
    f128 t1 = Vector4::DotEx<false, true, false, false>(a, b1);
    return F128::Or(t0, t1);
#endif
}

// r = { dot(a, b0), dot(a, b1), dot(a, b2), 0.f }
NLIB_M(f128)
Vector4::Dot3(SimdVectorArg a, SimdVectorArg b0, SimdVectorArg b1, SimdVectorArg b2) NLIB_NOEXCEPT {
#if !defined(NLIB_F128_SIMD_NOUSE) && defined(NLIB_NEON)
#ifdef __aarch64__
    float32x4_t zero = vdupq_n_f32(0);
    float32x4_t tmp0 = vmulq_f32(a, b0);
    float32x4_t tmp1 = vmulq_f32(a, b1);
    float32x4_t tmp2 = vmulq_f32(a, b2);
    tmp0             = vpaddq_f32(tmp0, tmp1);
    tmp2             = vpaddq_f32(tmp2, zero);
    tmp0             = vpaddq_f32(tmp0, tmp2);
    return tmp0;
#else
    f128 row0, row1, row2, row3;
    float32x4x2_t trn_f0_ = vtrnq_f32(b0, b1);
    float32x4x2_t trn_f1_ = vtrnq_f32(b2, vdupq_n_f32(0.f));
    row0     = vcombine_f32(vget_low_f32(trn_f0_.val[0]), vget_low_f32(trn_f1_.val[0]));
    row1     = vcombine_f32(vget_low_f32(trn_f0_.val[1]), vget_low_f32(trn_f1_.val[1]));
    row2     = vcombine_f32(vget_high_f32(trn_f0_.val[0]), vget_high_f32(trn_f1_.val[0]));
    row3     = vcombine_f32(vget_high_f32(trn_f0_.val[1]), vget_high_f32(trn_f1_.val[1]));
    f128 ret = F128::Mult<0>(a, row0, each_select32);
    ret      = F128::MultAdd<1>(a, row1, ret, each_select32);
    ret      = F128::MultAdd<2>(a, row2, ret, each_select32);
    return F128::MultAdd<3>(a, row3, ret, each_select32);
#endif
#else
    f128 t0 = Vector4::DotEx<true, false, false, false>(a, b0);
    f128 t1 = Vector4::DotEx<false, true, false, false>(a, b1);
    f128 t2 = Vector4::DotEx<false, false, true, false>(a, b2);
    return F128::Or(F128::Or(t0, t1), t2);
#endif
}

// r = { dot(a, b0), dot(a, b1), dot(a, b2), dot(a, b3) }
NLIB_M(f128)
Vector4::Dot4(SimdVectorArg a, SimdVectorArg b0, SimdVectorArg b1, SimdVectorArg b2,
              SimdVectorArg b3) NLIB_NOEXCEPT {
#if !defined(NLIB_F128_SIMD_NOUSE) && defined(NLIB_NEON)
#ifdef __aarch64__
    float32x4_t tmp0 = vmulq_f32(a, b0);
    float32x4_t tmp1 = vmulq_f32(a, b1);
    float32x4_t tmp2 = vmulq_f32(a, b2);
    float32x4_t tmp3 = vmulq_f32(a, b3);
    tmp0             = vpaddq_f32(tmp0, tmp1);
    tmp2             = vpaddq_f32(tmp2, tmp3);
    tmp0             = vpaddq_f32(tmp0, tmp2);
    return tmp0;
#else
    f128 row0, row1, row2, row3;
    float32x4x2_t trn_f0_ = vtrnq_f32(b0, b1);
    float32x4x2_t trn_f1_ = vtrnq_f32(b2, b3);
    row0     = vcombine_f32(vget_low_f32(trn_f0_.val[0]), vget_low_f32(trn_f1_.val[0]));
    row1     = vcombine_f32(vget_low_f32(trn_f0_.val[1]), vget_low_f32(trn_f1_.val[1]));
    row2     = vcombine_f32(vget_high_f32(trn_f0_.val[0]), vget_high_f32(trn_f1_.val[0]));
    row3     = vcombine_f32(vget_high_f32(trn_f0_.val[1]), vget_high_f32(trn_f1_.val[1]));
    f128 ret = F128::Mult<0>(a, row0, each_select32);
    ret      = F128::MultAdd<1>(a, row1, ret, each_select32);
    ret      = F128::MultAdd<2>(a, row2, ret, each_select32);
    return F128::MultAdd<3>(a, row3, ret, each_select32);
#endif
#else
    f128 t0 = Vector4::DotEx<true, false, false, false>(a, b0);
    f128 t1 = Vector4::DotEx<false, true, false, false>(a, b1);
    f128 t2 = Vector4::DotEx<false, false, true, false>(a, b2);
    f128 t3 = Vector4::DotEx<false, false, false, true>(a, b3);
    return F128::Or(F128::Or(t0, t1), F128::Or(t2, t3));
#endif
}

// nan if |vec| is too big, 0 if |vec| = 0
NLIB_M(SimdVector) Vector4::Normalize(SimdVectorArg vec) NLIB_NOEXCEPT {
    f128 dot = Vector4::Dot(vec, vec);
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    float32x4_t x;
    x          = vrsqrteq_f32(dot);
    x          = vmulq_f32(x, vrsqrtsq_f32(dot, vmulq_f32(x, x)));
    f128 rsqrt = vmulq_f32(x, vrsqrtsq_f32(dot, vmulq_f32(x, x)));
#else
    f128 rsqrt = F128::RecpSqrt(dot);
#endif
    f128 inf       = F128::SetInfinity();
    f128 eqzero    = F128::CmpEqZero(dot);
    f128 eqinf     = F128::CmpEq(dot, inf);
    SimdVector ret = F128::Mult(vec, rsqrt);
    f128 nan       = F128::SetNaN();
    ret            = F128::AndNot(eqzero, ret);
    ret            = F128::Select(eqinf, nan, ret);
    return ret;
}

// r = { lensq, lensq, lensq, lensq }
NLIB_M(f128) Vector4::LengthSq(SimdVectorArg vec) NLIB_NOEXCEPT {
    return Dot(vec, vec);
}

// r = { len, len, len, len }
NLIB_M(f128) Vector4::Length(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::Sqrt(Dot(vec, vec));
}

// r = { len, len, len, len }
NLIB_M(f128) Vector4::LengthEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::SqrtEst(Dot(vec, vec));
}

// r = { recpLen, recpLen, recpLen, recpLen }
NLIB_M(f128) Vector4::RecpLength(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::RecpSqrt(Dot(vec, vec));
}

// r = { recpLen, recpLen, recpLen, recpLen }
NLIB_M(f128) Vector4::RecpLengthEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::RecpSqrtEst(Dot(vec, vec));
}

NLIB_M(SimdVector) Vector4::NormalizeEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::Mult(vec, RecpLengthEst(vec));
}

// { radian, radian, radian, radian }
NLIB_M(f128)
Vector4::GetAngle(SimdVectorArg vec1_normalized, SimdVectorArg vec2_normalized) NLIB_NOEXCEPT {
    f128 ret = Dot(vec1_normalized, vec2_normalized);
    ret      = F128::Clamp(ret, F128::SetNegativeOne(), F128::SetOne());
    return F128::ArcCos(ret);
}

NLIB_M(SimdVector) Vector4::Reflect(SimdVectorArg vec, SimdVectorArg normal) NLIB_NOEXCEPT {
    f128 s = Dot(vec, normal);
    s      = F128::Add(s, s);
    return F128::MultSub(s, normal, vec);
}

/*
// r = m * vec
NLIB_M(SimdVector) Vector4::Transform(SimdMatrixArg m, SimdVectorArg vec) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = m.r[0].vec.v[0] * vec.vec.v[0] + m.r[0].vec.v[1] * vec.vec.v[1] +
                   m.r[0].vec.v[2] * vec.vec.v[2] + m.r[0].vec.v[3] * vec.vec.v[3];
    ret.vec.v[1] = m.r[1].vec.v[0] * vec.vec.v[0] + m.r[1].vec.v[1] * vec.vec.v[1] +
                   m.r[1].vec.v[2] * vec.vec.v[2] + m.r[1].vec.v[3] * vec.vec.v[3];
    ret.vec.v[2] = m.r[2].vec.v[0] * vec.vec.v[0] + m.r[2].vec.v[1] * vec.vec.v[1] +
                   m.r[2].vec.v[2] * vec.vec.v[2] + m.r[2].vec.v[3] * vec.vec.v[3];
    ret.vec.v[3] = m.r[3].vec.v[0] * vec.vec.v[0] + m.r[3].vec.v[1] * vec.vec.v[1] +
                   m.r[3].vec.v[2] * vec.vec.v[2] + m.r[3].vec.v[3] * vec.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    f128 tmp, ret;
    ret = _mm_dp_ps(m.r[0], vec, 0xF1);
    tmp = _mm_dp_ps(m.r[1], vec, 0xF2);
    ret = _mm_or_ps(ret, tmp);
    tmp = _mm_dp_ps(m.r[2], vec, 0xF4);
    ret = _mm_or_ps(ret, tmp);
    tmp = _mm_dp_ps(m.r[3], vec, 0xF8);
    ret = _mm_or_ps(ret, tmp);
    return ret;
#elif defined(NLIB_NEON)
    float32x4_t r0, r1;
    float32x2_t lo, hi, tmp;

    r0 = vmulq_f32(m.r[0], vec);
    r1 = vmulq_f32(m.r[1], vec);
    lo = vpadd_f32(vget_low_f32(r0), vget_high_f32(r0));
    tmp = vpadd_f32(vget_low_f32(r1), vget_high_f32(r1));
    lo = vpadd_f32(lo, tmp);

    r0 = vmulq_f32(m.r[2], vec);
    r1 = vmulq_f32(m.r[3], vec);
    hi = vpadd_f32(vget_low_f32(r0), vget_high_f32(r0));
    tmp = vpadd_f32(vget_low_f32(r1), vget_high_f32(r1));
    hi = vpadd_f32(hi, tmp);

    return vcombine_f32(lo, hi);
#endif
}
*/

// r = vec * m
NLIB_M(SimdVector) Vector4::Transform(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT {
    f128 ret = F128::Mult<0>(vec, m.r[0], each_select32);
    ret      = F128::MultAdd<1>(vec, m.r[1], ret, each_select32);
    ret      = F128::MultAdd<2>(vec, m.r[2], ret, each_select32);
    return F128::MultAdd<3>(vec, m.r[3], ret, each_select32);
}

#undef NLIB_M
#undef NLIB_MH

#endif  // NLIB_DOXYGEN

}  // namespace simd
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SIMD_SIMDVECTOR4_H_
