﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_
#define INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_

#include "nn/nlib/Swap.h"
#include "nn/nlib/succinct/Bp.h"
#include "nn/nlib/succinct/Sbv.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {

class NLIB_VIS_PUBLIC AhoCorasick NLIB_FINAL {
 public:
    static const size_t LABEL_WIDTH    = 8;  // 1,2,4,8
    static const int LABEL_MASK        = (1 << LABEL_WIDTH) - 1;
    static const int NUMLABEL_PER_BYTE = 8 / LABEL_WIDTH;

 public:
    NLIB_CEXPR AhoCorasick() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~AhoCorasick() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(AhoCorasick);
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    void Match(const char* doc, MatchCallback callback, void* user_obj) NLIB_NOEXCEPT {
        this->Match(doc, nlib_strlen(doc), callback, user_obj);
    }
    void Match(const char* doc, MatchCallback callback) NLIB_NOEXCEPT {
        this->Match(doc, nlib_strlen(doc), callback, nullptr);
    }
    void Match(const void* data, size_t n, MatchCallback callback, void* user_obj) NLIB_NOEXCEPT;
    void Match(const void* data, size_t n, MatchCallback callback) NLIB_NOEXCEPT {
        Match(data, n, callback, nullptr);
    }
    size_t MemSize() const NLIB_NOEXCEPT;

    void MemSize(size_t* len_store, size_t* goto_arc, size_t* report_arc,
                 size_t* failure_arc) const NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT;
    bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    bool Import(BinaryReader* r) NLIB_NOEXCEPT;

 private:
    struct AhoCorasickPrivate;
    AhoCorasickPrivate* prv_;
    friend class AhoCorasickBuilder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AhoCorasick);
};

}  // namespace succinct
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::succinct::AhoCorasick)

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_
