﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TESTING_PARAMVALUE_H_
#define INCLUDE_NN_NLIB_TESTING_PARAMVALUE_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/testing/NTest.h"
#include "nn/nlib/testing/ParamValueTypes.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/Nlist.h"

#if defined(_MSC_VER) && defined(nx_testing_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace testing {

class NLIB_VIS_PUBLIC TestWithParamBase : public Test {
 public:
    virtual void TestBody() NLIB_OVERRIDE;

 private:
    virtual void TestBodySub()      = 0;
    virtual bool SetParam(size_t i) = 0;
};

template<class T>
class TestWithParam : public TestWithParamBase {
 public:
    typedef T ParamType;
    TestWithParam() : param_(nullptr) {}
    const ParamType& GetParam() const { return *param_; }
    static Nlist<ParamType>* GetParams() { return &param_vec_; }

 private:
    virtual bool SetParam(size_t i) NLIB_OVERRIDE;

 private:
    const ParamType* param_;
    static Nlist<ParamType> param_vec_;

    NLIB_DISALLOW_COPY_AND_ASSIGN(TestWithParam);
};

template<class T>
bool TestWithParam<T>::SetParam(size_t i) {
    if (i < param_vec_.size()) {
        param_ = &param_vec_[i];
        return true;
    }
    return false;
}

template<>
NLIB_VIS_PUBLIC bool TestWithParam<bool>::SetParam(size_t i);

template<class T>
Nlist<T> TestWithParam<T>::param_vec_;

class NLIB_VIS_PUBLIC TestParamAdderBase {
 public:
    virtual ~TestParamAdderBase() {}
    virtual void AddParams() = 0;

 protected:
    TestParamAdderBase() {}

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(TestParamAdderBase);
};

class NLIB_VIS_PUBLIC TestParamAdderList {
 public:
    static void AddParams();
    static void AddTestParamAdder(TestParamAdderBase* v);

 public:
    TestParamAdderList() {}
    ~TestParamAdderList() {}

 private:
    static TestParamAdderBase* adder_list_[1024];
    static int adder_list_idx_;

    NLIB_DISALLOW_COPY_AND_ASSIGN(TestParamAdderList);
};

inline detail::ValuesBool Bool() {
    return detail::ValuesBool();
}

template<class C, size_t N>
detail::ValuesInObjArray<C, N> ValuesIn(const C (&a)[N]) {
    return detail::ValuesInObjArray<C, N>(a);
}

template<class Container>
detail::ValuesInObjContainer<Container> ValuesIn(const Container& c) {
    return detail::ValuesInObjContainer<Container>(c);
}

template<class Iterator>
detail::ValuesInObjIterator<Iterator> ValuesIn(Iterator first, Iterator last) {
    return detail::ValuesInObjIterator<Iterator>(first, last);
}

template<class T>
detail::ValuesRange<T, T> Range(T first, T last) {
    return detail::ValuesRange<T, T>(first, last, T(1));
}

template<class T, class S>
detail::ValuesRange<T, S> Range(T first, T last, S step) {
    return detail::ValuesRange<T, S>(first, last, step);
}

template<class T1>
detail::Values1<T1> Values(T1 v1) {
    return detail::Values1<T1>(v1);
}

template<class T1, class T2>
detail::Values2<T1, T2> Values(T1 v1, T2 v2) {
    return detail::Values2<T1, T2>(v1, v2);
}

template<class T1, class T2, class T3>
detail::Values3<T1, T2, T3> Values(T1 v1, T2 v2, T3 v3) {
    return detail::Values3<T1, T2, T3>(v1, v2, v3);
}

template<class T1, class T2, class T3, class T4>
detail::Values4<T1, T2, T3, T4> Values(T1 v1, T2 v2, T3 v3, T4 v4) {
    return detail::Values4<T1, T2, T3, T4>(v1, v2, v3, v4);
}

template<class T1, class T2, class T3, class T4, class T5>
detail::Values5<T1, T2, T3, T4, T5> Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5) {
    return detail::Values5<T1, T2, T3, T4, T5>(v1, v2, v3, v4, v5);
}

template<class T1, class T2, class T3, class T4, class T5, class T6>
detail::Values6<T1, T2, T3, T4, T5, T6> Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6) {
    return detail::Values6<T1, T2, T3, T4, T5, T6>(v1, v2, v3, v4, v5, v6);
}

template<class T1, class T2, class T3, class T4, class T5, class T6, class T7>
detail::Values7<T1, T2, T3, T4, T5, T6, T7>
Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7) {
    return detail::Values7<T1, T2, T3, T4, T5, T6, T7>(v1, v2, v3, v4, v5, v6, v7);
}

template<class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
detail::Values8<T1, T2, T3, T4, T5, T6, T7, T8>
Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7, T8 v8) {
    return detail::Values8<T1, T2, T3, T4, T5, T6, T7, T8>(v1, v2, v3, v4, v5, v6, v7, v8);
}

template<class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
detail::Values9<T1, T2, T3, T4, T5, T6, T7, T8, T9>
Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7, T8 v8, T9 v9) {
    return detail::Values9<T1, T2, T3, T4, T5, T6, T7, T8, T9>(v1, v2, v3, v4, v5, v6, v7, v8, v9);
}

template<class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9,
         class T10>
detail::Values10<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>
Values(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5, T6 v6, T7 v7, T8 v8, T9 v9, T10 v10) {
    return detail::Values10<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(v1, v2, v3, v4, v5, v6, v7, v8,
                                                                     v9, v10);
}

}  // namespace testing
NLIB_NAMESPACE_END

#define TEST_P(test_case, test_name)                                                               \
    class NLIB_TESTING_CLASS_NAME_(test_case, test_name) : public test_case {                      \
     public:                                                                                       \
        NLIB_TESTING_CLASS_NAME_(test_case, test_name)() {}                                        \
                                                                                                   \
     private:                                                                                      \
        virtual void TestBodySub();                                                                \
        static ::nlib_ns::testing::TestInfo* info_;                                                \
        NLIB_DISALLOW_COPY_AND_ASSIGN(NLIB_TESTING_CLASS_NAME_(test_case, test_name));             \
    };                                                                                             \
    ::nlib_ns::testing::TestInfo* NLIB_TESTING_CLASS_NAME_(test_case, test_name)::info_ =          \
        ::nlib_ns::testing::TestInfo::AddTestInfo<NLIB_TESTING_CLASS_NAME_(test_case, test_name)>( \
            #test_case, #test_name, -1, test_case::SetUpTestCase, test_case::TearDownTestCase);    \
    void NLIB_TESTING_CLASS_NAME_(test_case, test_name)::TestBodySub()

#define INSTANTIATE_TEST_CASE_P(prefix, test_case, generator)                                  \
    class TestParamAdder_##test_case##prefix : public ::nlib_ns::testing::TestParamAdderBase { \
     public:                                                                                   \
        TestParamAdder_##test_case##prefix() {                                                 \
            ::nlib_ns::testing::TestParamAdderList::AddTestParamAdder(this);                   \
        }                                                                                      \
        virtual void AddParams() { (generator).Gen(test_case::GetParams()); }                  \
    } ntest_##test_case##prefix##_glbval_

#if defined(_MSC_VER) && defined(nx_testing_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_TESTING_PARAMVALUE_H_
