﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TESTING_PRED_H_
#define INCLUDE_NN_NLIB_TESTING_PRED_H_

#include "nn/nlib/testing/NTest.h"

NLIB_NAMESPACE_BEGIN
namespace testing {

template<class Pred, class T1>
AssertionResult EvalPred1(const char* pred_text, const char* e1, Pred pred, const T1& v1) {
    return pred(v1) ? AssertionResult(true)
                    : AssertionResult(false)
                          << pred_text << "(" << e1 << ") evaluates to false, where"
                          << "\n"
                          << e1 << " evaluates to " << v1;
}

template<class Pred, class T1, class T2>
AssertionResult EvalPred2(const char* pred_text, const char* e1, const char* e2, Pred pred,
                          const T1& v1, const T2& v2) {
    return pred(v1, v2) ? AssertionResult(true)
                        : AssertionResult(false) << pred_text << "(" << e1 << ", " << e2
                                                 << ") evaluates to false, where"
                                                 << "\n"
                                                 << e1 << " evaluates to " << v1 << "\n"
                                                 << e2 << " evaluates to " << v2;
}

template<class Pred, class T1, class T2, class T3>
AssertionResult EvalPred3(const char* pred_text, const char* e1, const char* e2, const char* e3,
                          Pred pred, const T1& v1, const T2& v2, const T3& v3) {
    return pred(v1, v2, v3) ? AssertionResult(true)
                            : AssertionResult(false) << pred_text << "(" << e1 << ", " << e2 << ", "
                                                     << e3 << ") evaluates to false, where"
                                                     << "\n"
                                                     << e1 << " evaluates to " << v1 << "\n"
                                                     << e2 << " evaluates to " << v2 << "\n"
                                                     << e3 << " evaluates to " << v3;
}

template<class Pred, class T1, class T2, class T3, class T4>
AssertionResult
EvalPred4(const char* pred_text, const char* e1, const char* e2, const char* e3, const char* e4,
          Pred pred, const T1& v1, const T2& v2, const T3& v3, const T4& v4) {
    return pred(v1, v2, v3, v4) ? AssertionResult(true)
                                : AssertionResult(false)
                                      << pred_text << "(" << e1 << ", " << e2 << ", " << e3 << ", "
                                      << e4 << ") evaluates to false, where"
                                      << "\n"
                                      << e1 << " evaluates to " << v1 << "\n"
                                      << e2 << " evaluates to " << v2 << "\n"
                                      << e3 << " evaluates to " << v3 << "\n"
                                      << e4 << " evaluates to " << v4;
}

template<class Pred, class T1, class T2, class T3, class T4, class T5>
AssertionResult EvalPred5(const char* pred_text, const char* e1, const char* e2, const char* e3,
                          const char* e4, const char* e5, Pred pred, const T1& v1, const T2& v2,
                          const T3& v3, const T4& v4, const T5& v5) {
    return pred(v1, v2, v3, v4, v5) ? AssertionResult(true)
                                    : AssertionResult(false) << pred_text << "(" << e1 << ", " << e2
                                                             << ", " << e3 << ", " << e4 << ", "
                                                             << e5 << ") evaluates to false, where"
                                                             << "\n"
                                                             << e1 << " evaluates to " << v1 << "\n"
                                                             << e2 << " evaluates to " << v2 << "\n"
                                                             << e3 << " evaluates to " << v3 << "\n"
                                                             << e4 << " evaluates to " << v4 << "\n"
                                                             << e5 << " evaluates to " << v5;
}

}  // namespace testing
NLIB_NAMESPACE_END

#define NLIB_TESTING_PRED1_(pred, v1, iffail)                        \
    switch (0)                                                       \
    case 0:                                                          \
    default:                                                         \
        if (::nlib_ns::testing::AssertionResult ar =                 \
                ::nlib_ns::testing::EvalPred1(#pred, #v1, pred, v1)) \
            ;                                                        \
        else                                                         \
            iffail = ar

#define NLIB_TESTING_PRED2_(pred, v1, v2, iffail)                             \
    switch (0)                                                                \
    case 0:                                                                   \
    default:                                                                  \
        if (::nlib_ns::testing::AssertionResult ar =                          \
                ::nlib_ns::testing::EvalPred2(#pred, #v1, #v2, pred, v1, v2)) \
            ;                                                                 \
        else                                                                  \
            iffail = ar

#define NLIB_TESTING_PRED3_(pred, v1, v2, v3, iffail)                                  \
    switch (0)                                                                         \
    case 0:                                                                            \
    default:                                                                           \
        if (::nlib_ns::testing::AssertionResult ar =                                   \
                ::nlib_ns::testing::EvalPred3(#pred, #v1, #v2, #v3, pred, v1, v2, v3)) \
            ;                                                                          \
        else                                                                           \
            iffail = ar

#define NLIB_TESTING_PRED4_(pred, v1, v2, v3, v4, iffail)                                       \
    switch (0)                                                                                  \
    case 0:                                                                                     \
    default:                                                                                    \
        if (::nlib_ns::testing::AssertionResult ar =                                            \
                ::nlib_ns::testing::EvalPred4(#pred, #v1, #v2, #v3, #v4, pred, v1, v2, v3, v4)) \
            ;                                                                                   \
        else                                                                                    \
            iffail = ar

#define NLIB_TESTING_PRED5_(pred, v1, v2, v3, v4, v5, iffail)                       \
    switch (0)                                                                      \
    case 0:                                                                         \
    default:                                                                        \
        if (::nlib_ns::testing::AssertionResult ar = ::nlib_ns::testing::EvalPred5( \
                #pred, #v1, #v2, #v3, #v4, #v5, pred, v1, v2, v3, v4, v5))          \
            ;                                                                       \
        else                                                                        \
            iffail = ar

#define NLIB_TESTING_PRED_FORMAT1_(pred_format, v1, iffail)                \
    switch (0)                                                             \
    case 0:                                                                \
    default:                                                               \
        if (::nlib_ns::testing::AssertionResult ar = pred_format(#v1, v1)) \
            ;                                                              \
        else                                                               \
            iffail = ar

#define NLIB_TESTING_PRED_FORMAT2_(pred_format, v1, v2, iffail)                     \
    switch (0)                                                                      \
    case 0:                                                                         \
    default:                                                                        \
        if (::nlib_ns::testing::AssertionResult ar = pred_format(#v1, #v2, v1, v2)) \
            ;                                                                       \
        else                                                                        \
            iffail = ar

#define NLIB_TESTING_PRED_FORMAT3_(pred_format, v1, v2, v3, iffail)                          \
    switch (0)                                                                               \
    case 0:                                                                                  \
    default:                                                                                 \
        if (::nlib_ns::testing::AssertionResult ar = pred_format(#v1, #v2, #v3, v1, v2, v3)) \
            ;                                                                                \
        else                                                                                 \
            iffail = ar

#define NLIB_TESTING_PRED_FORMAT4_(pred_format, v1, v2, v3, v4, iffail) \
    switch (0)                                                          \
    case 0:                                                             \
    default:                                                            \
        if (::nlib_ns::testing::AssertionResult ar =                    \
                pred_format(#v1, #v2, #v3, #v4, v1, v2, v3, v4))        \
            ;                                                           \
        else                                                            \
            iffail = ar

#define NLIB_TESTING_PRED_FORMAT5_(pred_format, v1, v2, v3, v4, v5, iffail) \
    switch (0)                                                              \
    case 0:                                                                 \
    default:                                                                \
        if (::nlib_ns::testing::AssertionResult ar =                        \
                pred_format(#v1, #v2, #v3, #v4, #v5, v1, v2, v3, v4, v5))   \
            ;                                                               \
        else                                                                \
            iffail = ar

#define ASSERT_PRED1(pred, v1) NLIB_TESTING_PRED1_(pred, v1, NLIB_TESTING_FATAL_)
#define EXPECT_PRED1(pred, v1) NLIB_TESTING_PRED1_(pred, v1, NLIB_TESTING_NONFATAL_)
#define ASSERT_PRED_FORMAT1(pred_format, v1) \
    NLIB_TESTING_PRED_FORMAT1_(pred_format, v1, NLIB_TESTING_FATAL_)
#define EXPECT_PRED_FORMAT1(pred_format, v1) \
    NLIB_TESTING_PRED_FORMAT1_(pred_format, v1, NLIB_TESTING_NONFATAL_)

#define ASSERT_PRED2(pred, v1, v2) NLIB_TESTING_PRED2_(pred, v1, v2, NLIB_TESTING_FATAL_)
#define EXPECT_PRED2(pred, v1, v2) NLIB_TESTING_PRED2_(pred, v1, v2, NLIB_TESTING_NONFATAL_)
#define ASSERT_PRED_FORMAT2(pred_format, v1, v2) \
    NLIB_TESTING_PRED_FORMAT2_(pred_format, v1, v2, NLIB_TESTING_FATAL_)
#define EXPECT_PRED_FORMAT2(pred_format, v1, v2) \
    NLIB_TESTING_PRED_FORMAT2_(pred_format, v1, v2, NLIB_TESTING_NONFATAL_)

#define ASSERT_PRED3(pred, v1, v2, v3) NLIB_TESTING_PRED3_(pred, v1, v2, v3, NLIB_TESTING_FATAL_)
#define EXPECT_PRED3(pred, v1, v2, v3) NLIB_TESTING_PRED3_(pred, v1, v2, v3, NLIB_TESTING_NONFATAL_)
#define ASSERT_PRED_FORMAT3(pred_format, v1, v2, v3) \
    NLIB_TESTING_PRED_FORMAT3_(pred_format, v1, v2, v3, NLIB_TESTING_FATAL_)
#define EXPECT_PRED_FORMAT3(pred_format, v1, v2, v3) \
    NLIB_TESTING_PRED_FORMAT3_(pred_format, v1, v2, v3, NLIB_TESTING_NONFATAL_)

#define ASSERT_PRED4(pred, v1, v2, v3, v4) \
    NLIB_TESTING_PRED4_(pred, v1, v2, v3, v4, NLIB_TESTING_FATAL_)
#define EXPECT_PRED4(pred, v1, v2, v3, v4) \
    NLIB_TESTING_PRED4_(pred, v1, v2, v3, v4, NLIB_TESTING_NONFATAL_)
#define ASSERT_PRED_FORMAT4(pred_format, v1, v2, v3, v4) \
    NLIB_TESTING_PRED_FORMAT4_(pred_format, v1, v2, v3, v4, NLIB_TESTING_FATAL_)
#define EXPECT_PRED_FORMAT4(pred_format, v1, v2, v3, v4) \
    NLIB_TESTING_PRED_FORMAT4_(pred_format, v1, v2, v3, v4, NLIB_TESTING_NONFATAL_)

#define ASSERT_PRED5(pred, v1, v2, v3, v4, v5) \
    NLIB_TESTING_PRED5_(pred, v1, v2, v3, v4, v5, NLIB_TESTING_FATAL_)
#define EXPECT_PRED5(pred, v1, v2, v3, v4, v5) \
    NLIB_TESTING_PRED5_(pred, v1, v2, v3, v4, v5, NLIB_TESTING_NONFATAL_)
#define ASSERT_PRED_FORMAT5(pred_format, v1, v2, v3, v4, v5) \
    NLIB_TESTING_PRED_FORMAT5_(pred_format, v1, v2, v3, v4, v5, NLIB_TESTING_FATAL_)
#define EXPECT_PRED_FORMAT5(pred_format, v1, v2, v3, v4, v5) \
    NLIB_TESTING_PRED_FORMAT5_(pred_format, v1, v2, v3, v4, v5, NLIB_TESTING_NONFATAL_)

#endif  // INCLUDE_NN_NLIB_TESTING_PRED_H_
