﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TESTING_TESTING_H_
#define INCLUDE_NN_NLIB_TESTING_TESTING_H_

#include "nn/nlib/Config.h"

#ifndef NLIB_USE_GTEST
#include "nn/nlib/testing/NTest.h"
#include "nn/nlib/testing/Pred.h"
#include "nn/nlib/testing/Printer.h"
#include "nn/nlib/testing/ParamValue.h"
#include "nn/nlib/testing/ParamType.h"

namespace testing = ::nlib_ns::testing;
NLIB_NAMESPACE_BEGIN
namespace testing {
inline void InitGoogleTest(int* argc, char** argv) {
    InitNintendoTest(argc, argv);
}
inline void InitGoogleTest(int* argc, wchar_t** argv) {
    InitNintendoTest(argc, argv);
}
#define NLIB_TESTING_OSTREAM ::nlib_ns::testing::detail_func::DummyStream
}  // namespace testing
NLIB_NAMESPACE_END
#else
// See: https://code.google.com/p/googletest/
#include "gtest/gtest.h"

#define NLIB_TESTING_OSTREAM std::ostream
namespace testing {
inline void InitNintendoTest(int* argc, char** argv) {
    InitGoogleTest(argc, argv);
}
inline void InitNintendoTest(int* argc, wchar_t** argv) {
    InitGoogleTest(argc, argv);
}
}  // namespace testing

#endif

#if 1
#define EXPECT_ERR(e1, e2) EXPECT_STREQ(nlib_error_string(e1), nlib_error_string(e2))
#define ASSERT_ERR(e1, e2) ASSERT_STREQ(nlib_error_string(e1), nlib_error_string(e2))
#define EXPECT_NOERR(e) EXPECT_STREQ("OK", nlib_error_string(e))
#define ASSERT_NOERR(e) ASSERT_STREQ("OK", nlib_error_string(e))
#else
NLIB_NAMESPACE_BEGIN
namespace testing {
class NlibErrHelper {
 public:
    NlibErrHelper(errno_t e1, errno_t e2) NLIB_NOEXCEPT : e1_(e1), e2_(e2) {}
    ~NlibErrHelper() NLIB_NOEXCEPT {}
    NLIB_SAFE_BOOL(NlibErrHelper, (e1_ == e2_));
    errno_t GetE1() NLIB_NOEXCEPT { return e1_; }
    errno_t GetE2() NLIB_NOEXCEPT { return e2_; }

 private:
    errno_t e1_;
    errno_t e2_;
};
}  // namespace testing
NLIB_NAMESPACE_END

#define NLIB_CONCAT_(sym, l) sym##l
#define NLIB_CONCAT(sym, l) NLIB_CONCAT_(sym, l)

#define EXPECT_ERR(e1, e2)                                                 \
    switch (0)                                                             \
    case 0:                                                                \
    default:                                                               \
        if (::nlib_ns::testing::NlibErrHelper NLIB_CONCAT(noerr, l) =      \
                ::nlib_ns::testing::NlibErrHelper(e1, e2))                 \
            ;                                                              \
        else                                                               \
            EXPECT_STREQ(nlib_error_string(NLIB_CONCAT(noerr, l).GetE1()), \
                         nlib_error_string(NLIB_CONCAT(noerr, l).GetE2()))

#define ASSERT_ERR(e1, e2)                                                 \
    switch (0)                                                             \
    case 0:                                                                \
    default:                                                               \
        if (::nlib_ns::testing::NlibErrHelper NLIB_CONCAT(noerr, l) =      \
                ::nlib_ns::testing::NlibErrHelper(e1, e2))                 \
            ;                                                              \
        else                                                               \
            ASSERT_STREQ(nlib_error_string(NLIB_CONCAT(noerr, l).GetE1()), \
                         nlib_error_string(NLIB_CONCAT(noerr, l).GetE2()))

#define EXPECT_NOERR(e)                                               \
    switch (0)                                                        \
    case 0:                                                           \
    default:                                                          \
        if (::nlib_ns::testing::NlibErrHelper NLIB_CONCAT(noerr, l) = \
                ::nlib_ns::testing::NlibErrHelper(0, e))              \
            ;                                                         \
        else                                                          \
            EXPECT_STREQ("OK", nlib_error_string(NLIB_CONCAT(noerr, l).GetE2()))

#define ASSERT_NOERR(e)                                               \
    switch (0)                                                        \
    case 0:                                                           \
    default:                                                          \
        if (::nlib_ns::testing::NlibErrHelper NLIB_CONCAT(noerr, l) = \
                ::nlib_ns::testing::NlibErrHelper(0, e))              \
            ;                                                         \
        else                                                          \
            ASSERT_STREQ("OK", nlib_error_string(NLIB_CONCAT(noerr, l).GetE2()))

#endif

#if defined(_MSC_VER) && defined(_PREFAST_)
// for /analyze option
#include "nn/nlib/Cstring.h"
struct MsvcAnalyzeHelper_ {
    template<class T>
    MsvcAnalyzeHelper_& operator<<(T x) {
        return *this;
    }
    operator int() const { return 1; }
};
#undef FAIL
#define FAIL() throw MsvcAnalyzeHelper_()
#undef SUCCEED
#define SUCCEED()

#undef ASSERT_TRUE
#define ASSERT_TRUE(e) \
    if (!(e)) throw MsvcAnalyzeHelper_()
#undef ASSERT_FALSE
#define ASSERT_FALSE(e) \
    if (e) throw MsvcAnalyzeHelper_()
#undef EXPECT_TRUE
#define EXPECT_TRUE(e) \
    if (!(e)) throw MsvcAnalyzeHelper_()
#undef EXPECT_FALSE
#define EXPECT_FALSE(e) \
    if (e) throw MsvcAnalyzeHelper_()

#undef ASSERT_EQ
#define ASSERT_EQ(a, b) \
    if (!((a) == (b))) throw MsvcAnalyzeHelper_()
#undef EXPECT_EQ
#define EXPECT_EQ(a, b) \
    if (!((a) == (b))) throw MsvcAnalyzeHelper_()

#undef ASSERT_NE
#define ASSERT_NE(a, b) \
    if ((a) == (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_NE
#define EXPECT_NE(a, b) \
    if ((a) == (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_LE
#define ASSERT_LE(a, b) \
    if ((a) > (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_LE
#define EXPECT_LE(a, b) \
    if ((a) > (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_LT
#define ASSERT_LT(a, b) \
    if ((a) >= (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_LT
#define EXPECT_LT(a, b) \
    if ((a) >= (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_GE
#define ASSERT_GE(a, b) \
    if ((a) < (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_GE
#define EXPECT_GE(a, b) \
    if ((a) < (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_GT
#define ASSERT_GT(a, b) \
    if ((a) <= (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_GT
#define EXPECT_GT(a, b) \
    if ((a) <= (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_STREQ
#define ASSERT_STREQ(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) != 0) throw MsvcAnalyzeHelper_()
#undef EXPECT_STREQ
#define EXPECT_STREQ(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) != 0) throw MsvcAnalyzeHelper_()

#undef ASSERT_STRNE
#define ASSERT_STRNE(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) == 0) throw MsvcAnalyzeHelper_()
#undef EXPECT_STRNE
#define EXPECT_STRNE(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) == 0) throw MsvcAnalyzeHelper_()

#undef ASSERT_STRCASEEQ
#define ASSERT_STRCASEEQ(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) != 0) throw MsvcAnalyzeHelper_()
#undef EXPECT_STRCASEEQ
#define EXPECT_STRCASEEQ(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) != 0) throw MsvcAnalyzeHelper_()

#undef ASSERT_STRCASENE
#define ASSERT_STRCASENE(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) == 0) throw MsvcAnalyzeHelper_()
#undef EXPECT_STRCASENE
#define EXPECT_STRCASENE(a, b) \
    if (NLIB_NS::StrCmp((a), (b)) == 0) throw MsvcAnalyzeHelper_()

#undef ASSERT_FLOAT_EQ
#define ASSERT_FLOAT_EQ(a, b) \
    if ((a) != (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_FLOAT_EQ
#define EXPECT_FLOAT_EQ(a, b) \
    if ((a) != (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_DOUBLE_EQ
#define ASSERT_DOUBLE_EQ(a, b) \
    if ((a) != (b)) throw MsvcAnalyzeHelper_()
#undef EXPECT_DOUBLE_EQ
#define EXPECT_DOUBLE_EQ(a, b) \
    if ((a) != (b)) throw MsvcAnalyzeHelper_()

#undef ASSERT_NEAR
#define ASSERT_NEAR(a, b, c) \
    if (!((a) == (b))) throw MsvcAnalyzeHelper_()
#undef EXPECT_NEAR
#define EXPECT_NEAR(a, b, c) \
    if (!((a) == (b))) throw MsvcAnalyzeHelper_()

#endif

#endif  // INCLUDE_NN_NLIB_TESTING_TESTING_H_
