﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_
#define INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

class AsyncFileIoService;
template<class R>
class Future;

class NLIB_VIS_PUBLIC AsyncFileIo NLIB_FINAL {
 public:
    enum Mode {
        kAsyncFileIoRead  = NLIB_FD_O_RDONLY,
        kAsyncFileIoWrite = NLIB_FD_O_WRONLY | NLIB_FD_O_CREAT | NLIB_FD_O_TRUNC
    };
    static bool HasNativeAsyncFileIo() NLIB_NOEXCEPT;
    AsyncFileIo() NLIB_NOEXCEPT;
    ~AsyncFileIo() NLIB_NOEXCEPT;
    errno_t Init() NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Open(const char* path, int flags, int mode) NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Open(const char* path, int flags) NLIB_NOEXCEPT {
        return Open(path, flags, 0644);
    }
    NLIB_CHECK_RESULT errno_t FdOpen(nlib_fd fd) NLIB_NOEXCEPT;
    NLIB_ALWAYS_INLINE nlib_fd GetFd() const NLIB_NOEXCEPT { return fd_; }
    errno_t Close() NLIB_NOEXCEPT;
    errno_t Read(Future<size_t>* future, void* buf, size_t nbytes, nlib_offset ofs,
                 AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Read(size_t* read_bytes, void* buf, size_t nbytes, nlib_offset ofs,
                 AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Write(Future<size_t>* future, const void* buf, size_t nbytes, nlib_offset ofs,
                  AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Write(size_t* write_bytes, const void* buf, size_t nbytes, nlib_offset ofs,
                  AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Cancel(AsyncFileIoService* ioservice) NLIB_NOEXCEPT;

 private:
    nlib_fd fd_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AsyncFileIo);
};

struct AsyncFileIoServiceData;
class NLIB_VIS_PUBLIC AsyncFileIoService {
 public:
    AsyncFileIoService() NLIB_NOEXCEPT : data_(NULL) {}
    ~AsyncFileIoService() NLIB_NOEXCEPT;
    errno_t Init() NLIB_NOEXCEPT;

 private:
    AsyncFileIoServiceData* data_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AsyncFileIoService);
    friend class AsyncFileIo;
};

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_
