Param([string]$target,
      [string]$CMake = "cmake.exe")
Push-Location .
$scriptDir = Split-Path -Parent $MyInvocation.MyCommand.Path
Set-Location $scriptDir

if (-not ([System.Environment]::Is64BitProcess)) {
    Write-Warning "oops!: Use 64bit powershell"
}

Set-Item Env:Path ((($Env:Path -split ';') -match '^(?!.*cygwin)') -join ';')

$extra = ""
switch($target) {
    "vc120" {
        $gen = "Visual Studio 12 2013"
    }
    "vc120amd64" {
        $gen = "Visual Studio 12 2013 Win64"
    }
    "vc120dllamd64" {
        $gen = "Visual Studio 12 2013 Win64"
        $extra = "-D BUILD_SHARED_LIBS=ON"
    }
    "vc140" {
        $gen = "Visual Studio 14 2015"
    }
    "vc140amd64" {
        $gen = "Visual Studio 14 2015 Win64"
    }
    "vc140dllamd64" {
        $gen = "Visual Studio 14 2015 Win64"
        $extra = "-D BUILD_SHARED_LIBS=ON"
    }
    "vc150amd64" {
        $gen = "Visual Studio 15 2017 Win64"
    }
    "vc150dllamd64" {
        $gen = "Visual Studio 15 2017 Win64"
        $extra = "-D BUILD_SHARED_LIBS=ON"
    }
    default {
        if (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\15.0") {
            $target = "vc150amd64"
            $gen = "Visual Studio 15 2017 Win64"
        } elseif (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\14.0") {
            $target = "vc140amd64"
            $gen = "Visual Studio 14 2015 Win64"
        } elseif (Test-Path "HKCU:\SOFTWARE\Microsoft\VisualStudio\12.0") {
            $target = "vc120amd64"
            $gen = "Visual Studio 12 2013 Win64"
        } else {
            Write-Error "No Visual Studio found."
            exit 1
        }
    }
}

Push-Location .
try {
    New-Item $target -ItemType Directory
    Set-Location $target
    & $CMake -G $gen $extra -T host=x64 ..\
    if ($LastExitCode -ne 0) { throw }
} catch {
    Set-Location ..
    Remove-Item $target -Recurse -Force
    exit 1
} finally {
    Pop-Location
}

Pop-Location
