﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/
#include "def_serializer.h"
#include "nn/nlib/Cstring.h"

using nlib_ns::OutputStream;
using nlib_ns::InputStream;
using nlib_ns::exi::XmlStreamReader;
using nlib_ns::exi::XmlStreamWriter;
using nlib_ns::exi::ExiChar;
using nlib_ns::SnPrintf;

SimpleSerializer::SimpleSerializer(OutputStream* os) {
    writer_.reset(XmlStreamWriter::Create(os));
}

template <>
void SimpleSerializer::serialize(const StdString& x) {
    writer_->WriteStartElement(N("s"));
    writer_->WriteCharacters(x.c_str());
    writer_->WriteEndElement();
}

template <>
void SimpleSerializer::serialize(const int& x) {
    char b[32];
    SnPrintf(b, "%d", x);
    writer_->WriteStartElement(N("i"));
    writer_->WriteCharacters(NLIB_EXI_STRING(b));
    writer_->WriteEndElement();
}

template <>
void SimpleSerializer::serialize(const float& x) {
    char b[32];
    SnPrintf(b, "%f", x);
    writer_->WriteStartElement(N("f"));
    writer_->WriteCharacters(NLIB_EXI_STRING(b));
    writer_->WriteEndElement();
}

SimpleDeserializer::SimpleDeserializer(InputStream* is) {
    reader_.reset(XmlStreamReader::Create(is));
    is_failed_ = false;
    cache_ = XmlStreamReader::NONE;
}

void SimpleDeserializer::Next(XmlStreamReader::XmlStreamConstants expect) {
    if (is_failed_) return;
    if (cache_ == XmlStreamReader::NONE) cache_ = reader_->Next();
    is_failed_ |= (cache_ != expect);
    cache_ = XmlStreamReader::NONE;
}

bool SimpleDeserializer::Peek(XmlStreamReader::XmlStreamConstants expect) {
    if (is_failed_) return false;
    if (cache_ == XmlStreamReader::NONE) cache_ = reader_->Next();
    return (cache_ == expect);
}

template <>
void SimpleDeserializer::serialize(StdString& x) {  // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);     // s
    this->Next(XmlStreamReader::CHARACTERS);
    x = reader_->GetText();
    this->Next(XmlStreamReader::END_ELEMENT);  // s
}

template <>
void SimpleDeserializer::serialize(int& x) {     // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);  // i
    this->Next(XmlStreamReader::CHARACTERS);
    errno_t e = nlib_strto_int32(&x, M(reader_->GetText()), NULL, 10);
    if (e != 0) x = 0;
    this->Next(XmlStreamReader::END_ELEMENT);  // i
}

template <>
void SimpleDeserializer::serialize(float& x) {   // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);  // f
    this->Next(XmlStreamReader::CHARACTERS);
    errno_t e = nlib_strto_float(&x, M(reader_->GetText()), NULL);
    if (e != 0) x = 0.f;
    this->Next(XmlStreamReader::END_ELEMENT);  // f
}
