﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "nn/nlib/threading/Barrier.h"
using ::nlib_ns::threading::Barrier;

const int kNumThread = 8;
nlib_thread g_th[kNumThread];
Barrier g_barrier;

void ThreadFunc(void* p) {
    int n = static_cast<int>(reinterpret_cast<intptr_t>(p));
    nlib_printf("Thread %d: Step 1\n", n);
    g_barrier.Wait();  // blocks until kNumThread threads comes here

    nlib_printf("Thread %d: Step 2\n", n);
    g_barrier.Wait();  // blocks until kNumThread threads comes here

    nlib_printf("Thread %d: Step 3\n", n);
    g_barrier.Wait();  // blocks until kNumThread threads comes here
}

bool BarrierDemo() {
    nlib_printf("%d threads run, and sync by a Barrier object.\n", kNumThread);
    nlib_printf("The Barrier object can sync mutiple times.\n\n");
    g_barrier.Init(kNumThread);

    intptr_t i;
    errno_t e;
    bool fatal = false;
    for (i = 0; i < kNumThread; ++i) {
        e = nlib_thread_create(&g_th[i], NULL, ThreadFunc, (void*)i);
        if (e != 0) {
            g_th[i] = NLIB_THREAD_INVALID;
            fatal = true;
        }
    }
    if (fatal) return false;
    for (i = 0; i < kNumThread; ++i) {
        (void)nlib_thread_join(g_th[i]);
    }
    return true;
}

static bool SampleMain(int, char**) { return BarrierDemo(); }

NLIB_MAINFUNC
