﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "nn/nlib/threading/Semaphore.h"
using ::nlib_ns::threading::Semaphore;

const int kNumThread = 10;
nlib_thread g_th[kNumThread];
Semaphore g_semaphore;

void SemaphoreDemoFunc(void* p) {
    int thread_num = static_cast<int>(reinterpret_cast<intptr_t>(p));
    nlib_printf(" Thread %d, Trying wait semaphore\n", thread_num);
    g_semaphore.Acquire();

    nlib_printf(" Thread %d, Success get semaphore\n", thread_num);
    for (int i = 0; i < 2; ++i) {
        nlib_printf("  Thread %d, doing jobs using limited resource\n", thread_num);
        nlib_sleep(10000);  // 1 msec
    }
    nlib_printf(" Thread %d, Release semaphore\n", thread_num);

    g_semaphore.Release(NULL);
}

bool SemaphoreDemo() {
    errno_t e;
    nlib_printf("-- Semaphore Demo --\n");

    // initializes Semaphore
    e = g_semaphore.Init(0);
    if (e != 0) return false;

    intptr_t i;
    for (i = 0; i < kNumThread; ++i) {
        e = nlib_thread_create(&g_th[i], NULL, SemaphoreDemoFunc, (void*)i);
        if (e != 0) return false;
    }

    // kNumThread threads share 2 resources
    nlib_printf("Main Thread: Release semaphores\n");
    e = g_semaphore.Release(2, NULL);
    NLIB_ASSERT_NOERR(e);
    for (i = 0; i < kNumThread; ++i) {
        (void)nlib_thread_join(g_th[i]);
    }
    return true;
}

static bool SampleMain(int, char**) { return SemaphoreDemo(); }

NLIB_MAINFUNC
