<#
    Copyright (C)Nintendo All rights reserved.

    These coded instructions, statements, and computer programs contain proprietary
    information of Nintendo and/or its licensed developers and are protected by
    national and international copyright laws. They may not be disclosed to third
    parties or copied or duplicated in any form, in whole or in part, without the
    prior written consent of Nintendo.

    The content herein is highly confidential and should be handled accordingly.
#>

<#
.SYNOPSIS
    Updates Visual Studios 2017+
.DESCRIPTION
    Executes 'vs_installer.exe update --quiet --installPath ....'
.PARAMETER Inspection
    The output of Inspection.ps1
#>

#Requires -Version 5
Param($Inspection,
      [switch]$ForceCheck,
      [switch]$Quiet)

Set-Variable script_dir (Resolve-Path (Split-Path -Parent $MyInvocation.MyCommand.Path)).Path -Option ReadOnly
if (!$Inspection) {
    $Inspection = & $script_dir\Inspection.ps1 -Msvc
}
$prop = Get-ItemProperty $script_dir\Update-VisualStudio.ps1
$wtime = $prop.LastWriteTime
$atime = $prop.LastAccessTime
if (!$ForceCheck -and $Inspection.Msvc -and ($atime -gt $wtime) -and ($atime.AddMinutes(360) -gt (Get-Date))) {
    Write-Output $Inspection.Msvc
    return
}

$vs_installer = "${Env:ProgramFiles(x86)}\Microsoft Visual Studio\Installer\vs_installer.exe"
if (Test-Path $vs_installer) {
    Get-ChildItem "${Env:ProgramFiles(x86)}\Microsoft Visual Studio" -Directory | Where-Object {
        ($_.Name -match "[0-9]+") -or ($_.Name -eq "Preview")
    } | ForEach-Object {
        foreach ($edition in (Get-ChildItem $_.FullName -Directory)) {
            $install_path = ("`"{0}`"" -f $edition.FullName)
            Write-Host "executing $vs_installer, please wait..."
            $proc = Start-Process $vs_installer -PassThru -Wait -ArgumentList @("update", "--installPath", $install_path, "--quiet", "--noUpdateInstaller")
            Write-Host "done"
            if ($proc.ExitCode -ne 0) {
                if (!$Quiet) {
                    Write-Host "vs_installer.exe may require update. Run in non-quiet mode"
                    Start-Process $vs_installer -Wait -ArgumentList @("update", "--installPath", $install_path)
                } else {
                    throw "fail in quiet mode. Please upate vs_installer.exe itself."
                }
            }
        }
    }
}

$Inspection = & $script_dir\Inspection.ps1 -Msvc
Set-ItemProperty -Path $script_dir\Update-VisualStudio.ps1 -Name LastAccessTime -Value $(Get-Date)
Write-Output $Inspection.Msvc
