#!/bin/bash

if ! type uname > /dev/null 2>&1; then
  echo Cannot find uname.
  exit 1
fi

if ! [[ $(uname) =~ CYGWIN.* ]]; then
  echo Please call this script from cygwin.
  exit 1
fi

if ! [[ $(uname -m) =~ .*64.* ]]; then
  echo use 64bit cygwin
  exit 1
fi

if ! type git > /dev/null 2>&1; then
  echo Please install git manually.
  exit 1
fi

if ! type wget > /dev/null 2>&1; then
  echo Please install wget manually.
  exit 1
fi

if ! type tar > /dev/null 2>&1; then
  echo Please install tar manually.
  exit 1
fi

if ! type gawk > /dev/null 2>&1; then
  echo Please install gawk manually.
  exit 1
fi

if ! type gpg > /dev/null 2>&1; then
  echo Please install gpg manually.
  exit 1
fi

if ! type gpgv > /dev/null 2>&1; then
  echo Please install gpgv manually.
  exit 1
fi

if [[ ! -e ~/.wgetrc ]]; then
  set +u
  echo "" > ~/.wgetrc
  if [[ -n "${http_proxy}" ]]; then
    echo "http_proxy=${http_proxy}" >> ~/.wgetrc
  elif [[ -n "${HTTP_PROXY}" ]]; then
    echo "http_proxy=${HTTP_PROXY}" >> ~/.wgetrc
  fi
  if [[ -n "${https_proxy}" ]]; then
    echo "https_proxy=${https_proxy}" >> ~/.wgetrc
  elif [[ -n "${HTTPS_PROXY}" ]]; then
    echo "https_proxy=${HTTPS_PROXY}" >> ~/.wgetrc
  fi
  set -u
fi

pushd .
cd $(dirname ${0})

# https://github.com/kou1okada/apt-cyg
# wget,ca-certificates,gnupg required
if [[ ! -e /usr/local/bin/apt-cyg ]]; then
  echo installing apt-cyg
  git clone https://github.com/kou1okada/apt-cyg.git
  chmod +x apt-cyg/apt-cyg && cp ./apt-cyg/apt-cyg /usr/local/bin/
fi

if [[ ! -e /usr/local/bin/apt-cyg ]]; then
  echo Please install apt-cyg manually.
  exit 1
fi

yes | apt-cyg upgrade-self
yes | apt-cyg update-setup
yes | apt-cyg update

# apt-cyg --mirror http://ftp.iij.ad.jp/pub/cygwin/
yes | apt-cyg update
yes | apt-cyg install tar gzip gawk zip unzip binutils
yes | apt-cyg install gcc-g++ make cmake ninja
yes | apt-cyg install zlib zlib-devel libcurl-devel libboost-devel
yes | apt-cyg install ccache

if [[ ${1} ]]; then
  yes | apt-cyg install procps-ng
  yes | apt-cyg install gdb gperf
  yes | apt-cyg install cccc libxml2
  # yes | apt-cyg install doxygen graphviz
  # yes | apt-cyg install openssh flex jq rsync

  yes | apt-cyg install python python3 python-setuptools
  if [[ ! -e /usr/local/bin/gcovr ]]; then
    # rebaseall may be needed
    easy_install-2.7 pip
    pip install gcovr
  fi

  if ! type sloccount > /dev/null 2>&1; then
    pushd
    cd
    wget -O - http://www.dwheeler.com/sloccount/sloccount-2.26.tar.gz | tar zxvf -
    cd ./sloccount-2.26
    make EXE_SUFFIX=.exe
    make install
    popd
  fi
fi

popd

# echo "You may have to do '/usr/bin/rebaseall' and '/usr/bin/peflagsall' on ash"
