#!/bin/bash
#
# Sets up and updates the development environment for nlib on WSL(Ubuntu)

lsb_release -i | grep Ubuntu > /dev/null
if [ ${?} -ne 0 ]; then
  echo Please call this script from Ubuntu
  exit 1
fi

pushd .
cd $(dirname ${0})

sudo -E apt update
sudo -E apt -y install aptitude
sudo -E apt upgrade
sudo -E apt -y install g++ clang
sudo -E apt -y install make git ccache
sudo -E apt -y install zlib1g-dev libcurl4-openssl-dev
sudo -E apt -y install ninja-build # you may have to install the newest ninja instead from https://github.com/ninja-build/ninja/releases

if [[ ${1} ]]; then
  sudo -E apt -y install python-dev clang-tidy gcovr sloccount
  sudo -E apt -y install libxml2 libgtest-dev
  sudo -E apt -y install libboost-system-dev libboost-thread-dev libboost-date-time-dev libboost-regex-dev
fi

if [[ ! -e /usr/local/bin/cmake ]]; then
  pushd .
  cd
  # needs 'https_proxy=http://${xxx.xxx.xxx.xxx}:${ss}/' in ~/.wgetrc if proxy needed
  wget -nc -O - https://cmake.org/files/v3.10/cmake-3.10.0-Linux-x86_64.tar.gz | tar zxvf -
  sudo -E ln -s $HOME/cmake-3.10.0-Linux-x86_64/bin/cmake /usr/local/bin/cmake
  sudo -E ln -s $HOME/cmake-3.10.0-Linux-x86_64/bin/ccmake /usr/local/bin/ccmake
  sudo -E ln -s $HOME/cmake-3.10.0-Linux-x86_64/bin/ctest /usr/local/bin/ctest
  sudo -E ln -s $HOME/cmake-3.10.0-Linux-x86_64/bin/cmake-gui /usr/local/bin/cmake-gui
  sudo -E ln -s $HOME/cmake-3.10.0-Linux-x86_64/bin/cpack /usr/local/bin/cpack
  popd
fi

popd
